Package: locuszoomr
Title: Gene Locus Plot with Gene Annotations
Version: 0.3.8
Authors@R: 
    c(person(given = "Myles",family = "Lewis",
    role = c("aut", "cre"),
    email = "myles.lewis@qmul.ac.uk",
    comment = c(ORCID = "0000-0001-9365-5345")))
BugReports: https://github.com/myles-lewis/locuszoomr/issues
URL: https://github.com/myles-lewis/locuszoomr
Description: Publication-ready regional gene locus plots similar to those produced by the web interface 'LocusZoom' <https://my.locuszoom.org>, but running locally in R. Genetic or genomic data with gene annotation tracks are plotted via R base graphics, 'ggplot2' or 'plotly', allowing flexibility and easy customisation including laying out multiple locus plots on the same page. It uses the 'LDlink' API <https://ldlink.nih.gov/?tab=apiaccess> to query linkage disequilibrium data from the 1000 Genomes Project and can overlay this on plots <doi:10.1093/bioadv/vbaf006>.
Language: en-gb
License: GPL (>= 3)
Encoding: UTF-8
Depends: R (>= 3.5)
biocViews:
Imports: AnnotationFilter, BiocGenerics, cowplot, dplyr, ensembldb,
        GenomeInfoDb, GenomicRanges, gggrid, ggplot2, ggrepel,
        graphics, grDevices, grid, IRanges, LDlinkR, memoise, plotly,
        rlang, rtracklayer, zoo
RoxygenNote: 7.3.2
Suggests: AnnotationHub, EnsDb.Hsapiens.v75, knitr, rmarkdown
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2025-03-03 14:04:10 UTC; myles
Author: Myles Lewis [aut, cre] (<https://orcid.org/0000-0001-9365-5345>)
Maintainer: Myles Lewis <myles.lewis@qmul.ac.uk>
Repository: CRAN
Date/Publication: 2025-03-03 14:30:02 UTC
Built: R 4.6.0; ; 2026-04-30 13:13:20 UTC; unix

Package: locuszoomr
Title: Gene Locus Plot with Gene Annotations
Version: 1.0.0
Authors@R: 
    c(person(given = "Myles",family = "Lewis",
    role = c("aut", "cre"),
    email = "myles.lewis@qmul.ac.uk",
    comment = c(ORCID = "0000-0001-9365-5345")),
    person(given = "Tom",family = "Willis",
    role = c("ctb"))
    )
BugReports: https://github.com/myles-lewis/locuszoomr/issues
URL: https://github.com/myles-lewis/locuszoomr
Description: Publication-ready regional gene locus plots similar to those produced by the web interface 'LocusZoom' <https://my.locuszoom.org>, but running locally in R. Genetic or genomic data with gene annotation tracks are plotted via R base graphics, 'ggplot2' or 'plotly', allowing flexibility and easy customisation including laying out multiple locus plots on the same page. It uses the 'LDlink' API <https://ldlink.nih.gov/?tab=apiaccess> to query linkage disequilibrium data from the 1000 Genomes Project and can overlay this on plots <doi:10.1093/bioadv/vbaf006>.
Language: en-gb
License: GPL (>= 3)
Encoding: UTF-8
Depends: R (>= 3.5)
biocViews:
Imports: AnnotationFilter, BiocGenerics, cowplot, dplyr, DT, ensembldb,
        GenomeInfoDb, GenomicRanges, gggrid, ggplot2, ggrepel,
        graphics, grDevices, grid, gtools, htmltools, IRanges, LDlinkR,
        memoise, plotly, Rcpp, rlang, rtracklayer, shiny,
        shinycssloaders, shinyFeedback, shinyWidgets, zoo
LinkingTo: Rcpp
Suggests: AnnotationDbi, AnnotationHub, EnsDb.Hsapiens.v75, knitr,
        org.Hs.eg.db, rmarkdown
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-09-02 23:18:28 UTC; hhx107
Author: Myles Lewis [aut, cre] (ORCID: <https://orcid.org/0000-0001-9365-5345>),
  Tom Willis [ctb]
Maintainer: Myles Lewis <myles.lewis@qmul.ac.uk>
Repository: CRAN
Date/Publication: 2026-09-03 06:50:03 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-09-03 08:06:39 UTC; unix
Archs: locuszoomr.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(summary,locus)
export(eqtl_plot)
export(genetrack_ly)
export(genetracks)
export(genetracks_grob)
export(gg_addgenes)
export(gg_genetracks)
export(gg_scatter)
export(line_plot)
export(link_LD)
export(link_eqtl)
export(link_recomb)
export(locus)
export(locus_ggplot)
export(locus_plot)
export(locus_plotly)
export(multi_layout)
export(overlay_plot)
export(quick_peak)
export(scatter_plot)
export(scatter_plotly)
export(set_layers)
importFrom(AnnotationFilter,AnnotationFilterList)
importFrom(AnnotationFilter,ExonEndFilter)
importFrom(AnnotationFilter,ExonStartFilter)
importFrom(AnnotationFilter,GeneIdFilter)
importFrom(AnnotationFilter,GeneNameFilter)
importFrom(AnnotationFilter,SeqNameFilter)
importFrom(AnnotationFilter,TxEndFilter)
importFrom(AnnotationFilter,TxStartFilter)
importFrom(BiocGenerics,end)
importFrom(BiocGenerics,start)
importFrom(GenomeInfoDb,"genome<-")
importFrom(GenomeInfoDb,genome)
importFrom(GenomeInfoDb,seqlengths)
importFrom(GenomeInfoDb,seqnames)
importFrom(GenomicRanges,GRanges)
importFrom(IRanges,IRanges)
importFrom(LDlinkR,LDexpress)
importFrom(LDlinkR,LDmatrix)
importFrom(LDlinkR,LDproxy)
importFrom(cowplot,plot_grid)
importFrom(dplyr,bind_rows)
importFrom(ensembldb,ensemblVersion)
importFrom(ensembldb,exons)
importFrom(ensembldb,genes)
importFrom(ensembldb,organism)
importFrom(gggrid,grid_panel)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,layer_scales)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_shape_manual)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,sec_axis)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,unit)
importFrom(ggplot2,xlab)
importFrom(ggplot2,xlim)
importFrom(ggplot2,ylim)
importFrom(ggrepel,geom_text_repel)
importFrom(grDevices,adjustcolor)
importFrom(grDevices,col2rgb)
importFrom(grDevices,hcl.colors)
importFrom(grDevices,rainbow)
importFrom(grDevices,rgb)
importFrom(graphics,axTicks)
importFrom(graphics,axis)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot.new)
importFrom(graphics,points)
importFrom(graphics,rect)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(grid,gList)
importFrom(grid,gTree)
importFrom(grid,gpar)
importFrom(grid,polylineGrob)
importFrom(grid,rectGrob)
importFrom(grid,textGrob)
importFrom(grid,viewport)
importFrom(grid,xaxisGrob)
importFrom(memoise,drop_cache)
importFrom(memoise,memoise)
importFrom(plotly,"%>%")
importFrom(plotly,add_segments)
importFrom(plotly,add_text)
importFrom(plotly,add_trace)
importFrom(plotly,plot_ly)
importFrom(plotly,plotly_build)
importFrom(plotly,plotly_empty)
importFrom(rlang,.data)
importFrom(rtracklayer,browserSession)
importFrom(rtracklayer,getTable)
importFrom(rtracklayer,ucscTableQuery)
importFrom(stats,relevel)
importFrom(zoo,na.approx)

# locuszoomr

<!-- badges: start -->
[![CRAN status](https://www.r-pkg.org/badges/version/locuszoomr)](https://CRAN.R-project.org/package=locuszoomr)
[![Downloads](https://cranlogs.r-pkg.org/badges/locuszoomr)](https://CRAN.R-project.org/package=locuszoomr)
[![Downloads](https://cranlogs.r-pkg.org/badges/grand-total/locuszoomr)](https://CRAN.R-project.org/package=locuszoomr)
<!-- badges: end -->

This is a pure R implementation of locuszoom for plotting genetic data at
genomic loci accompanied by gene annotations. Plots can be produced in base
graphics, ggplot2 or plotly. Plots can be stacked or laid out with multiple
plots per page, or the gene track can be plotted separately and added to your
own plots.

The LDlink API can be queried to obtain linkage disequilibrium data from 1000
Genomes. Recombination rate can also be shown by querying UCSC genome browser.

See the detailed vignette for code examples.

# Installation

Bioconductor package `ensembldb` and an Ensembl database installed either as a
package or accessed through Bioconductor package `AnnotationHub` are required
before installation.

```
if (!requireNamespace("BiocManager", quietly = TRUE))
  install.packages("BiocManager")
BiocManager::install("ensembldb")
BiocManager::install("EnsDb.Hsapiens.v75")
```

Install from CRAN
```
install.packages("locuszoomr")
```

Install from Github
```
devtools::install_github("myles-lewis/locuszoomr")
```

`locuszoomr` can leverage the `LDlinkR` package to query the 1000 Genomes
Project for linkage disequilibrium (LD) across SNPs. In order to make use of
this API function you will need a personal access token, available from the 
[LDlink website](https://ldlink.nih.gov/?tab=apiaccess).

We recommend that users who want to add recombination rate lines to multiple
plots download the recombination rate track from UCSC and use it as described in
the 'Add recombination rate' section in the vignette.

# Example locus plot

```
# Locus plot using SLE GWAS data from Bentham et al 2015
# Using subset of data embedded in the package
library(locuszoomr)
data(SLE_gwas_sub)

library(EnsDb.Hsapiens.v75)
loc <- locus(gene = 'UBE2L3', SLE_gwas_sub, flank = 1e5,
             ens_db = "EnsDb.Hsapiens.v75")
summary(loc)
locus_plot(loc)

# Or FTP download the full summary statistics from
# https://www.ebi.ac.uk/gwas/studies/GCST003156
library(data.table)
SLE_gwas <- fread('../bentham_2015_26502338_sle_efo0002690_1_gwas.sumstats.tsv')

loc <- locus(gene = 'UBE2L3', SLE_gwas, flank = 1e5,
             ens_db = "EnsDb.Hsapiens.v75")
locus_plot(loc)
```

# Example layered plot shown in the paper

```
library(locuszoomr)
library(EnsDb.Hsapiens.v75)

data(SLE_gwas_sub)
loc <- locus(data = SLE_gwas_sub, gene = 'IRF5', flank = c(1e5, 2e5),
             ens_db = "EnsDb.Hsapiens.v75")

# add recombination rate
loc <- link_recomb(loc, genome = "hg19")

# add LD and eQTL data
# users must obtain an API token from https://ldlink.nih.gov/?tab=apiaccess
loc <- link_LD(loc, token = "your_API_token")
loc <- link_eqtl(loc, token = "your_API_token")

# set up layered plot with 2 plots & a gene track
pdf("locuszoomr_demo.pdf", width = 4.5, height = 7)
oldpar <- set_layers(2)
scatter_plot(loc, xticks = FALSE, labels = c("index", "rs113708239"),
             label_x = c(-4, 4))
eqtl_plot(loc, xlab = "")
genetracks(loc, highlight = "IRF5")
par(oldpar)
dev.off()
```

# Citation

If you use this package please cite as:

Lewis MJ, Wang S. (2025) locuszoomr: an R package for visualising
publication-ready regional gene locus plots. *Bioinformatics Advances* 2025;
vbaf006, [doi:10.1093/bioadv/vbaf006](https://doi.org/10.1093/bioadv/vbaf006)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE
)
library(locuszoomr)

## ----eval = FALSE-------------------------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE))
#   install.packages("BiocManager")
# BiocManager::install("ensembldb")
# BiocManager::install("EnsDb.Hsapiens.v75")

## ----eval = FALSE-------------------------------------------------------------
# install.packages("locuszoomr")

## ----eval = FALSE-------------------------------------------------------------
# devtools::install_github("myles-lewis/locuszoomr")

## -----------------------------------------------------------------------------
library(locuszoomr)
data(SLE_gwas_sub)  ## limited subset of data from SLE GWAS
head(SLE_gwas_sub)

## ----eval=FALSE---------------------------------------------------------------
# library(EnsDb.Hsapiens.v75)
# loc <- locus(data = SLE_gwas_sub, gene = 'UBE2L3', flank = 1e5,
#              ens_db = "EnsDb.Hsapiens.v75")
# summary(loc)
# locus_plot(loc)

## ----echo=FALSE, message=FALSE, fig.dim=c(8, 7)-------------------------------
if (require(EnsDb.Hsapiens.v75)) {
loc <- locus(data = SLE_gwas_sub, gene = 'UBE2L3', flank = 1e5,
             ens_db = "EnsDb.Hsapiens.v75")
summary(loc)
locus_plot(loc)
}

## ----eval=FALSE---------------------------------------------------------------
# library(AnnotationHub)
# ah <- AnnotationHub()
# query(ah, c("EnsDb", "Homo sapiens"))

## ----eval=FALSE---------------------------------------------------------------
# ## AnnotationHub with 25 records
# ## # snapshotDate(): 2023-04-25
# ## # $dataprovider: Ensembl
# ## # $species: Homo sapiens
# ## # $rdataclass: EnsDb
# ## # additional mcols(): taxonomyid, genome, description, coordinate_1_based, maintainer,
# ## #   rdatadateadded, preparerclass, tags, rdatapath, sourceurl, sourcetype
# ## # retrieve records with, e.g., 'object[["AH53211"]]'
# ##
# ##              title
# ##   AH53211  | Ensembl 87 EnsDb for Homo Sapiens
# ##   ...        ...
# ##   AH100643 | Ensembl 106 EnsDb for Homo sapiens
# ##   AH104864 | Ensembl 107 EnsDb for Homo sapiens
# ##   AH109336 | Ensembl 108 EnsDb for Homo sapiens
# ##   AH109606 | Ensembl 109 EnsDb for Homo sapiens
# ##   AH113665 | Ensembl 110 EnsDb for Homo sapiens

## ----eval=FALSE---------------------------------------------------------------
# ensDb_v106 <- ah[["AH100643"]]
# 
# # built-in mini dataset
# data("SLE_gwas_sub")
# loc <- locus(data = SLE_gwas_sub, gene = 'UBE2L3', fix_window = 1e6,
#              ens_db = ensDb_v106)
# locus_plot(loc)

## ----eval = FALSE-------------------------------------------------------------
# # Locus plot using SLE GWAS data from Bentham et al 2015
# # FTP download full summary statistics from
# # https://www.ebi.ac.uk/gwas/studies/GCST003156
# library(data.table)
# SLE_gwas <- fread('../bentham_2015_26502338_sle_efo0002690_1_gwas.sumstats.tsv')
# loc <- locus(SLE_gwas, gene = 'UBE2L3', flank = 1e5,
#              ens_db = "EnsDb.Hsapiens.v75")
# loc <- link_LD(loc, token = "your_token")
# locus_plot(loc)

## ----eval=FALSE---------------------------------------------------------------
# loc <- locus(SLE_gwas_sub, gene = 'UBE2L3', flank = 1e5, LD = "r2",
#              ens_db = "EnsDb.Hsapiens.v75")
# locus_plot(loc, labels = c("index", "rs140492"),
#                 label_x = c(4, -5))

## ----echo=FALSE, message=FALSE, fig.dim=c(8, 7)-------------------------------
if (require(EnsDb.Hsapiens.v75)) {
loc <- locus(SLE_gwas_sub, gene = 'UBE2L3', flank = 1e5, LD = "r2",
             ens_db = "EnsDb.Hsapiens.v75")
locus_plot(loc, labels = c("index", "rs140492"), label_x = c(4, -5))
}

## ----eval=FALSE---------------------------------------------------------------
# loc3 <- locus(SLE_gwas_sub, gene = 'STAT4', flank = 1e5, LD = "r2",
#               ens_db = "EnsDb.Hsapiens.v75")
# loc3 <- link_recomb(loc3)
# locus_plot(loc3)

## ----out.width='80%', echo=FALSE----------------------------------------------
knitr::include_graphics("recomb.png")

## ----eval=FALSE---------------------------------------------------------------
# library(rtracklayer)
# recomb.hg19 <- import.bw("/../hapMapRelease24CombinedRecombMap.bw")
# loc3 <- link_recomb(loc3, recomb = recomb.hg19)
# locus_plot(loc3)

## ----eval=FALSE---------------------------------------------------------------
# # add column 'typed' as a factor which is 1 for typed, 0 for imputed
# loc$data$typed <- factor(rbinom(n = nrow(loc$data), 1, 0.3))
# # convert column to shapes by adding a column called 'pch'
# loc$data$pch <- c(21, 24)[loc$data$typed]
# locus_plot(loc)
# # pch 21 = circles = imputed
# # pch 24 = triangles = typed

## ----eval=FALSE---------------------------------------------------------------
# # Filter by gene biotype
# locus_plot(loc, filter_gene_biotype = "protein_coding")
# 
# # Custom selection of genes using gene names
# locus_plot(loc, filter_gene_name = c('UBE2L3', 'RIMBP3C', 'YDJC', 'PPIL2',
#                                      'PI4KAP2', 'MIR301B'))

## ----eval=FALSE---------------------------------------------------------------
# genetracks(loc, highlight = "UBE2L3")

## ----echo=FALSE, fig.dim=c(7, 3.5)--------------------------------------------
if (require(EnsDb.Hsapiens.v75)) {
genetracks(loc, highlight = "UBE2L3")
}

## ----eval=FALSE---------------------------------------------------------------
# # Limit the number of tracks
# # Filter by gene biotype
# # Customise colours
# genetracks(loc, maxrows = 3, filter_gene_biotype = 'protein_coding',
#            gene_col = 'grey', exon_col = 'orange', exon_border = 'darkgrey')

## ----echo=FALSE, fig.dim=c(7, 2.5)--------------------------------------------
if (require(EnsDb.Hsapiens.v75)) {
genetracks(loc, maxrows = 3, filter_gene_biotype = 'protein_coding',
           gene_col = 'grey', exon_col = 'orange', exon_border = 'darkgrey')
}

## ----eval = FALSE-------------------------------------------------------------
# loc00 <- locus(gene = 'UBE2L3', flank = 1e5, ens_db = "EnsDb.Hsapiens.v75")
# 
# genetracks(loc00)  # base graphics
# gg_genetracks(loc00)  # ggplot2
# genetrack_ly(loc00)  # plotly

## ----eval=FALSE---------------------------------------------------------------
# # obtain GTEx eQTL data from LDlinkR
# # needs personal access token for LDlink
# loc2 <- locus(SLE_gwas_sub, gene = 'IRF5', flank = c(7e4, 2e5), LD = "r2",
#               ens_db = "EnsDb.Hsapiens.v75")
# loc2 <- link_eqtl(loc2, token = "..")
# head(loc2$LDexp)  # show eQTL data

## ----eval=FALSE---------------------------------------------------------------
# table(loc2$LDexp$Gene_Symbol)
# ##      AHCYL2           IRF5            KCP        METTL2B           NRF1   RP11-212P7.2
# ##           5           1618              1              2              9              3
# ## ...
# 
# table(loc2$LDexp$Tissue)
# ##             Adipose - Subcutaneous          Adipose - Visceral (Omentum)
# ##                                 78                                    69
# ##                      Adrenal Gland                        Artery - Aorta
# ##                                 38                                   109
# ## ...

## ----eval=FALSE---------------------------------------------------------------
# # just plot eQTL results
# eqtl_plot(loc2, tissue = "Whole Blood", eqtl_gene = "IRF5")
# 
# # overlay eqtl plot
# overlay_plot(loc2, eqtl_gene = "IRF5")

## ----out.width='60%', fig.align="center", echo=FALSE--------------------------
knitr::include_graphics("eqtl_overlay.png")

## ----eval=FALSE---------------------------------------------------------------
# locb <- locus(SLE_gwas_sub, gene = 'UBE2L3', flank = 1e5, yvar = "beta",
#               ens_db = "EnsDb.Hsapiens.v75")
# locus_plot(locb)

## ----echo=FALSE, message=FALSE, fig.dim=c(8, 7)-------------------------------
if (require(EnsDb.Hsapiens.v75)) {
locb <- locus(SLE_gwas_sub, gene = 'UBE2L3', flank = 1e5, yvar = "beta",
              ens_db = "EnsDb.Hsapiens.v75")
locus_plot(locb)
}

## ----eval=FALSE---------------------------------------------------------------
# genes <- c("STAT4", "IRF5", "UBE2L3")
# 
# # generate list of 'locus' class objects, one for each gene
# loclist <- lapply(genes, locus,
#                   data = SLE_gwas_sub,
#                   ens_db = "EnsDb.Hsapiens.v75",
#                   LD = "r2")
# 
# ## produce 3 locus plots, one on each page
# pdf("myplot.pdf")
# multi_layout(loclist)
# dev.off()
# 
# ## place 3 locus plots in a row on a single page
# pdf("myplot.pdf")
# multi_layout(loclist, ncol = 3, labels = "index")
# dev.off()
# 
# ## full control
# loc2 <- locus(SLE_gwas_sub, gene = 'IRF5', flank = c(7e4, 2e5), LD = "r2",
#               ens_db = "EnsDb.Hsapiens.v75")
# loc3 <- locus(SLE_gwas_sub, gene = 'STAT4', flank = 1e5, LD = "r2",
#               ens_db = "EnsDb.Hsapiens.v75")
# 
# pdf("myplot.pdf", width = 9, height = 6)
# multi_layout(ncol = 3,
#              plots = {
#                locus_plot(loc, use_layout = FALSE, legend_pos = 'topleft')
#                locus_plot(loc2, use_layout = FALSE, legend_pos = NULL)
#                locus_plot(loc3, use_layout = FALSE, legend_pos = NULL,
#                           labels = "index")
#              })
# dev.off()

## ----echo=FALSE, message=FALSE, fig.dim = c(9,5)------------------------------
if (require(EnsDb.Hsapiens.v75)) {
loc2 <- locus(SLE_gwas_sub, gene = 'IRF5', flank = c(7e4, 2e5), LD = "r2",
              ens_db = "EnsDb.Hsapiens.v75")
loc3 <- locus(SLE_gwas_sub, gene = 'STAT4', flank = 1e5, LD = "r2",
              ens_db = "EnsDb.Hsapiens.v75")

multi_layout(ncol = 3,
             plots = {
               locus_plot(loc, use_layout = FALSE, legend_pos = 'topleft')
               locus_plot(loc2, use_layout = FALSE, legend_pos = NULL)
               locus_plot(loc3, use_layout = FALSE, legend_pos = NULL,
                          labels = "index")
             })
}

## ----eval=FALSE---------------------------------------------------------------
# pdf("myplot2.pdf", width = 6, height = 8)
# # set up layered plot with 2 plots & a gene track; store old par() settings
# oldpar <- set_layers(2)
# scatter_plot(loc, xticks = FALSE)
# line_plot(loc, col = "orange", xticks = FALSE)
# genetracks(loc)
# par(oldpar)  # revert par() settings
# dev.off()

## ----echo=FALSE, fig.dim = c(6,8)---------------------------------------------
if (require(EnsDb.Hsapiens.v75)) {
oldpar <- set_layers(2)
scatter_plot(loc, xticks = FALSE)
line_plot(loc, col = "orange", xticks = FALSE)
genetracks(loc)
par(oldpar)
}

## ----eval=FALSE---------------------------------------------------------------
# dat <- SLE_gwas_sub
# dat$p2 <- -log10(dat$p * 0.1)
# locp <- locus(dat, gene = 'UBE2L3', flank = 1e5, ens_db = "EnsDb.Hsapiens.v75")
# locp2 <- locus(dat, gene = 'UBE2L3', flank = 1e5, yvar = "p2",
#                ens_db = "EnsDb.Hsapiens.v75")
# 
# oldpar <- set_layers(1)
# scatter_plot(locp, xticks = FALSE, pcutoff = NULL, ylim = c(0, 16))
# scatter_plot(locp2, xticks = FALSE, pcutoff = NULL, scheme = "orange",
#              pch = 22, add = TRUE)
# genetracks(loc)
# par(oldpar)

## ----echo=FALSE, message=FALSE, fig.dim = c(7,7)------------------------------
if (require(EnsDb.Hsapiens.v75)) {
dat <- SLE_gwas_sub
dat$p2 <- -log10(dat$p * 0.1)
locp <- locus(dat, gene = 'UBE2L3', flank = 1e5, ens_db = "EnsDb.Hsapiens.v75")
locp2 <- locus(dat, gene = 'UBE2L3', flank = 1e5, yvar = "p2",
               ens_db = "EnsDb.Hsapiens.v75")

oldpar <- set_layers(1)
scatter_plot(locp, xticks = FALSE, pcutoff = NULL, ylim = c(0, 16))
scatter_plot(locp2, xticks = FALSE, pcutoff = NULL, scheme = "orange",
             pch = 22, add = TRUE)
genetracks(loc)
par(oldpar)
}

## ----eval=FALSE---------------------------------------------------------------
# # add vertical lines for gene of interest under the main plot
# pf <- quote({
#   v <- locp$TX[locp$TX$gene_name == "UBE2L3", c("start", "end")]
#   abline(v = v, col = "orange")
# })
# 
# pl <- quote({
#   # add custom text label for index SNP
#   lx <- locp$data$pos[locp$data$rsid == locp$index_snp]
#   ly <- locp$data$logP[locp$data$rsid == locp$index_snp]
#   text(lx, ly, locp$index_snp, pos = 4, cex = 0.8)
#   # add extra points
#   px <- rep(22.05e6, 3)
#   py <- 10:12
#   points(px, py, pch = 21, bg = "green")
#   # add custom legend
#   legend("topleft", legend = c("group A", "group B"),
#          pch = 21, pt.bg = c("blue", "green"), bty = "n")
# })
# 
# locus_plot(locp, pcutoff = NULL, panel.first = pf, panel.last = pl)

## ----echo=FALSE, fig.dim = c(7,7)---------------------------------------------
if (require(EnsDb.Hsapiens.v75)) {
pf <- quote({
  v <- locp$TX[locp$TX$gene_name == "UBE2L3", c("start", "end")]
  abline(v = v, col = "orange")
})

pl <- quote({
  lx <- locp$data$pos[locp$data$rsid == locp$index_snp]
  ly <- locp$data$logP[locp$data$rsid == locp$index_snp]
  text(lx, ly, locp$index_snp, pos = 4, cex = 0.8)
  px <- rep(22.05e6, 3)
  py <- 10:12
  points(px, py, pch = 21, bg = "green")
  legend("topleft", legend = c("group A", "group B"),
         pch = 21, pt.bg = c("blue", "green"), bty = "n")
})

locus_plot(locp, pcutoff = NULL, panel.first = pf, panel.last = pl)
}

## ----eval=FALSE---------------------------------------------------------------
# locus_ggplot(loc)

## ----eval=FALSE---------------------------------------------------------------
# gg_genetracks(loc)

## ----eval=FALSE---------------------------------------------------------------
# p <- gg_scatter(loc)
# p

## ----eval=FALSE---------------------------------------------------------------
# gg_addgenes(p, loc)

## ----eval=FALSE---------------------------------------------------------------
# p <- gg_scatter(loc) +
#   theme(legend.text = element_text(size = 14),
#         legend.title = element_text(size = 14))
# gg_addgenes(p, loc)

## ----eval=FALSE---------------------------------------------------------------
# locus_ggplot(loc, beta = "beta")
# 
# # add column 'typed' as a factor with levels "typed", "imputed"
# loc$data$typed <- factor(rbinom(n = nrow(loc$data), 1, 0.3), labels = c("imputed", "typed"))
# locus_ggplot(loc, shape = "typed")

## ----eval=FALSE---------------------------------------------------------------
# g <- gg_genetracks(loc)
# library(cowplot)
# plot_grid(p, p, g, ncol = 1, rel_heights = c(2, 2, 1), align = "v")
# 
# # patchwork method
# library(patchwork)
# p / p / g
# 
# # patchwork method 2
# wrap_plots(p, p, g, ncol = 1)

## ----eval=FALSE---------------------------------------------------------------
# library(cowplot)
# p1 <- locus_ggplot(loc, labels = "index", nudge_x = 0.03)
# p2 <- locus_ggplot(loc2, legend_pos = NULL)
# plot_grid(p1, p2, ncol = 2)

## ----echo=FALSE, fig.dim = c(12, 7)-------------------------------------------
if (require(EnsDb.Hsapiens.v75)) {
p1 <- locus_ggplot(loc, labels = "index", nudge_x = 0.03)
p2 <- locus_ggplot(loc2, legend_pos = NULL)
cowplot::plot_grid(p1, p2, ncol = 2)
}

## ----eval=FALSE---------------------------------------------------------------
# library(ggpubr)
# pdf("my_ggplot.pdf", width = 10)
# ggarrange(p1, p2, ncol = 2)
# dev.off()
# 
# library(gridExtra)
# pdf("my_ggplot.pdf", width = 10)
# grid.arrange(p1, p2, ncol = 2)
# dev.off()

## ----eval=FALSE---------------------------------------------------------------
# locus_plotly(loc2)

## ----out.width='70%', fig.align="center", echo=FALSE--------------------------
knitr::include_graphics("plotly.png")

## ----eval=FALSE---------------------------------------------------------------
# # FTP download full summary statistics of this SLE GWAS from
# # https://www.ebi.ac.uk/gwas/studies/GCST003156
# library(data.table)
# SLE_gwas <- fread('../bentham_2015_26502338_sle_efo0002690_1_gwas.sumstats.tsv')
# 
# pks <- quick_peak(SLE_gwas)
# ## 34 peaks found (0.328 secs)
# 
# top_snps <- SLE_gwas$rsid[pks]
# head(top_snps)
# ## [1] "rs141910407" "rs114056368" "rs4274624"   "rs17849501"  "rs1143679"   "rs114115096"

## ----eval=FALSE---------------------------------------------------------------
# all_loci <- lapply(top_snps, function(i) {
#   loc <- locus(data = SLE_gwas, index_snp = i, fix_window = 1e6,
#         ens_db = "EnsDb.Hsapiens.v75")
#   link_recomb(loc)
# })
# 
# pdf("sle_loci.pdf")
# tmp <- lapply(all_loci, locus_plot, labels = "index")
# dev.off()

## ----eval=FALSE---------------------------------------------------------------
# pp <- lapply(all_loci, locus_ggplot, labels = "index", nudge_y = 1)
# 
# library(gridExtra)
# pdf("sle_loci_gg.pdf")
# for (i in seq_along(pp)) {
#   grid.arrange(pp[[i]])
# }
# dev.off()

#' Query UCSC for Recombination data 
#' 
#' Adds recombination data to a 'locus' object by querying UCSC genome browser.
#' 
#' @param loc Object of class 'locus' generated by [locus()]
#' @param genome Either `"hg38"` or `"hg19"`
#' @param table Optional character value specifying which recombination table to
#'   use.
#' @param recomb Optional `GRanges` class object of recombination data.
#' @details
#' Uses the `rtracklayer` package to query UCSC genome browser for recombination
#' rate data.
#' 
#' Possible options for `table` for hg19 are `"hapMapRelease24YRIRecombMap"`,
#' `"hapMapRelease24CEURecombMap"`, `"hapMapRelease24CombinedRecombMap"` (the
#' default). The only option for `table` for hg38 is `"recomb1000GAvg"` (the
#' default).
#' 
#' If you are doing many queries, it may be much faster to download the entire 
#' recombination track data (around 30 MB for hg38) from the Recombination Rate 
#' Tracks page at
#' [UCSC genome browser](https://genome.ucsc.edu/cgi-bin/hgTrackUi?g=recombRate2).
#' The link to the hg38 download folder is 
#' <http://hgdownload.soe.ucsc.edu/gbdb/hg38/recombRate/> and for hg19 is 
#' <http://hgdownload.soe.ucsc.edu/gbdb/hg19/decode/>. These .bw files can be
#' converted to useable `GRanges` objects using `rtracklayer::import.bw()` (see
#' the vignette).
#' 
#' Sometimes `rtracklayer` generates intermittent API errors or warnings: try
#' calling `link_recomb()` again. If warnings persist restart your R session.
#' Errors are handled gracefully using `try()` to allow users to wrap
#' `link_recomb()` in a loop without quitting halfway. Error messages are still
#' shown. Successful API calls are cached using `memoise` to reduce API
#' requests.
#' 
#' @returns A list object of class 'locus'. Recombination data is added as list
#'   element `recomb`.
#' @importFrom GenomeInfoDb genome<- seqnames
#' @importFrom GenomicRanges GRanges
#' @importFrom IRanges IRanges
#' @importFrom rtracklayer browserSession ucscTableQuery getTable
#' @importFrom memoise drop_cache
#' @export
#'
link_recomb <- function(loc,
                        genome = loc$genome,
                        table = NULL,
                        recomb = NULL) {
  if (!inherits(loc, "locus")) stop("Not a locus object")
  if (!is.null(recomb)) {
    if (!inherits(recomb, "GRanges")) {
      warning("`recomb` is not a 'GRanges' class object")
    }
    seqname <- loc$seqname
    if (!grepl("chr", seqname)) seqname <- paste0("chr", seqname)
    rec <- recomb[seqnames(recomb) == seqname, ]
    rec <- rec[end(rec) > loc$xrange[1] & start(rec) < loc$xrange[2], ]
    rec <- as.data.frame(rec)[, c("start", "end", "score")]
    colnames(rec)[3] <- "value"
    loc$recomb <- rec
    return(loc)
  }
  genome <- switch(genome, "GRCh37" = "hg19", "GRCh38" = "hg38", genome)
  loc_genome <- switch(loc$genome, "GRCh37" = "hg19", "GRCh38" = "hg38",
                       loc$genome)
  if (loc_genome != genome) warning("mismatched genome build")
  loc$recomb <- get_recomb(genome, loc$xrange, loc$seqname, table)
  loc
}


query_recomb <- function(gen, xrange, seqname, table = NULL) {
  if (is.null(table)) {
    table <- if (gen == "hg38") {"recomb1000GAvg"
    } else if (gen == "hg19") "hapMapRelease24CombinedRecombMap"
  }
  if (!grepl("chr", seqname)) seqname <- paste0("chr", seqname)
  gr <- GRanges(ranges = IRanges(start = xrange[1], end = xrange[2]),
                seqnames = seqname)
  message("Retrieving recombination data from UCSC")
  session <- browserSession("UCSC")
  genome(session) <- gen
  query <- ucscTableQuery(session, table = table, range = gr)
  gtab <- try(getTable(query))
  if (inherits(gtab, "try-error")) return(NULL)
  gtab
}

# use memoise to reduce calls to UCSC API
mem_query_recomb <- memoise(query_recomb)

# drop memoise cache if error occurs
get_recomb <- function(gen, xrange, seqname, table = NULL) {
  ret <- mem_query_recomb(gen, xrange, seqname, table)
  if (is.null(ret)) drop_cache(mem_query_recomb)(gen, xrange, seqname, table)
  ret
}

#' Locus eQTL plot
#'
#' Produces a plot of eQTL data embedded in a 'locus' class object. Intended for
#' use with [set_layers()].
#'
#' @param loc Object of class 'locus' to use for plot. See [locus].
#' @param tissue GTex tissue in which eQTL has been measured
#' @param eqtl_gene Gene showing eQTL effect
#' @param scheme Character string specifying palette for effect size showing
#'   up/downregulation eQTL using [grDevices::hcl.colors]. Alternatively a
#'   vector of 6 colours.
#' @param col Outline point colour. `NA` for no outlines.
#' @param pcutoff Cut-off for p value significance. Defaults to p = 5e-08. Set
#'   to `NULL` to disable.
#' @param xlab x axis title.
#' @param ylab y axis title.
#' @param cex.axis Specifies font size for axis numbering.
#' @param xticks Logical whether x axis numbers and axis title are plotted.
#' @param border Logical whether a bounding box is plotted around upper and
#'   lower plots.
#' @param add Logical whether to add points to an existing plot or generate a
#'   new plot.
#' @param align Logical whether set [par()] to align the plot.
#' @param legend_pos Character value specifying legend position. See [legend()].
#' @param ... Other arguments passed to [plot()] for the scatter plot.
#' @return No return value. Produces a scatter plot using base graphics.
#' @seealso [locus()] [set_layers()] [scatter_plot()]
#' @importFrom graphics points
#' @export
#' 
eqtl_plot <- function(loc,
                      tissue = "Whole Blood",
                      eqtl_gene = loc$gene,
                      scheme = "RdYlBu",
                      col = NA,
                      pcutoff = NULL,
                      xlab = NULL,
                      ylab = expression("-log"[10] ~ "P"),
                      cex.axis = 0.9,
                      xticks = TRUE,
                      border = FALSE,
                      add = FALSE,
                      align = TRUE, 
                      legend_pos = "topright", ...) {
  if (!inherits(loc, "locus")) stop("Object of class 'locus' required")
  
  if (!"LDexp" %in% names(loc)) stop("Contains no eQTL data")
  data <- loc$LDexp
  data <- data[data$Tissue == tissue & data$Gene_Symbol == eqtl_gene, ]
  if (nrow(data) == 0) stop("No data")
  
  data$pos <- gsub(".*:", "", data$Position_grch37)  # remove up to ':'
  data$pos <- as.numeric(data$pos)
  data$logP <- -log10(data$P_value)
  
  # fix effect allele not being minor allele
  data$Effect_Allele_Freq <- gsub(".*=", "", data$Effect_Allele_Freq)
  data$Effect_Allele_Freq <- as.numeric(data$Effect_Allele_Freq)
  swap <- !is.na(data$Effect_Allele_Freq) & data$Effect_Allele_Freq > 0.5
  data$Effect_Size[swap] <- -data$Effect_Size[swap]
  data$pch <- -sign(data$Effect_Size) / 2 + 24.5
  equp <- sign(data$Effect_Size) == 1
  if (is.character(scheme)) {
    scheme <- hcl.colors(9, scheme)[-c(4:6)]
  }
  up_cols <- rev(scheme[1:3])
  down_cols <- scheme[4:6]
  ecol <- cut(abs(data$Effect_Size), breaks = 3)
  data$bg[equp] <- up_cols[ecol[equp]]
  data$bg[!equp] <- down_cols[ecol[!equp]]
  labs <- levels(ecol)
  cutlev <- cbind(lower = as.numeric( sub("\\((.+),.*", "\\1", labs) ),
                  upper = as.numeric( sub("[^,]*,([^]]*)\\]", "\\1", labs) ))
  cutlev <- signif(cutlev, 2)
  
  if (is.null(xlab)) xlab <- paste("Chromosome", loc$seqname, "(Mb)")
  
  # scatter plot
  recomb <- !is.null(loc$recomb)
  if (align) {
    op <- par(mar = c(ifelse(xticks, 3, 0.1), 3.5, 2,
                      ifelse(recomb, 3.5, 1.5)))
    on.exit(par(op))
  }
  
  if (!is.null(pcutoff)) {
    abl <- quote(abline(h = -log10(pcutoff), col = 'darkgrey', lty = 2))
  } else abl <- NULL
  
  new.args <- list(...)
  if (add) {
    plot.args <- list(x = data$pos, y = data$logP,
           pch = data$pch, bg = data$bg, col = col)
    if (length(new.args)) plot.args[names(new.args)] <- new.args
    do.call("points", plot.args)
    return()
  }
  plot.args <- list(x = data$pos, y = data$logP,
                    pch = data$pch, bg = data$bg, col = col,
                    las = 1, font.main = 1,
                    xlim = loc$xrange,
                    ylim = c(0, max(data$logP, na.rm = TRUE)),
                    xlab = if (xticks) xlab else "",
                    ylab = ylab,
                    bty = if (border) 'o' else 'l',
                    cex.axis = cex.axis,
                    xaxt = 'n',
                    tcl = -0.3, 
                    mgp = c(1.7, 0.5, 0),
                    panel.first = abl)
  if (length(new.args)) plot.args[names(new.args)] <- new.args
  do.call("plot", plot.args)
  
  if (xticks) {
    axis(1, at = axTicks(1), labels = axTicks(1) / 1e6, cex.axis = cex.axis,
         mgp = c(1.7, 0.4, 0), tcl = -0.3)
  } else if (!border) {
    axis(1, at = axTicks(1), labels = FALSE, tcl = -0.3)
  }
  if (!is.null(legend_pos)) {
    legtext <- c(rev(paste(cutlev[,1], cutlev[,2], sep=" : ")),
                 paste(-cutlev[,2], -cutlev[,1], sep=" : "))
    legend(legend_pos, legend = legtext, pch = rep(c(24, 25), each=3),
           col = col, pt.bg = c(rev(up_cols), down_cols),
           title = "eQTL effect",
           bty = "n", cex = 0.85, pt.cex = 1, y.intersp = 0.96)
  }
}

#' Locus scatter plotly
#'
#' Produces a scatter plot from a 'locus' class object using plotly.
#'
#' @param loc Object of class 'locus' to use for plot. See [locus].
#' @param index_snp Specifies index SNP or a vector of SNPs to be shown in a
#'   different colour and symbol. Defaults to the SNP with the lowest p-value.
#'   Set to `NULL` to not show this.
#' @param pcutoff Cut-off for p value significance. Defaults to p = 5e-08. Set
#'   to `NULL` to disable.
#' @param scheme Vector of 3 colours if LD is not shown: 1st = normal points,
#'   2nd = colour for significant points, 3rd = index SNP(s).
#' @param xlab x axis title.
#' @param ylab y axis title.
#' @param yzero Logical whether to force y axis limit to include y=0.
#' @param showLD Logical whether to show LD with colours
#' @param LD_scheme Vector of colours for plotting LD. The first colour is for
#'   SNPs which lack LD information. The next 5 colours are for r^2 or D' LD
#'   results ranging from 0 to 1 in intervals of 0.2. The final colour is for
#'   the index SNP.
#' @param marker_outline Specifies colour for outlining points.
#' @param marker_size Value for size of markers in plotly units.
#' @param recomb_col Colour for recombination rate line if recombination rate
#'   data is present. Set to `NA` to hide the line. See [link_recomb()] to add
#'   recombination rate data.
#' @param eqtl_gene Column name in `loc$data` for eQTL genes.
#' @param beta Optional column name for beta coefficient to display upward
#'   triangles for positive beta and downward triangles for negative beta
#'   (significant SNPs only).
#' @param add_hover Optional vector of column names in `loc$data` to add to the
#'   plotly hover text for scatter points.
#' @param showlegend Logical whether to show a legend for the scatter points.
#' @param height Height in pixels (optional, defaults to automatic sizing).
#' @param webGL Logical whether to use webGL or SVG for scatter plot.
#' @return A `plotly` scatter plot.
#' @seealso [locus()] [locus_plotly()]
#' @importFrom plotly add_trace plotly_build
#' @importFrom stats relevel
#' @export
#' 
scatter_plotly <- function(loc,
                           index_snp = loc$index_snp,
                           pcutoff = 5e-08,
                           scheme = c('grey', 'dodgerblue', 'red'),
                           xlab = NULL,
                           ylab = NULL,
                           yzero = (loc$yvar == "logP"),
                           showLD = TRUE,
                           LD_scheme = c('grey', 'royalblue', 'cyan2', 'green3', 
                                         'orange', 'red', 'purple'),
                           marker_outline = "black",
                           marker_size = 7,
                           recomb_col = "blue",
                           eqtl_gene = NULL,
                           beta = NULL,
                           add_hover = NULL,
                           showlegend = TRUE,
                           height = NULL,
                           webGL = TRUE) {
  if (!inherits(loc, "locus")) stop("Object of class 'locus' required")
  if (is.null(loc$data)) stop("No SNPs/data points", call. = FALSE)
  
  .call <- match.call()
  data <- loc$data
  if (is.null(xlab)) xlab <- paste("Chromosome", loc$seqname, "(Mb)")
  if (is.null(ylab)) {
    ylab <- if (loc$yvar == "logP") "-log<sub>10</sub> P" else loc$yvar
  }
  hasLD <- "ld" %in% colnames(data)
  leg <- list()
  if (!"bg" %in% colnames(data)) {
    if (showLD & hasLD) {
      data$bg <- cut(data$ld, -1:6/5, labels = FALSE)
      data$bg[is.na(data$bg)] <- 1L
      data$bg[data[, loc$labs] %in% index_snp] <- 7L
      data <- data[order(data$bg), ]
      scheme <- rep_len(LD_scheme, 7 - is.null(index_snp))
      data$bg <- factor(data$bg, levels = 1:7,
                        labels = c("unknown", "0.0 - 0.2", "0.2 - 0.4",
                                   "0.4 - 0.6", "0.6 - 0.8", "0.8 - 1.0",
                                   "index"))
      leg <- list(title = list(text = "Linkage r<sup>2</sup>"))
    } else if (!is.null(eqtl_gene)) {
      bg <- data[, eqtl_gene]
      bg[data[, loc$p] > pcutoff] <- "ns"
      data$bg <- relevel(factor(bg, levels = unique(bg)), "ns")
      if (is.null(.call$scheme)) scheme <- eqtl_scheme(nlevels(data$bg))
    } else {
      # default colours
      showLD <- FALSE
      data$bg <- scheme[1]
      if (loc$yvar == "logP") data$bg[data[, loc$p] < pcutoff] <- scheme[2]
      data$bg[data[, loc$labs] %in% index_snp] <- scheme[3]
      data$bg <- factor(data$bg, levels = scheme,
                        labels = c("ns", paste("P <", signif(pcutoff, 3)), "index"))
    }
  }
  if (!is.null(beta)) {
    # beta symbols
    data[, beta] <- signif(data[, beta], 3)
    symbol <- as.character(sign(data[, beta]))
    ind <- data[, loc$p] > pcutoff
    symbol[ind] <- "ns"
    data$symbol <- factor(symbol, levels = c("ns", "1", "-1"),
                          labels = c(" ", "up", "down"))
    # labels = c(" ", "&#946; > 0", "&#946; < 0")
    symbols <- c(21L, 24L, 25L)
    data$size <- 1L
    data$size[!ind] <- 2L
    sizes <- if (sum(!ind) == 0) c(50, 50) else c(50, 100)
    if (!webGL) sizes <- sizes/2
    leg <- list(traceorder = "reversed")
  } else {
    if (is.null(eqtl_gene)) {
      # default plot
      data$symbol <- data$bg
      symbols <- c(rep("circle", length(scheme) -1), "diamond")
    } else {
      # eqtl gene only
      data$symbol <- 1L
      symbols <- "circle"  
    }
  }
  
  # scatter plotly
  recomb <- !is.null(loc$recomb) & !is.na(recomb_col)
  
  xlim <- loc$xrange / 1e6
  xext <- diff(xlim) * 0.01
  xlim <- xlim + c(-xext, xext)
  
  ylim <- range(data[, loc$yvar], na.rm = TRUE)
  if (yzero) ylim[1] <- min(c(0, ylim[1]))
  ydiff <- diff(ylim)
  ylim[2] <- ylim[2] + ydiff * 0.05
  ylim[1] <- if (ylim[1] != 0) ylim[1] - ydiff *0.05 else ylim[1] - ydiff *0.02
  
  hovertext <- paste0(data[, loc$labs], "<br>Chr ",
                      data[, loc$chrom], ": ", data[, loc$pos],
                      "<br>P = ", signif(data[, loc$p], 3))
  add_hover <- c(beta, eqtl_gene, add_hover)
  if (!is.null(add_hover)) {
    for (i in add_hover) {
      hovertext <- paste0(hovertext, "<br>", i, ": ", data[, i])
    }
  }
  
  hline <- list(type = "line",
                line = list(width = 1, color = '#999999', dash = 'dash'),
                x0 = 0, x1 = 1, y0 = -log10(pcutoff), y1 = -log10(pcutoff),
                xref = "paper", layer = "below")
  type <- if (webGL) "scattergl" else "scatter"
  
  if (!recomb) {
    if (is.null(beta)) {
      # standard plotly
      p <- plot_ly(x = data[, loc$pos] / 1e6, y = data[, loc$yvar],
                   color = data$bg, colors = scheme,
                   symbol = data$symbol, symbols = symbols,
                   marker = list(size = marker_size, opacity = 0.8,
                                 line = list(width = 1, color = marker_outline)),
                   text = hovertext, hoverinfo = 'text',
                   key = data[, loc$labs],
                   showlegend = showlegend,
                   source = "plotly_locus", height = height,
                   type = type, mode = "markers")
    } else {
      # beta shapes
      p <- plot_ly(x = data[, loc$pos] / 1e6, y = data[, loc$yvar],
                   color = data$bg, colors = scheme,
                   symbol = data$symbol, symbols = symbols,
                   size = data$size, sizes = sizes,
                   marker = list(opacity = 0.8,
                                 line = list(width = 1, color = marker_outline)),
                   text = hovertext, hoverinfo = 'text',
                   key = data[, loc$labs],
                   showlegend = showlegend,
                   source = "plotly_locus", height = height,
                   type = type, mode = "markers")
    }
    p <- p %>%
      plotly::layout(xaxis = list(title = xlab,
                                  ticks = "outside",
                                  zeroline = FALSE, showgrid = FALSE,
                                  range = as.list(xlim)),
                     yaxis = list(title = ylab,
                                  ticks = "outside",
                                  fixedrange = TRUE,
                                  showline = TRUE,
                                  range = ylim),
                     shapes = hline, legend = leg, dragmode = "pan")
  } else {
    # double y axis with recombination
    ylim2 <- c(-2, 102)
    if (is.null(beta)) {
      # standard plotly
      p <- plot_ly(source = "plotly_locus", height = height) %>%
        # recombination line
        add_trace(x = loc$recomb$start / 1e6, y = loc$recomb$value,
                  hoverinfo = "none", colors = scheme,  # colors must go here
                  symbols = symbols,
                  name = "recombination", yaxis = "y2",
                  line = list(color = recomb_col, width = 1.5),
                  mode = "lines", type = type, showlegend = FALSE) %>%
        # scatter plot
        add_trace(x = data[, loc$pos] / 1e6, y = data[, loc$yvar],
                  color = data$bg,
                  symbol = data$symbol,
                  marker = list(size = marker_size, opacity = 0.8,
                                line = list(width = 1, color = marker_outline)),
                  text = hovertext, hoverinfo = 'text', key = data[, loc$labs],
                  showlegend = showlegend,
                  type = type, mode = "markers")
    } else {
      # beta shapes
      p <- plot_ly(source = "plotly_locus", height = height) %>%
        # recombination line
        add_trace(x = loc$recomb$start / 1e6, y = loc$recomb$value,
                  hoverinfo = "none", colors = scheme,  # colors must go here
                  symbols = symbols, sizes = sizes,
                  name = "recombination", yaxis = "y2",
                  line = list(color = recomb_col, width = 1.5),
                  mode = "lines", type = type, showlegend = FALSE) %>%
        # scatter plot
        add_trace(x = data[, loc$pos] / 1e6, y = data[, loc$yvar],
                  color = data$bg,
                  symbol = data$symbol,
                  size = data$size,
                  marker = list(opacity = 0.8,
                                line = list(width = 1, color = marker_outline)),
                  text = hovertext, hoverinfo = 'text', key = data[, loc$labs],
                  showlegend = showlegend,
                  type = type, mode = "markers")
    }
    p <- p %>%
      plotly::layout(xaxis = list(title = xlab,
                                  ticks = "outside",
                                  zeroline = FALSE,
                                  range = as.list(xlim)),
                     yaxis = list(title = ylab,
                                  ticks = "outside", showgrid = FALSE,
                                  showline = TRUE, fixedrange = TRUE,
                                  range = ylim),
                     yaxis2 = list(overlaying = "y", side = "right",
                                   title = "Recombination rate (%)",
                                   ticks = "outside", showgrid = FALSE,
                                   showline = TRUE, fixedrange = TRUE,
                                   zeroline = FALSE, range = ylim2),
                     shapes = hline,
                     legend = c(leg, x = 1.1, y = 1), showlegend = TRUE)
  }
  p <- p %>%
    plotly::config(displaylogo = FALSE,
                   modeBarButtonsToRemove = c("select2d", "lasso2d",
                                              "autoScale2d", "resetScale2d",
                                              "hoverClosest", "hoverCompare"))
  
  if (hasLD) suppressWarnings(plotly_build(p)) else p
}


#' @importFrom grDevices rainbow
eqtl_scheme <- function(n) {
  if (n < 8) {
    scheme <- c('grey', 'purple', 'green3', 'orange', 'royalblue', 'red',
                'cyan')[1:n]
  } else {
    scheme <- c('grey', rainbow(n -1))
  }
  scheme
}

#' SLE GWAS data subset
#'
#' Dataset of SNPs at 3 gene loci (UBE2L3, STAT4, IRF5) from GWAS on SLE
#' (Bentham et al, 2015, Nature Genetics 47(12):1457-64, PMID: 26502338).
#'
#' @format Data frame with 1990 rows and 11 variables
#' @source \url{https://www.ebi.ac.uk/gwas/studies/GCST003156}
#' @usage data(SLE_gwas_sub)
"SLE_gwas_sub"

#' Set up a column of multiple plots
#'
#' Uses [graphics::layout()] to set up multiple locus plots aligned in a column.
#'
#' @param n Number of plots (not including gene tracks on bottom)
#' @param heights Vector of length `nrow + 1` specifying height for plots with
#'   a gene track on the bottom
#' @param rev Logical whether to reverse plotting order and plot from bottom to 
#'   top
#' @return Sets [graphics::layout()] to enable multiple plots aligned in a column. The
#'   gene track is assumed to be positioned on the bottom. Returns `par()`
#'   invisibly so that layout can be reset to default at the end of plotting.
#' @seealso [graphics::layout()]
#' @export

set_layers <- function(n = 1,
                       heights = c(rep(3, n), 2),
                       rev = FALSE) {
  op <- par(no.readonly = TRUE)
  s <- if (rev) rev(seq_len(n +1)) else seq_len(n +1)
  mat <- matrix(s)
  graphics::layout(mat, heights = heights)
  invisible(op)
}

#' Plot gene tracks
#' 
#' Plot gene annotation tracks from `ensembldb` data.
#' 
#' @param locus Object of class 'locus' generated by [locus()].
#' @param filter_gene_name Vector of gene names to display.
#' @param filter_gene_biotype Vector of gene biotypes to be filtered. Use
#' [ensembldb::listGenebiotypes()] to display possible biotypes. For example, 
#' `ensembldb::listGenebiotypes(EnsDb.Hsapiens.v75)`
#' @param cex.axis Specifies font size for axis numbering.
#' @param cex.lab Specifies font size for axis titles.
#' @param cex.text Font size for gene text.
#' @param showExons Logical whether to show exons or simply show whole gene as a
#'   rectangle. If `showExons = FALSE` colours are specified by `exon_border`
#'   for rectangle border and `gene_col` for the fill colour.
#' @param maxrows Specifies maximum number of rows to display in gene 
#' annotation panel.
#' @param xticks Logical whether x axis ticks and numbers are plotted.
#' @param xlab Title for x axis. Defaults to chromosome `seqname` specified 
#' in `locus`.
#' @param border Logical whether a bounding box is plotted.
#' @param gene_col Colour for gene lines.
#' @param exon_col Fill colour for exons.
#' @param exon_border Border line colour outlining exons (or genes if
#'   `showExons` is `FALSE`). Set to `NA` for no border.
#' @param text_pos Character value of either 'top' or 'left' specifying
#'   placement of gene name labels.
#' @param italics Logical whether gene text is in italics.
#' @param highlight Vector of genes to highlight.
#' @param highlight_col Single colour or vector of colours for highlighted
#'   genes.
#' @param prioritise Vector of genes to be placed first in the gene tracks.
#' @param blanks Controls handling of genes with blank names: `"fill"` replaces
#'   blank gene symbols with ensembl gene ids. `"hide"` hides genes which are
#'   missing gene symbols.
#' @param showRecomb Logical controls alignment of right margin if
#'   recombination data present.
#' @param align Logical whether to set [par()] to align the plot.
#' @details
#' This function is called by [locus_plot()]. It can be used to plot the gene
#' annotation tracks on their own. It uses base graphics, so
#' [graphics::layout()] can be used to position adjacent plots above or below.
#' 
#' `gene_col`, `exon_col` and `exon_border` set colours for all genes, while
#' `highlight` and `highlight_col` can optionally be used together to highlight
#' specific genes of interest. For full control over every single gene, users
#' can add columns `gene_col`, `exon_col` and `exon_border` to the `TX` object
#' within the 'locus' object. Columns added to `TX` override their equivalent 
#' arguments.
#'
#' @return No return value.
#' @examples
#' if(require(EnsDb.Hsapiens.v75)) {
#' data(SLE_gwas_sub)
#' loc <- locus(SLE_gwas_sub, gene = 'UBE2L3', flank = 1e5,
#'              ens_db = "EnsDb.Hsapiens.v75")
#' genetracks(loc)
#' 
#' ## Limit the number of tracks
#' genetracks(loc, maxrows = 4)
#' 
#' ## Filter by gene biotype
#' genetracks(loc, filter_gene_biotype = 'protein_coding')
#' 
#' ## Customise colours
#' genetracks(loc, gene_col = 'grey', exon_col = 'orange',
#'            exon_border = 'darkgrey')
#' }
#' @importFrom BiocGenerics start end
#' @importFrom graphics axTicks axis lines rect text plot.new strwidth
#' @export

genetracks <- function(locus,
                       filter_gene_name = NULL,
                       filter_gene_biotype = NULL,
                       border = FALSE,
                       cex.axis = 0.9,
                       cex.lab = 1,
                       cex.text = 0.7,
                       gene_col = ifelse(showExons, 'blue4', 'skyblue'),
                       exon_col = 'blue4',
                       exon_border = 'blue4',
                       showExons = TRUE,
                       maxrows = NULL,
                       text_pos = 'top',
                       italics = FALSE,
                       xticks = TRUE,
                       xlab = NULL,
                       highlight = NULL,
                       highlight_col = "red",
                       prioritise = highlight,
                       blanks = c("fill", "hide"),
                       showRecomb = TRUE,
                       align = TRUE) {
  if (!inherits(locus, "locus")) stop("Object of class 'locus' required")
  blanks <- match.arg(blanks)
  TX <- locus$TX
  EX <- locus$EX
  xrange <- locus$xrange
  if (!is.null(filter_gene_name)) {
    TX <- TX[TX$gene_name %in% filter_gene_name, ]
  }
  if (!is.null(filter_gene_biotype)) {
    TX <- TX[TX$gene_biotype %in% filter_gene_biotype, ]
  }
  if (is.null(xlab)) xlab <- paste("Chromosome", locus$seqname, "(Mb)")
  
  recomb <- !is.null(locus$recomb) & showRecomb
  if (align) {
    op <- par(mar = c(ifelse(xticks, 3.5, 1), 3.5, 0.25,
                      ifelse(recomb, 3.5, 1.5)))
    on.exit(par(op))
  }
  
  if (nrow(TX) != 0) {
    TX <- gene_colours(TX, gene_col, exon_col, exon_border, showExons,
                       highlight, highlight_col)
    TX <- mapRow(TX, xlim = xrange, cex.text = cex.text, text_pos = text_pos,
                 blanks = blanks, prioritise = prioritise)
    maxrows <- if (is.null(maxrows)) max(TX$row) else min(c(max(TX$row), maxrows))
    if (max(TX$row) > maxrows) message(max(TX$row), " tracks needed to show all genes")
    TX <- TX[TX$row <= maxrows, ]
  } else maxrows <- 1
  
  plot(NA, xlim = xrange,
       ylim = c(-maxrows - 0.3, -0.3), 
       bty = if (border) 'o' else 'n',
       yaxt = 'n', xaxt = 'n',
       xlab = if (xticks) xlab else "",
       ylab = "",
       cex.lab = cex.lab,
       font.main = 1,
       mgp = c(1.7, 0.4, 0))
  if (xticks) {
    xd <- diff(xrange) * 0.04
    axis(1, at = xrange + c(-xd, xd), labels = FALSE, lwd.ticks = 0)  # extend line
    axis(1, at = axTicks(1), labels = axTicks(1) / 1e6, cex.axis = cex.axis,
         lwd = 0, lwd.ticks = 1,
         tcl = -0.3, mgp = c(1.7, 0.4, 0))
  }
  if (nrow(TX) == 0) {
    message("No genes to plot")
    return(invisible(NULL))
  }
  
  exheight <- switch(text_pos, "top" = 0.15, "left" = 0.3)
  if (showExons) {
    for (i in seq_len(nrow(TX))) {
      lines(TX[i, c('start', 'end')], rep(-TX[i, 'row'], 2),
            col = TX$gene_col[i], lwd = 1.5, lend = 1)
      e <- EX[EX$gene_id == TX$gene_id[i], ]
      exstart <- start(e)
      exend <- end(e)
      rect(exstart, -TX[i, 'row'] - exheight, exend, -TX[i, 'row'] + exheight,
           col = TX$exon_col[i], border = TX$exon_border[i],
           lwd = 0.5, lend = 2, ljoin = 1)
    }
  } else {
    # without exons
    rect(TX[, 'start'], -TX[, 'row'] - exheight,
         TX[, 'end'], -TX[, 'row'] + exheight,
         col = TX$gene_col, lwd = 1, lend = 2, ljoin = 1, border = exon_border)
  }
  
  font <- if (italics) 3 else NULL
  if (text_pos == "top") {
    tfilter <- which(TX$tmin >= (xrange[1] - diff(xrange) * 0.04) & 
                       (TX$tmax <= xrange[2] + diff(xrange) * 0.04))
    for (i in tfilter) {
      text(TX$mean[i], -TX[i, 'row'] + 0.45,
           labels = bquote_gene(TX$gene_name[i], TX$strand[i], italics),
           cex = cex.text, xpd = NA)
    }
  } else if (text_pos == "left") {
    tfilter <- if (border) {
      which(TX$tmin >= xrange[1])
    } else seq_len(nrow(TX))
    for (i in tfilter) {
      text(max(c(TX$start[i], xrange[1] - diff(xrange) * 0.04)), -TX[i, 'row'],
           labels = bquote_gene(TX$gene_name[i], TX$strand[i], italics),
           cex = cex.text, pos = 2, xpd = NA)
    }
  }
  
}

bquote_gene <- function(gene, strand, italics) {
  if (strand == "+") {
    if (!italics) {bquote(.(gene) * symbol("\256"))
    } else bquote(italic(.(gene)) * symbol("\256"))
  } else {
    if (!italics) {bquote(symbol("\254") * .(gene))
    } else bquote(symbol("\254") * italic(.(gene)))
  }
}

# map genes into rows without overlap
mapRow <- function(TX, gap = diff(xlim) * 0.02, cex.text = 0.7, 
                   xlim = range(TX[, c('start', 'end')]),
                   text_pos = 'top', blanks = "fill", prioritise = NULL) {
  blank <- TX$gene_name == ""
  if (any(blank)) {
    if (blanks == "fill") {
      TX$gene_name[blank] <- TX$gene_id[blank]
    } else if (blanks == "hide") {
      TX <- TX[!blank, ]
    }
  }
  w <- match(prioritise, TX$gene_name)
  w <- w[!is.na(w)]
  nprot <- TX$gene_biotype != "protein_coding"
  o <- order(nprot)
  o <- o[!o %in% w]
  TX <- TX[c(w, o), ]
  
  gw <- strwidth(paste0("--", TX$gene_name), units = "inch", 
                 cex = cex.text) * diff(xlim) / par("pin")[1]
  TX$mean <- rowMeans(TX[, c('start', 'end')])
  if (text_pos == 'top') {
    xd <- diff(xlim) * 0.04
    xlim2 <- xlim + c(-xd, xd)
    # left edge
    fix_left <- (TX$mean - gw / 2) < xlim2[1] & (xlim2[1] + gw) < TX$end
    if (any(fix_left)) {
      TX$mean[fix_left] <- xlim2[1] + gw[fix_left] / 2
    }
    # right edge
    fix_right <- (TX$mean + gw / 2) > xlim2[2] & (xlim2[2] - gw) > TX$start
    if (any(fix_right)) {
      TX$mean[fix_right] <- xlim2[2] - gw[fix_right] / 2
    }
    TX$tmin <- TX$mean - gw / 2
    TX$tmax <- TX$mean + gw / 2
  } else if (text_pos == 'left') {
    TX$tmin <- TX$start - gw - gap
    TX$tmax <- TX$end
  } else if (text_pos == 'none') {
    TX$tmax <- TX$tmin <- TX$mean
  }
  TX$min <- pmin(TX$start, TX$end, TX$tmin) - gap / 2
  TX$max <- pmax(TX$start, TX$end, TX$tmax) + gap / 2
  TX$row <- 0
  j <- 1
  while (any(TX$row == 0)) {
    xset <- which(TX$row == 0)
    for (i in xset) {
      # overlap detection
      if (!any(TX$min[i] < TX$max[TX$row == j] &
               TX$max[i] > TX$min[TX$row == j])) {
        TX$row[i] <- j
      }
    }
    j <- j + 1
  }
  TX
}


# highlight selected genes
gene_colours <- function(TX, gene_col, exon_col, exon_border, showExons,
                         highlight, highlight_col) {
  if (is.null(TX$gene_col)) TX$gene_col <- gene_col
  if (is.null(TX$exon_col)) TX$exon_col <- exon_col
  if (is.null(TX$exon_border)) TX$exon_border <- exon_border
  w <- match(highlight, TX$gene_name)
  w <- w[!is.na(w)]
  if (length(w) > 0) {
    TX$gene_col[w] <- highlight_col
    if (showExons) {
      TX$exon_col[w] <- highlight_col
      TX$exon_border[w] <- highlight_col
    }
  }
  TX
}

#' Locus scatter plot
#'
#' Produces a base graphics scatter plot from a 'locus' class object. This
#' function is called by [locus_plot()] to generate the scatter plot portion.
#' Can be used manually with [set_layers()].
#'
#' @param loc Object of class 'locus' to use for plot. See [locus].
#' @param index_snp Specifies index SNP or a vector of SNPs to be shown in a
#'   different colour and symbol. Defaults to the SNP with the lowest p-value.
#'   Set to `NULL` to not show this.
#' @param pcutoff Cut-off for p value significance. Defaults to p = 5e-08. Set
#'   to `NULL` to disable.
#' @param scheme Vector of 3 colours if LD is not shown: 1st = normal points,
#'   2nd = colour for significant points, 3rd = index SNP(s).
#' @param cex Specifies size for points.
#' @param cex.axis Specifies font size for axis numbering.
#' @param cex.lab Specifies font size for axis titles.
#' @param xlab x axis title.
#' @param ylab y axis title.
#' @param ylim y axis limits (y1, y2).
#' @param ylim2 Secondary y axis limits for recombination line, if present.
#' @param yzero Logical whether to force y axis limit to include y=0.
#' @param xticks Logical whether x axis numbers and axis title are plotted.
#' @param border Logical whether a bounding box is plotted around upper and
#'   lower plots.
#' @param showLD Logical whether to show LD with colours
#' @param LD_scheme Vector of colours for plotting LD. The first colour is for
#'   SNPs which lack LD information. The next 5 colours are for r2 or D' LD
#'   results ranging from 0 to 1 in intervals of 0.2. The final colour is for
#'   the index SNP.
#' @param recomb_col Colour for recombination rate line if recombination rate
#'   data is present. Set to `NA` to hide the line. See [link_recomb()] to add
#'   recombination rate data.
#' @param recomb_offset Offset from 0-1 which shifts the scatter plot up and
#'   recombination line plot down. Recommended value 0.1.
#' @param legend_pos Position of legend. See [legend()]. Set to `NULL` to hide
#'   legend.
#' @param labels Character vector of SNP or genomic feature IDs to label. The
#'   value "index" selects the highest point or index SNP as defined when
#'   [locus()] is called. Set to `NULL` to remove all labels.
#' @param label_x Value or vector for position of label as percentage of x axis
#'   scale.
#' @param label_y Value or vector for position of label as percentage of y axis
#'   scale.
#' @param eqtl_gene Column name in `loc$data` for colouring eQTL genes.
#' @param beta Optional column name for beta coefficient to display upward
#'   triangles for positive beta and downward triangles for negative beta
#'   (significant SNPs only).
#' @param add Logical whether to add points to an existing plot or generate a
#'   new plot.
#' @param align Logical whether to set [par()] to align the plot.
#' @param ... Other arguments passed to [plot()] to control the scatter plot
#'   e.g. `main`, `ylim` etc.
#' @return No return value. Produces a scatter plot using base graphics.
#' @details
#' Advanced users familiar with base graphics can customise every single point
#' on the scatter plot, by adding columns named `bg`, `col`, `pch` or `cex`
#' directly to the dataframe stored in `$data` element of the 'locus' object.
#' Setting these will overrule any default settings. These columns refer to
#' their respective base graphics arguments, see [graphics::points()].
#' 
#' @seealso [locus()] [set_layers()]
#' @importFrom graphics par legend mtext
#' @export
#' 
scatter_plot <- function(loc,
                         index_snp = loc$index_snp,
                         pcutoff = 5e-08,
                         scheme = c('grey', 'dodgerblue', 'red'),
                         cex = 1,
                         cex.axis = 0.9,
                         cex.lab = 1,
                         xlab = NULL,
                         ylab = NULL,
                         ylim = NULL,
                         ylim2 = c(0, 100),
                         yzero = (loc$yvar == "logP"),
                         xticks = TRUE,
                         border = FALSE,
                         showLD = TRUE,
                         LD_scheme = c('grey', 'royalblue', 'cyan2', 'green3', 
                                       'orange', 'red', 'purple'),
                         recomb_col = "blue",
                         recomb_offset = 0,
                         legend_pos = 'topleft',
                         labels = NULL,
                         label_x = 4, label_y = 4,
                         eqtl_gene = NULL,
                         beta = NULL,
                         add = FALSE,
                         align = TRUE, ...) {
  if (!inherits(loc, "locus")) stop("Object of class 'locus' required")
  if (is.null(loc$data)) stop("No data points, only gene tracks")
  
  .call <- match.call()
  data <- loc$data
  if (is.null(xlab)) xlab <- paste("Chromosome", loc$seqname, "(Mb)")
  if (is.null(ylab)) {
    ylab <- if (loc$yvar == "logP") expression("-log"[10] ~ "P") else loc$yvar
  }
  hasLD <- "ld" %in% colnames(data)
  if (!"bg" %in% colnames(data)) {
    if (showLD & hasLD) {
      data$ld[data$ld == 0] <- 1e-6
      data$bg <- cut(data$ld, -1:6/5, labels = FALSE)
      data$bg[is.na(data$bg)] <- 1L
      data$bg[data[, loc$labs] %in% index_snp] <- 7L
      data <- data[order(data$bg), ]
      LD_scheme <- rep_len(LD_scheme, 7)
      data$bg <- LD_scheme[data$bg]
    } else if (!is.null(eqtl_gene)) {
      # eqtl gene colours
      bg <- data[, eqtl_gene]
      bg[data[, loc$p] > pcutoff] <- "ns"
      bg <- relevel(factor(bg, levels = unique(bg)), "ns")
      if (is.null(.call$scheme)) scheme <- eqtl_scheme(nlevels(bg))
      data$bg <- scheme[bg]
    } else {
      # default colours
      data$bg <- 1L
      if (loc$yvar == "logP") data$bg[data[, loc$p] < pcutoff] <- 2L
      data$bg[data[, loc$labs] %in% index_snp] <- 3L
      data <- data[order(data$bg), ]
      data$bg <- scheme[data$bg]
    }
  }
  
  # scatter plot
  recomb <- !is.null(loc$recomb) & !is.na(recomb_col)
  if (align) {
    op <- par(mar = c(ifelse(xticks, 3, 0.1), 3.5, 2,
                      ifelse(recomb, 3.5, 1.5)))
    on.exit(par(op))
  }
  
  if (is.null(ylim)) {
    ylim <- range(data[, loc$yvar], na.rm = TRUE)
    if (yzero & is.null(ylim)) ylim[1] <- min(c(0, ylim[1]))
    if (!is.null(labels) & (border | recomb)) {
      ylim[2] <- ylim[2] + diff(ylim) * 0.08
    }
  }
  # offset y1
  yd <- diff(ylim)
  if (recomb && recomb_offset != 0) ylim[1] <- ylim[1] - yd * recomb_offset
  
  panel.first <- quote({
    if (loc$yvar == "logP" & !is.null(pcutoff)) {
      abline(h = -log10(pcutoff), col = 'darkgrey', lty = 2)
    }
    if (recomb) {
      yd2 <- diff(ylim2)
      fy2 <- function(yy) (yy - ylim2[1]) / yd2 * yd + ylim[1]
      ry <- fy2(loc$recomb$value)
      lines(loc$recomb$start, ry, col = recomb_col)
      labs2 <- pretty(ylim2)
      at <- fy2(labs2)
      axis(4, at = at, labels = labs2,
           las = 1, tcl = plot.args$tcl, mgp = plot.args$mgp,
           cex.axis = cex.axis)
      mtext("Recombination rate (%)", 4, cex = cex.lab * par("cex"),
            line = plot.args$mgp[1],
            adj = max(c(0.5 - recomb_offset / 2, 0)))
    }
  })
  
  # shapes
  pch <- rep(21L, nrow(data))
  pch[data[, loc$labs] %in% index_snp] <- 23L
  if (!is.null(beta)) {
    sig <- data[, loc$p] < pcutoff
    pch[sig] <- 24 + (1 - sign(data[sig, beta])) / 2
  }
  if ("pch" %in% colnames(data)) pch <- data$pch
  col <- "black"
  if ("col" %in% colnames(data)) col <- data$col
  if ("cex" %in% colnames(data)) cex <- data$cex
  
  new.args <- list(...)
  if (add) {
    plot.args <- list(x = data[, loc$pos], y = data[, loc$yvar],
                      pch = pch, bg = data$bg, cex = cex)
    if (length(new.args)) plot.args[names(new.args)] <- new.args
    return(do.call("points", plot.args))
  }
  
  bty <- if (border | recomb) 'o' else 'l'
  plot.args <- list(x = data[, loc$pos], y = data[, loc$yvar],
               pch = pch, bg = data$bg, col = col,
               las = 1, font.main = 1,
               cex = cex, cex.axis = cex.axis, cex.lab = cex.lab,
               xlim = loc$xrange,
               ylim = ylim,
               xlab = if (xticks) xlab else "",
               ylab = ylab,
               bty = bty,
               xaxt = 'n',
               tcl = -0.3, 
               mgp = c(1.7, 0.5, 0),
               panel.first = panel.first)
  if (recomb && recomb_offset != 0) {
    plot.args$ylab <- ""
    plot.args <- c(plot.args, yaxt = 'n')
  }
  if (length(new.args)) plot.args[names(new.args)] <- new.args
  do.call("plot", plot.args)
  
  # offset y1 axis ticks
  if (recomb && recomb_offset != 0) {
    ypretty <- pretty(c(min(data[, loc$yvar], na.rm = TRUE), ylim[2]))
    axis(2, at = ypretty, las = 1, mgp = plot.args$mgp, cex.axis = cex.axis,
         tcl = plot.args$tcl)
    mtext(ylab, 2, cex = cex.lab * par("cex"), line = plot.args$mgp[1],
          adj = min(c(0.5 + recomb_offset / 2.7, 1)))
  }
  
  # add labels
  if (!is.null(labels)) {
    i <- grep("index", labels, ignore.case = TRUE)
    if (length(i) > 0) {
      if (length(index_snp) == 1) {
        labels[i] <- index_snp
      } else {
        labels <- labels[-i]
        labels <- c(index_snp, labels)
      }
    }
    ind <- match(labels, data[, loc$labs])
    if (any(is.na(ind))) {
      message("label ", paste(labels[is.na(ind)], collapse = ", "),
              " not found")
    }
    lx <- data[ind, loc$pos]
    ly <- data[ind, loc$yvar]
    labs <- data[ind, loc$labs]
    add_labels(lx, ly, labs, label_x, label_y, cex = cex.axis *0.95)
  }
  
  if (xticks) {
    axis(1, at = axTicks(1), labels = axTicks(1) / 1e6, cex.axis = cex.axis,
         mgp = c(1.7, 0.4, 0), tcl = plot.args$tcl)
  } else if (!border) {
    axis(1, at = axTicks(1), labels = FALSE, tcl = plot.args$tcl)
  }
  if (!is.null(legend_pos)) {
    leg <- pt.bg <- pch <- title <- NULL
    if (!is.null(eqtl_gene)) {
      leg <- levels(bg)[-1]
      pt.bg <- scheme[-1]
      pch <- c(rep(21, length(scheme) -1))
    } else if (showLD & hasLD) {
      leg <- c("0.8 - 1.0", "0.6 - 0.8", "0.4 - 0.6", "0.2 - 0.4",
               "0.0 - 0.2")
      title <- expression({r^2})
      pch <- rep(21, 5)
      pt.bg <- rev(LD_scheme[-c(1, 7)])
    }
    if (!is.null(beta)) {
      leg <- c(leg, expression({beta > 0}), expression({beta < 0}))
      pch <- c(pch, 2, 6)
      pt.bg <- c(pt.bg, NA, NA)
    }
    if (!is.null(leg)) {
      legend(legend_pos, legend = leg, y.intersp = 0.96, title = title,
             pch = pch, pt.bg = pt.bg, col = 'black', bty = 'n', cex = 0.8)
    }
  }
}


add_labels <- function(lx, ly, labs, label_x, label_y, cex = 1) {
  label_x <- rep_len(label_x, length(lx))
  label_y <- rep_len(label_y, length(ly))
  dx <- diff(par("usr")[1:2]) * label_x /100
  dy <- diff(par("usr")[3:4]) * label_y /100
  dlines(lx, ly, dx, dy, xpd = NA)
  adj1 <- -sign(dx) *0.56+0.5
  adj2 <- -sign(dy) +0.5
  adj2[abs(label_x) > abs(label_y)] <- 0.5
  adj1[abs(label_x) < abs(label_y)] <- 0.5
  if (length(unique(adj1)) == 1 & length(unique(adj2)) == 1) {
    # unique adj
    adj <- c(adj1[1], adj2[1])
    text(lx + dx, ly + dy, labs,
         adj = adj, cex = cex, xpd = NA)
  } else {
    # varying adj
    adj <- cbind(adj1, adj2)
    for (i in seq_along(labs)) {
      text(lx[i] + dx[i], ly[i] + dy[i], labs[i],
           adj = adj[i,], cex = cex, xpd = NA)
    }
  }
}


dlines <- function(x, y, dx, dy, ...) {
  mx <- cbind(x, x + dx, NA)
  my <- cbind(y, y + dy, NA)
  xs <- as.vector(t(mx))
  ys <- as.vector(t(my))
  lines(xs, ys, ...)
}

#' Fast peak finder in GWAS data
#' 
#' Simple but fast function for finding peaks in genome-wide association study
#' (GWAS) data based on setting a minimum distance between peaks.
#' 
#' @param data GWAS dataset (data.frame or data.table)
#' @param npeaks Number of peaks to find. If set to `NA`, algorithm finds all
#'   distinct peaks separated from one another by region size specified by
#'   `span`.
#' @param p_cutoff Specifies cut-off for p-value significance above which
#'   p-values are ignored.
#' @param span Minimum genomic distance between peaks (default 1 Mb)
#' @param min_points Minimum number of p-value significant points which must lie
#'   within the span of a peak. This removes peaks with single or only a few low
#'   p-value SNPs. To disable set `min_points` to 1 or less.
#' @param chrom Determines which column in `data` contains chromosome
#'   information. If `NULL` tries to autodetect the column.
#' @param pos Determines which column in `data` contains position information.
#'   If `NULL` tries to autodetect the column.
#' @param p Determines which column in `data` contains SNP p-values. If `NULL`
#'   tries to autodetect the column.
#' @details
#' This function is designed for speed. SNP p-values are filtered to only those
#' which are significant as specified by `p_cutoff`. Each peak is identified as
#' the SNP with the lowest p-value and then SNPs in proximity to each peak
#' within the distance specified by `span` are removed. Regions such as the HLA
#' whose peaks may well be broader than `span` may produce multiple entries.
#' @returns Vector of row indices
#' @export

quick_peak <- function(data, npeaks = NA, p_cutoff = 5e-08, span = 1e6,
                       min_points = 2,
                       chrom = NULL, pos = NULL, p = NULL) {
  start <- Sys.time()
  # autodetect column headings
  dc <- detect_cols(data, chrom, pos, p)
  chrom <- dc$chrom
  pos <- dc$pos
  p <- dc$p
  
  if (is.na(npeaks)) npeaks <- Inf
  
  i <- 2
  index <- order(data[, p])
  index <- index[data[index, p] < p_cutoff]
  pks <- index[1]
  del <- which(abs(data[pks, pos] - data[index, pos]) <= span &
                 data[pks, chrom] == data[index, chrom])[-1]
  if (length(del) > 0) index <- index[-del]
  while (length(pks) < npeaks & i <= length(index)) {
    if (all(abs(data[index[i], pos] - data[pks, pos]) > span |
            data[index[i], chrom] != data[pks, chrom])) {
      del <- which(abs(data[index[i], pos] - data[index, pos]) <= span & 
                     data[index[i], chrom] == data[index, chrom])
      if (length(del) >= min_points) {
        pks <- c(pks, index[i])
        del <- del[-1]
        if (length(del) > 0) index <- index[-del]
      }
    }
    i <- i + 1
  }
  end <- Sys.time()
  message(length(pks), " peaks found (", format(end - start, digits = 3), ")")
  if (!is.infinite(npeaks) & length(pks) < npeaks)
    message("lower p_cutoff to find more peaks")
  pks
}

#' Create locus object for plotting
#' 
#' Creates object of class 'locus' for genomic locus plot similar to 
#' `locuszoom`.
#' 
#' @details 
#' This is an R version of `locuszoom` (http://locuszoom.org) for generating
#' publication ready Manhattan plots of gene loci. It references Ensembl
#' databases using the `ensembldb` Bioconductor package framework for annotating
#' genes and exons in the locus.
#' 
#' @param data Dataset (data.frame or data.table) to use for plot. We recommend
#'   that tibbles are converted to a normal data.frame. If unspecified or
#'   `NULL`, gene track information alone is returned.
#' @param gene Optional character value specifying which gene to view. Either
#'   `gene`, or `xrange` plus `seqname`, or `index_snp` must be specified.
#' @param xrange Optional vector of genomic position range for the x axis.
#' @param seqname Optional, specifies which chromosome to plot.
#' @param flank Single value or vector with 2 values for how much flanking 
#' region left and right of the gene to show. Defaults to 100kb.
#' @param fix_window Optional alternative to `flank`, which allows users to
#'   specify a fixed genomic window centred on the specified gene. Both `flank`
#'   and `fix_window` cannot be specified simultaneously.
#' @param ens_db Either a character string which specifies which Ensembl
#'   database package (version 86 and earlier for Homo sapiens) to query for
#'   gene and exon positions (see `ensembldb` Bioconductor package). Or an
#'   `ensembldb` object which can be obtained from the AnnotationHub database.
#'   See the vignette and the `AnnotationHub` Bioconductor package for how to
#'   create this object.
#' @param chrom Determines which column in `data` contains chromosome 
#' information. If `NULL` tries to autodetect the column.
#' @param pos Determines which column in `data` contains position information. 
#' If `NULL` tries to autodetect the column.
#' @param p Determines which column in `data` contains SNP p-values. 
#' If `NULL` tries to autodetect the column.
#' @param yvar Specifies column in `data` for plotting on the y axis as an
#'   alternative to specifying p-values. Both `p` and `yvar` cannot be specified
#'   simultaneously.
#' @param labs Determines which column in `data` contains SNP rs IDs.
#' If `NULL` tries to autodetect the column.
#' @param index_snp Specifies the index SNP. If not specified, the SNP with the
#'   lowest P value is selected. Can be used to specify locus region instead of
#'   specifying `gene`, or `seqname` and `xrange`.
#' @param LD Optional character value to specify which column in `data` contains
#'   LD information.
#' @param std_filter Logical, whether standard filters on chromosomes 1-22, X &
#'   Y, and filtering of genes to only those whose transcript ids start with
#'   "ENS" are applied. For users with novel genome assemblies, this probably
#'   needs to be set to `FALSE`.
#' @return Returns a list object of class 'locus' ready for plotting,
#'   containing:
#' \item{seqname}{chromosome value}
#' \item{xrange}{vector of genomic position range}
#' \item{gene}{gene name}
#' \item{ens_db}{Ensembl or AnnotationHub database}
#' \item{ens_version}{Ensembl database version}
#' \item{organism}{Ensembl database organism}
#' \item{genome}{Ensembl data genome build}
#' \item{chrom}{column name in `data` containing chromosome information}
#' \item{pos}{column name in `data` containing position}
#' \item{p}{column name in `data` containing p-value}
#' \item{yvar}{column name in `data` to be plotted on y axis as alternative to 
#' `p`}
#' \item{labs}{column name in `data` containing SNP IDs}
#' \item{index_snp}{id of the most significant SNP}
#' \item{data}{the subset of GWAS data to be plotted}
#' \item{TX}{dataframe of transcript annotations}
#' \item{EX}{`GRanges` object of exon annotations}
#' If `data` is `NULL` when `locus()` is called then gene track information
#' alone is returned.
#' @seealso [locus_plot()] [locus_ggplot()] [locus_plotly()]
#' @examples
#' ## Bioconductor package EnsDb.Hsapiens.v75 is needed for these examples
#' if(require(EnsDb.Hsapiens.v75)) {
#' data(SLE_gwas_sub)
#' loc <- locus(SLE_gwas_sub, gene = 'UBE2L3', flank = 1e5,
#'              ens_db = "EnsDb.Hsapiens.v75")
#' summary(loc)
#' locus_plot(loc)
#' loc2 <- locus(SLE_gwas_sub, gene = 'STAT4', flank = 1e5,
#'               ens_db = "EnsDb.Hsapiens.v75")
#' locus_plot(loc2)
#' }
#' @importFrom ensembldb genes exons ensemblVersion organism
#' @importFrom BiocGenerics start end
#' @importFrom AnnotationFilter GeneNameFilter AnnotationFilterList SeqNameFilter GeneIdFilter TxStartFilter TxEndFilter ExonStartFilter ExonEndFilter
#' @importFrom GenomeInfoDb seqlengths genome
#' @importFrom memoise memoise
#' @export

locus <- function(data = NULL,
                  gene = NULL,
                  xrange = NULL, seqname = NULL,
                  flank = NULL, fix_window = NULL,
                  ens_db,
                  chrom = NULL, pos = NULL, p = NULL, yvar = NULL,
                  labs = NULL,
                  index_snp = NULL,
                  LD = NULL,
                  std_filter = TRUE) {
  if (is.character(ens_db)) {
    if (!ens_db %in% (.packages())) {
      stop("Ensembl database not loaded. Try: library(", ens_db, ")",
           call. = FALSE)
    }
    edb <- get(ens_db)
  } else edb <- ens_db
  if (!is.null(flank) & !is.null(fix_window))
    stop("both `flank` and `fix_window` cannot be specified at the same time")
  if (is.null(flank)) flank <- 1e5
  flank <- rep_len(flank, 2)
  seqfilt <- if (std_filter) SeqNameFilter(c(1:22, 'X', 'Y')) else NULL
  genefilt <- if (std_filter) GeneIdFilter("ENS", "startsWith") else NULL
  
  if (!is.null(gene)) {
    locus <- genes(edb, filter = AnnotationFilterList(
      GeneNameFilter(gene), seqfilt))
    
    if (length(locus) == 0) stop("gene not found")
    if (length(locus) > 1) {
      message(sprintf('Identified %d genes matching name \'%s\', taking first',
                      length(locus), gene))
      locus <- locus[1]
    }
    seqname <- names(seqlengths(locus))
    if (is.null(fix_window)) {
      xrange <- c(start(locus) - flank[1], end(locus) + flank[2])
    } else {
      m <- mean(c(start(locus), end(locus)))
      xrange <- as.integer(c(m - fix_window/2, m + fix_window/2))
    }
    xrange[xrange < 0] <- 0
  }
  
  if (!is.null(data)) {
    if (inherits(data, "tbl_df")) data <- as.data.frame(data)
    
    # autodetect headings
    dc <- detect_cols(data, chrom, pos, p, labs, yvar)
    chrom <- dc$chrom
    pos <- dc$pos
    p <- dc$p
    labs <- dc$labs
    
    if (!is.null(index_snp) & is.null(gene) & is.null(seqname) & is.null(xrange)) {
      # region based on index SNP
      if (!index_snp %in% data[, labs])
        stop("SNP specified by `index_snp` not found")
      ind <- which(data[, labs] == index_snp)
      if (length(ind) > 1) message("SNP found more than once")
      seqname <- data[ind[1], chrom]
      snp_pos <- data[ind[1], pos]
      xrange <- if (is.null(fix_window)) {
        c(snp_pos - flank[1], snp_pos + flank[2])
      } else {
        as.integer(c(snp_pos - fix_window/2, snp_pos + fix_window/2))
      }
      xrange[xrange < 0] <- 0
    }
  }
  
  if (is.null(xrange) | is.null(seqname)) stop('No locus specified')
  msg <- paste0("chromosome ", seqname, ", position ", xrange[1], " to ",
                xrange[2])
  if (!is.null(gene)) msg <- paste(gene, msg, sep = ", ")
  if (!is.null(index_snp)) msg <- paste(index_snp, msg, sep = ", ")
  message(msg)
  
  if (!is.null(data)) {
    data <- data[which(data[, chrom] == seqname), ]
    data <- data[which(data[, pos] > xrange[1] & data[, pos] < xrange[2]), ]
    # smallest floating point
    data[data[, p] < 5e-324, p] <- 5e-324
    if (is.null(yvar)) {
      data$logP <- -log10(data[, p])
      yvar <- "logP"
    }
    data <- as.data.frame(data)

    if (nrow(data) == 0) {
      message("Locus contains no SNPs/datapoints")
      data <- NULL
    } else {
      message(nrow(data), " SNPs/datapoints")
      if (is.null(index_snp)) index_snp <- data[which.max(data[, yvar]), labs]
      if (is.character(LD)) {
        colnames(data)[which(colnames(data) == LD)] <- "ld"
      }
    }
  }
  
  seqname <- gsub("chr|[[:punct:]]", "", seqname, ignore.case = TRUE)
  if (!seqname %in% c(1:22, "X", "Y")) 
    warning("`seqname` refers to a non-conventional chromosome")
  TX <- ensembldb::genes(edb, filter = AnnotationFilterList(
    SeqNameFilter(seqname),
    TxStartFilter(xrange[2], condition = "<"),
    TxEndFilter(xrange[1], condition = ">"), genefilt))
  TX <- data.frame(TX)
  TX <- TX[! is.na(TX$start), ]
  TX <- TX[!duplicated(TX$gene_id), ]
  
  if (nrow(TX) == 0) {
    message("No gene transcripts")
    # Creating empty exons object here in suitable format
    EX <- ensembldb::exons(edb, filter = AnnotationFilterList(
      SeqNameFilter(seqname),
      ExonStartFilter(xrange[2], condition = "<"),
      ExonEndFilter(xrange[1], condition = ">"), genefilt))
  } else {
    EX <- ensembldb::exons(edb, filter = GeneIdFilter(TX$gene_id))
  }

  loc <- list(seqname = seqname, xrange = xrange, gene = gene,
              ens_db = ens_db,
              ens_version = ensemblVersion(edb),
              organism = organism(edb),
              genome = unname(genome(edb)[1]),
              chrom = chrom, pos = pos, p = p, yvar = yvar, labs = labs,
              index_snp = index_snp,
              data = data, TX = TX, EX = EX)
  class(loc) <- "locus"
  loc
}


#' @export
summary.locus <- function(object, ...) {
  if (!is.null(object$gene)) {
    cat("Gene", object$gene, "\n")
  } else if (!is.null(object$index_snp)) {
    cat("Index SNP", object$index_snp, "\n")
  }
  cat(paste0("Chromosome ", object$seqname, ", position ",
             format(object$xrange[1], big.mark=","), " to ",
             format(object$xrange[2], big.mark=","), "\n"))
  nr <- nrow(object$data)
  if (is.null(nr)) nr <- 0
  cat(nr, "SNPs/datapoints\n")
  cat(nrow(object$TX), "gene transcripts\n")
  if (nrow(object$TX) > 0) {
    tb <- sort(c(table(object$TX$gene_biotype)), decreasing = TRUE)
    cat(paste(tb, names(tb), collapse = ", "), "\n")
  }
  if (!is.null(object$ens_version)) {
    cat("Ensembl version:", object$ens_version, "\n")
    cat("Organism:", object$organism, "\n")
    cat("Genome build:", object$genome, "\n")
  }
}


detect_cols <- function(data, chrom, pos, p, labs = NULL, yvar = NULL) {
  # autodetect headings
  if (is.null(chrom)) {
    w <- grep("chr", colnames(data), ignore.case = TRUE)
    if (length(w) == 1) {
      chrom <- colnames(data)[w]
    } else stop("unable to autodetect chromosome column")
  }
  if (is.null(pos)) {
    w <- grep("pos", colnames(data), ignore.case = TRUE)
    if (length(w) == 1) {
      pos <- colnames(data)[w]
    } else stop("unable to autodetect SNP position column")
  }
  if (!is.null(p) && !is.null(yvar)) stop("cannot specify both `p` and `yvar`")
  if (is.null(p) && is.null(yvar)) {
    if ("p" %in% colnames(data)) {
      p <- "p"
    } else {
      w <- grep("^p?val", colnames(data), ignore.case = TRUE)
      if (length(w) == 1) {
        p <- colnames(data)[w]
      } else stop("unable to autodetect p-value column")
    }
  }
  if (is.null(labs)) {
    w <- grep("rs?id", colnames(data), ignore.case = TRUE)
    if (length(w) > 1) stop("unable to autodetect SNP id column")
    if (length(w) == 0) {
      w <- grep("SNP", colnames(data), ignore.case = TRUE)
    }
    if (length(w) == 1) {
      labs <- colnames(data)[w]
    } else stop("unable to autodetect SNP id column")
  }
  
  # check headings
  if (!chrom %in% colnames(data)) {
    stop("Column specified by `chrom` not found in `data`")}
  if (!pos %in% colnames(data)) {
    stop("Column specified by `pos` not found in `data`")}
  if (is.null(yvar) && !p %in% colnames(data)) {
    stop("Column specified by `p` not found in `data`")}
  if (!labs %in% colnames(data)) {
    stop("Column specified by `labs` not found in `data`")}
  if (!is.null(yvar)) {
    if (!yvar %in% colnames(data)) {
      stop("Column specified by `yvar` not found in `data`")
    }
  }
  
  list(chrom = chrom, pos = pos, p = p, labs = labs)
}

#' Locus plotly
#' 
#' Genomic locus plot similar to locuszoom, using plotly.
#' 
#' @details 
#' This is an R/plotly version of locuszoom for exploring regional Manhattan
#' plots of gene loci. Use [locus()] first to generate an object of class
#' 'locus' for plotting. This references a selected Ensembl database for
#' annotating genes and exons. Hover over the points or gene tracks to reveal
#' more information.
#' 
#' @param loc Object of class 'locus' to use for plot. See [locus()].
#' @param heights Vector controlling relative height of each panel on 0-1 scale.
#'   Alternatively a vector of length 2 of height in pixels passed to
#'   `scatter_plotly()` and `genetrack_ly()`.
#' @param filter_gene_name Vector of gene names to display.
#' @param filter_gene_biotype Vector of gene biotypes to be filtered. Use
#' [ensembldb::listGenebiotypes()] to display possible biotypes. For example, 
#' `ensembldb::listGenebiotypes(EnsDb.Hsapiens.v75)`
#' @param cex.text Font size for gene text.
#' @param italics Logical whether gene text is in italics.
#' @param gene_col Colour for gene lines.
#' @param exon_col Fill colour for exons.
#' @param exon_border Border line colour outlining exons (or genes if
#'   `showExons` is `FALSE`). Set to `NA` for no border.
#' @param showExons Logical whether to show exons or simply show whole gene as a
#'   rectangle. If `showExons = FALSE` colours are specified by `exon_border`
#'   for rectangle border and `gene_col` for the fill colour.
#' @param maxrows Specifies maximum number of rows to display in gene 
#' annotation panel.
#' @param width Width of plotly plot in pixels which is purely used to prevent
#'   overlapping text for gene names.
#' @param xlab Title for x axis. Defaults to chromosome `seqname` specified 
#' in `locus`.
#' @param prioritise Vector of genes to be placed first in the gene tracks.
#' @param blanks Controls handling of genes with blank names: `"fill"` replaces
#'   blank gene symbols with ensembl gene ids. `"hide"` completely hides genes
#'   which are missing gene symbols. `"show"` shows gene lines but no label
#'   (hovertext is still available).
#' @param ... Optional arguments passed to [scatter_plotly()] to control the
#'   scatter plot.
#' @returns A 'plotly' plotting object showing a scatter plot above gene tracks.
#' @seealso [locus()] [genetrack_ly()] [scatter_plotly()]
#' @examples
#' if(require(EnsDb.Hsapiens.v75)) {
#' data(SLE_gwas_sub)
#' loc <- locus(SLE_gwas_sub, gene = "IRF5", flank = c(7e4, 2e5), LD = "r2",
#'              ens_db = "EnsDb.Hsapiens.v75")
#' locus_plotly(loc)
#' }
#' @export

locus_plotly <- function(loc, heights = c(0.6, 0.4),
                         filter_gene_name = NULL,
                         filter_gene_biotype = NULL,
                         cex.text = 0.7,
                         italics = FALSE,
                         gene_col = ifelse(showExons, 'blue4', 'skyblue'),
                         exon_col = 'blue4',
                         exon_border = 'blue4',
                         showExons = TRUE,
                         maxrows = 8,
                         width = 600,
                         xlab = NULL,
                         prioritise = NULL,
                         blanks = "show",
                         ...) {
  pheights <- NULL
  if (any(heights > 1)) {
    pheights <- heights
    pheights[2] <- sum(heights)
    heights <- heights / sum(heights)
  }
  
  g <- genetrack_ly(loc, filter_gene_name, filter_gene_biotype, cex.text, 
                    italics, gene_col, exon_col, exon_border, showExons, 
                    maxrows, width, xlab, prioritise, blanks,
                    height = pheights[2])
  p <- scatter_plotly(loc, xlab = xlab, height = pheights[1], ...)
  
  plotly::subplot(p, g, shareX = TRUE, nrows = 2, heights = heights,
                  titleY = TRUE, margin = 0)
}

#' Plot overlaying eQTL and GWAS data
#'
#' Experimental plotting function for overlaying eQTL data from GTEx on top of
#' GWAS results. y axis shows the -log10 p-value for the GWAS result.
#' Significant eQTL for the specified gene are overlaid using colours and
#' symbols.
#' 
#' @param loc Object of class 'locus' to use for plot. See [locus()].
#' @param base_col Colour of points for SNPs which do not have eQTLs.
#' @param alpha Alpha opacity for non-eQTL points
#' @param scheme Character string specifying palette for effect size showing
#'   up/downregulation eQTL using [grDevices::hcl.colors]. Alternatively a
#'   vector of 6 colours.
#' @param tissue GTex tissue in which eQTL has been measured
#' @param eqtl_gene Gene showing eQTL effect
#' @param legend_pos Character value specifying legend position. See [legend()].
#' @param ... Other arguments passed to [locus_plot()] for the locus plot.
#' @return No return value. Produces a plot using base graphics.
#' 
#' @importFrom grDevices adjustcolor hcl.colors
#' @export

overlay_plot <- function(loc,
                         base_col = 'black',
                         alpha = 0.5,
                         scheme = "RdYlBu",
                         tissue = "Whole Blood",
                         eqtl_gene = loc$gene,
                         legend_pos = "topright",
                         ...) {
  if (!inherits(loc, "locus")) stop("Object of class 'locus' required")
  if (!"LDexp" %in% names(loc)) stop("Missing eQTL data")
  if (is.null(eqtl_gene)) stop("eqtl_gene not specified")
  
  loc$data$bg <- adjustcolor(base_col, alpha.f = alpha)
  loc$data$pch <- 21
  # loc$data$col <- NA
  
  LDX <- loc$LDexp[loc$LDexp$Tissue == tissue & loc$LDexp$Gene_Symbol == eqtl_gene, ]
  # match by rsid
  ind <- match(loc$data[, loc$labs], LDX$RS_ID)
  message(sum(!is.na(ind)), "/", nrow(LDX), " matched eQTL SNPs (total ", 
          nrow(loc$data), ")")
  
  if (all(is.na(ind))) {
    message("No significant eQTL")
  } else {
    loc$data$eqtl_effect <- NA
    loc$data$eqtl_effect <- LDX$Effect_Size[ind]
    loc$data$eqtl_p <- LDX$P_value[ind]
    loc$data$eqtl_effect_allele <- LDX$Effect_Allele[ind]
    # gwas allele and eqtl effect allele are the same
    mismatch <- which(loc$data$eqtl_effect_allele != loc$data$effect_allele)
    which_rev <- loc$data$other_allele[mismatch] == loc$data$eqtl_effect_allele[mismatch]
    rev_effect <- mismatch[which_rev]
    mismatch <- mismatch[!which_rev]
    loc$data$eqtl_effect[rev_effect] <- -loc$data$eqtl_effect[rev_effect]
    loc$data$eqtl_effect[mismatch] <- NA
    if (length(scheme) == 1) {
      scheme <- hcl.colors(9, scheme)[-c(4:6)]
    }
    up_cols <- rev(scheme[1:3])
    down_cols <- scheme[4:6]
    ecol <- cut(abs(loc$data$eqtl_effect), breaks = 3)
    eqind <- !is.na(loc$data$eqtl_effect)
    eqdown <- eqind & sign(loc$data$eqtl_effect) == -1
    equp <- eqind & sign(loc$data$eqtl_effect) == 1
    loc$data$bg[equp] <- up_cols[ecol[equp]]
    loc$data$bg[eqdown] <- down_cols[ecol[eqdown]]
    loc$data$pch[eqind] <- 24.5 - sign(loc$data$eqtl_effect[eqind]) / 2
    # loc$data$col[eqind] <- "black"
    loc$data <- loc$data[order(loc$data$pch), ]
    pcex <- rep_len(0.9, nrow(loc$data))
    pcex[loc$data$pch != 21] <- 1.1
    labs <- levels(ecol)
    cutlev <- cbind(lower = as.numeric( sub("\\((.+),.*", "\\1", labs) ),
                    upper = as.numeric( sub("[^,]*,([^]]*)\\]", "\\1", labs) ))
    cutlev <- signif(cutlev, 2)
  }
  
  if (!is.null(legend_pos)) {
    legtext <- c(rev(paste(cutlev[,1], cutlev[,2], sep=" : ")),
                 paste(-cutlev[,2], -cutlev[,1], sep=" : "))
    legendFUN <- substitute(legend(lpos,
                                   legend = legtext,
                                   pch = rep(c(24, 25), each=3),
                                   pt.bg = cols, col = NA,
                                   title = "eQTL effect",
                                   bty = "n", cex = 0.85, pt.cex = 1,
                                   y.intersp = 0.96),
                            list(lpos = legend_pos, legtext = legtext,
                                 cols = c(rev(up_cols), down_cols)))
  } else legendFUN <- NULL
  
  locus_plot(loc, cex = pcex, col = NA, showLD = FALSE,
             panel.last = legendFUN, ...)
}

#' Obtain LD at a locus from LDlink
#' 
#' Adds LD information to a 'locus' class object. It queries LDlink
#' (https://ldlink.nci.nih.gov/) via the `LDlinkR` package to retrieve linkage
#' disequilibrium (LD) information on a reference SNP.
#' 
#' @param loc Object of class 'locus' generated by [locus()] 
#' @param pop A 1000 Genomes Project population, (e.g. YRI or CEU), multiple 
#' allowed, default = "CEU". Passed to `LDlinkR::LDmatrix()`.
#' @param r2d Either "r2" for LD r^2 or "d" for LD D', default = "r2". Passed 
#' to `LDlinkR::LDmatrix()` or `LDproxy()`.
#' @param token Personal access token for accessing 1000 Genomes LD data via 
#' LDlink API. See `LDlinkR` package documentation.
#' @param method Either `"proxy"` or `"matrix"`. Controls whether to use
#'    `LDproxy()` or `LDmatrix()` to obtain LD data.
#' @param genome_build Choose between one of the three options: 'grch37' for
#'   genome build GRCh37 (hg19), 'grch38' for GRCh38 (hg38), or
#'   'grch38_high_coverage' for GRCh38 High Coverage (hg38) 1000 Genome Project
#'   data sets. Default is GRCh37 (hg19).
#' @param ... Optional arguments which are passed on to `LDlinkR::LDmatrix()` or
#'   `LDlinkR::LDproxy()`
#' @return Returns a list object of class 'locus'. LD information is added as a
#'   column `ld` in list element `data`.
#' @seealso [locus()]
#' @details
#' The argument `method` controls which LDlinkR function is used to retrieve LD
#' data. `LDmatrix()` is slower but usually more complete for small queries
#' (<1000 SNPs). However, it has a limit of 1000 SNPs which can be queried.
#' `LDproxy()` is faster but data on some SNPs may be absent.
#' 
#' Note, SNPs have to be correctly formatted as required by LDlinkR, either as
#' rsID (works with either method) or chromosome coordinate e.g. "chr7:24966446"
#' (works with LDproxy only). Default genome build is `grch37`, see `LDproxy()`
#' or `LDmatrix()`.
#' 
#' @importFrom LDlinkR LDmatrix LDexpress LDproxy
#' @export

link_LD <- function(loc,
                    pop = "CEU",
                    r2d = "r2",
                    token = "",
                    method = c("proxy", "matrix"),
                    genome_build = loc$genome, ...) {
  if (!inherits(loc, "locus")) stop("Not a locus object")
  if (!requireNamespace("LDlinkR", quietly = TRUE)) {
    stop("Package 'LDlinkR' must be installed to use this feature",
         call. = FALSE)
  }
  if (token == "") stop("token is missing")
  
  start <- Sys.time()
  labs <- loc$labs
  index_snp <- loc$index_snp
  snp_col <- if (grepl("rs", index_snp)) "RS_Number" else "Coord"
  rslist <- loc$data[, labs]
  if (length(rslist) > 1000) {
    rslist <- rslist[order(loc$data$logP, decreasing = TRUE)]
    rslist <- unique(c(index_snp, rslist))[seq_len(1000)]
  }
  method <- match.arg(method)
  genome_build <- tolower(genome_build)
  if (!grepl(loc$genome, genome_build, ignore.case = TRUE)) {
    warning("mismatched genome build")
  }
  if (method == "proxy") {
    ldp <- try(mem_LDproxy(index_snp, pop = pop, r2d = r2d, token = token,
                           genome_build = genome_build, ...))
    if (!inherits(ldp, "try-error")) {
      loc$data$ld <- ldp[match(loc$data[, labs], ldp[, snp_col]), "R2"]
    }
  } else {
    message("Obtaining LD on ", length(rslist), " SNPs. ", appendLF = FALSE)
    ldm <- mem_LDmatrix(rslist, pop = pop, r2d = r2d, token = token,
                        genome_build = genome_build, ...)
    if (index_snp %in% colnames(ldm)) {
      ld <- ldm[, index_snp]
      loc$data$ld <- ld[match(loc$data[, labs], ldm$RS_number)]
    } else message("Index SNP not found in LDlink data")
  }
  end <- Sys.time()
  m <- sum(!is.na(loc$data$ld))
  message("Matched ", m, " SNPs (", format(end - start, digits = 3),")")
  
  loc
}


# use memoise to reduce calls to LDlink API
mem_LDmatrix <- memoise(LDlinkR::LDmatrix)

mem_LDproxy <- memoise(LDlinkR::LDproxy)

#' Layout multiple locus plots
#'
#' Produces pages with multiple locus plots on.
#'
#' @param plots Either an 'expression' to be evaluated which is a series of
#'   calls to [locus_plot()] or similar plotting functions, or a list of 'locus'
#'   class objects which are plotted in sequence.
#' @param nrow Number of rows of plots
#' @param ncol Number of columns of plots
#' @param heights Vector of length 2 specifying height for plot and gene tracks
#' @param legend_pos A keyword either "topleft" or "topright" or `NULL` to hide
#'   the legend. Not invoked if `plots` is an expression. The legend is only
#'   shown on one plot on each page.
#' @param ... Optional arguments passed to [locus_plot()] if `plots` contains a
#'   list
#' @return No return value.
#' @seealso [locus_plot()]
#' @examples
#' if(require(EnsDb.Hsapiens.v75)) {
#' 
#' data(SLE_gwas_sub)
#' genes <- c("STAT4", "UBE2L3", "IRF5")
#' loclist <- lapply(genes, locus,
#'                   data = SLE_gwas_sub,
#'                   ens_db = "EnsDb.Hsapiens.v75",
#'                   LD = "r2")
#' ## produce 3 locus plots, one on each page
#' multi_layout(loclist)
#' 
#' ## place 3 locus plots in a row on a single page
#' multi_layout(loclist, ncol = 3)
#' 
#' ## full control
#' loc <- locus(SLE_gwas_sub, gene = 'STAT4', flank = 1e5, LD = "r2",
#'              ens_db = "EnsDb.Hsapiens.v75")
#' loc2 <- locus(SLE_gwas_sub, gene = 'IRF5', flank = c(7e4, 2e5), LD = "r2",
#'               ens_db = "EnsDb.Hsapiens.v75")
#' loc3 <- locus(SLE_gwas_sub, gene = 'UBE2L3', LD = "r2",
#'               ens_db = "EnsDb.Hsapiens.v75")
#' multi_layout(ncol = 3,
#'              plots = {
#'                locus_plot(loc, use_layout = FALSE, legend_pos = 'topleft')
#'                locus_plot(loc2, use_layout = FALSE, legend_pos = NULL)
#'                locus_plot(loc3, use_layout = FALSE, legend_pos = NULL)
#'              })
#' 
#' }
#' @export

multi_layout <- function(plots,
                         nrow = 1, ncol = 1, heights = c(3, 2),
                         legend_pos = "topleft",
                         ...) {
  op <- par(no.readonly = TRUE)
  xr <- seq_len(ncol) * 2
  x2 <- c(xr, xr-1)
  x3 <- rep(x2, nrow) + (rep(seq_len(nrow), each = length(x2)) -1) * length(x2)
  mat <- matrix(x3, nrow = nrow * 2, ncol = ncol, byrow = TRUE)
  graphics::layout(mat, heights = rep(heights, nrow))
  on.exit(par(op))
  if (is.list(plots)) {
    npage <- nrow * ncol
    lpos <- if (!is.null(legend_pos) && legend_pos == "topleft") {1
      } else if (nrow == 1) 0 else ncol
    lapply(seq_along(plots), function(i) {
      leg <- NULL
      if (i %% npage == lpos) leg <- legend_pos
      locus_plot(plots[[i]], use_layout = FALSE, legend_pos = leg, ...)
    })
  } else plots
  invisible()
}

#' Locus plot
#' 
#' Genomic locus plot similar to locuszoom.
#' 
#' @details 
#' This is an R version of locuszoom for generating publication ready Manhattan 
#' plots of gene loci. It references Ensembl databases for annotating genes 
#' and exons. Use [locus()] first to generate an object of class 'locus' for
#' plotting. LDlink web server can be queried using function [link_LD()] to
#' retrieve linkage disequilibrium (LD) information on the index SNP.
#' 
#' Arguments to control plotting of the gene tracks are passed onto
#' [genetracks()] and for the scatter plot are passed via `...` to
#' [scatter_plot()]. See the documentation for each of these functions for
#' details.
#' 
#' @param loc Object of class 'locus' to use for plot. See [locus()].
#' @param filter_gene_name Vector of gene names to display.
#' @param filter_gene_biotype Vector of gene biotypes to be filtered. Use
#' [ensembldb::listGenebiotypes()] to display possible biotypes. For example, 
#' `ensembldb::listGenebiotypes(EnsDb.Hsapiens.v75)`
#' @param xlab x axis title.
#' @param cex Specifies size for points.
#' @param cex.axis Specifies font size for axis numbering.
#' @param cex.lab Specifies font size for axis titles.
#' @param cex.text Font size for gene text.
#' @param use_layout Logical whether `graphics::layout` is called. Default
#'   `TRUE` is for a standard single plot. Set to `FALSE` if a more complex
#'   layout with multiple plots is required e.g. using [multi_layout()].
#' @param heights Ratio of top to bottom plot. See [graphics::layout].
#' @param showExons Logical whether to show exons or simply show whole gene as a
#'   rectangle
#' @param maxrows Specifies maximum number of rows to display in gene 
#' annotation panel.
#' @param xticks Character value of either 'top' or 'bottom' specifying 
#' whether x axis ticks and numbers are plotted on top or bottom plot window.
#' @param border Logical whether a bounding box is plotted around upper and 
#' lower plots.
#' @param gene_col Colour for gene lines.
#' @param exon_col Fill colour for exons.
#' @param exon_border Border line colour outlining exons (or genes if
#'   `showExons` is `FALSE`). Set to `NA` for no border.
#' @param text_pos Character value of either 'top' or 'left' specifying
#'   placement of gene name labels.
#' @param italics Logical whether gene text is in italics.
#' @param highlight Vector of genes to highlight.
#' @param highlight_col Single colour or vector of colours for highlighted
#'   genes.
#' @param prioritise Vector of genes to be placed first in the gene tracks.
#' @param blanks Controls handling of genes with blank names: `"fill"` replaces
#'   blank gene symbols with ensembl gene ids. `"hide"` hides genes which are
#'   missing gene symbols.
#' @param recomb_col Colour for recombination rate line if recombination rate
#'   data is present. Set to `NA` to hide the line. See [link_recomb()] to add
#'   recombination rate data.
#' @param ... Other arguments passed to [scatter_plot()] e.g. `index_snp`,
#'   `pcutoff`, `scheme`, `recomb_offset`, etc, and arguments for [plot()] e.g.
#'   `ylab`, `main`, etc to control the scatter plot.
#' @return No return value.
#' @seealso [locus()] [scatter_plot()] [genetracks()]
#' @examples
#' if(require(EnsDb.Hsapiens.v75)) {
#' data(SLE_gwas_sub)
#' loc <- locus(SLE_gwas_sub, gene = 'UBE2L3', flank = 1e5,
#'              ens_db = "EnsDb.Hsapiens.v75")
#' locus_plot(loc)
#' 
#' ## Use embedded LD information in column `r2`
#' loc2 <- locus(SLE_gwas_sub, gene = 'IRF5', flank = c(7e4, 2e5), LD = "r2",
#'               ens_db = "EnsDb.Hsapiens.v75")
#' ## Add label for index SNP
#' locus_plot(loc2, labels = "index")
#' }
#' @export

locus_plot <- function(loc,
                       filter_gene_name = NULL,
                       filter_gene_biotype = NULL,
                       xlab = NULL,
                       cex = 1,
                       cex.axis = 0.9,
                       cex.lab = 1,
                       cex.text = 0.7,
                       use_layout = TRUE,
                       heights = c(3, 2),
                       showExons = TRUE,
                       maxrows = 7,
                       xticks = 'bottom',
                       border = FALSE,
                       gene_col = ifelse(showExons, 'blue4', 'skyblue'),
                       exon_col = 'blue4',
                       exon_border = 'blue4',
                       text_pos = 'top',
                       italics = FALSE,
                       highlight = NULL,
                       highlight_col = "red",
                       prioritise = highlight,
                       blanks = 'fill',
                       recomb_col = "blue", ...) {
  if (!inherits(loc, "locus")) stop("Object of class 'locus' required")
  if (is.null(loc$data)) stop("No SNPs/data points")
  
  if (use_layout) {
    op0 <- set_layers(1, heights, rev = TRUE)
    on.exit(par(op0), add = TRUE)
  }
  
  # lower panel gene tracks at locus
  genetracks(loc, filter_gene_name, filter_gene_biotype,
             border, cex.axis, cex.lab, cex.text, gene_col, exon_col, exon_border,
             showExons, maxrows, text_pos, italics,
             xticks = (xticks == 'bottom'),
             xlab = if (xticks == 'bottom') xlab else "",
             highlight = highlight, highlight_col = highlight_col,
             prioritise = prioritise,
             blanks = blanks, showRecomb = !is.na(recomb_col))
  
  # upper panel plot points
  scatter_plot(loc, xticks = (xticks == 'top'),
               border = border, xlab = xlab,
               cex = cex, cex.axis = cex.axis, cex.lab = cex.lab,
               recomb_col = recomb_col, ...)
}

#' Locus line plot
#'
#' Produces a line plot from a 'locus' class object. Intended for use with
#' [set_layers()].
#'
#' @param loc Object of class 'locus' to use for plot. See [locus].
#' @param pcutoff Cut-off for p value significance. Defaults to p = 5e-08. Set
#'   to `NULL` to disable.
#' @param xlab x axis title.
#' @param ylab y axis title.
#' @param cex.axis Specifies font size for axis numbering.
#' @param xticks Logical whether x axis numbers and axis title are plotted.
#' @param border Logical whether a bounding box is plotted around upper and
#'   lower plots.
#' @param align Logical whether set [par()] to align the plot.
#' @param ... Other arguments passed to [plot()] for the scatter plot.
#' @return No return value. Produces a scatter plot using base graphics.
#' @seealso [locus()] [set_layers()] [scatter_plot()]
#' @export
#' 
line_plot <- function(loc,
                      pcutoff = 5e-08,
                      xlab = NULL,
                      ylab = expression("-log"[10] ~ "P"),
                      cex.axis = 1,
                      xticks = FALSE,
                      border = FALSE,
                      align = TRUE, ...) {
  if (!inherits(loc, "locus")) stop("Object of class 'locus' required")
  if (is.null(loc$data)) stop("No data points, only gene tracks")
  data <- loc$data
  if (is.null(xlab)) xlab <- paste("Chromosome", loc$seqname, "(Mb)")
  
  # line plot
  if (align) {
    op <- par(mar = c(ifelse(xticks, 3, 0.1), 3.5, 2, 1.5))
    on.exit(par(op))
  }
  
  if (!is.null(pcutoff)) {
    abl <- quote(abline(h = -log10(pcutoff), col = 'darkgrey', lty = 2))
  } else abl <- NULL
  
  new.args <- list(...)
  plot.args <- list(x = data[, loc$pos], y = data$logP,
                    type = "l",
                    las = 1, font.main = 1,
                    xlim = loc$xrange,
                    xlab = if (xticks) xlab else "",
                    ylab = ylab,
                    bty = if (border) 'o' else 'l',
                    cex.axis = cex.axis,
                    xaxt = 'n',
                    tcl = -0.3, 
                    mgp = c(1.7, 0.5, 0),
                    panel.first = abl)
  if (length(new.args)) plot.args[names(new.args)] <- new.args
  do.call("plot", plot.args)
  
  if (xticks) {
    axis(1, at = axTicks(1), labels = axTicks(1) / 1e6, cex.axis = cex.axis,
         mgp = c(1.7, 0.4, 0), tcl = -0.3)
  } else if (!border) {
    axis(1, at = axTicks(1), labels = FALSE, tcl = -0.3)
  }
}

#' Locus plot using ggplot2
#'
#' Genomic locus plot similar to locuszoom.
#'
#' Arguments to control plotting of the gene tracks are passed onto
#' [gg_genetracks()] and for the scatter plot are passed via `...` to
#' [gg_scatter()]. See the documentation for each of these functions for
#' details.
#'
#' @param loc Object of class 'locus' to use for plot. See [locus()].
#' @param heights Vector supplying the ratio of top to bottom plot.
#' @param filter_gene_name Vector of gene names to display.
#' @param filter_gene_biotype Vector of gene biotypes to be filtered. Use
#' [ensembldb::listGenebiotypes()] to display possible biotypes. For example, 
#' `ensembldb::listGenebiotypes(EnsDb.Hsapiens.v75)`
#' @param border Logical whether a bounding box is plotted.
#' @param cex.axis Specifies font size for axis numbering.
#' @param cex.lab Specifies font size for axis titles.
#' @param cex.text Font size for gene text.
#' @param gene_col Colour for gene lines.
#' @param exon_col Fill colour for exons.
#' @param exon_border Border line colour outlining exons (or genes if
#'   `showExons` is `FALSE`). Set to `NA` for no border.
#' @param showExons Logical whether to show exons or simply show whole gene as a
#'   rectangle. If `showExons = FALSE` colours are specified by `exon_border`
#'   for rectangle border and `gene_col` for the fill colour.
#' @param maxrows Specifies maximum number of rows to display in gene annotation
#'   panel.
#' @param text_pos Character value of either 'top' or 'left' specifying
#'   placement of gene name labels.
#' @param italics Logical whether gene text is in italics.
#' @param xticks Logical whether x axis ticks and numbers are plotted.
#' @param xlab Title for x axis. Defaults to chromosome `seqname` specified in
#'   `locus`.
#' @param highlight Vector of genes to highlight.
#' @param highlight_col Single colour or vector of colours for highlighted
#'   genes.
#' @param prioritise Vector of genes to be placed first in the gene tracks.
#' @param blanks Controls handling of genes with blank names: `"fill"` replaces
#'   blank gene symbols with ensembl gene ids. `"hide"` hides genes which are
#'   missing gene symbols.
#' @param ... Additional arguments passed to [gg_scatter()] to control
#'   the scatter plot, e.g. `pcutoff`, `scheme`, `recomb_offset` etc.
#' @return Returns a ggplot2 plot containing a scatter plot with genetracks
#'   underneath.
#' @seealso [gg_scatter()] [gg_genetracks()]
#' @examples
#' if(require(EnsDb.Hsapiens.v75)) {
#' data(SLE_gwas_sub)
#' loc <- locus(SLE_gwas_sub, gene = 'IRF5', flank = c(7e4, 2e5), LD = "r2",
#'              ens_db = "EnsDb.Hsapiens.v75")
#' locus_ggplot(loc)
#' }
#' @importFrom cowplot plot_grid
#' @export

locus_ggplot <- function(loc, heights = c(3, 2),
                         filter_gene_name = NULL,
                         filter_gene_biotype = NULL,
                         border = FALSE,
                         cex.axis = 1,
                         cex.lab = 1,
                         cex.text = 0.7,
                         gene_col = ifelse(showExons, 'blue4', 'skyblue'),
                         exon_col = 'blue4',
                         exon_border = 'blue4',
                         showExons = TRUE,
                         maxrows = 12,
                         text_pos = 'top',
                         italics = FALSE,
                         xticks = "top",
                         xlab = NULL,
                         highlight = NULL,
                         highlight_col = "red",
                         prioritise = highlight,
                         blanks = "fill",
                         ...) {
  if (!inherits(loc, "locus")) stop("Object of class 'locus' required")
  if (is.null(loc$data)) stop("No SNPs/data points")
  p <- gg_scatter(loc,
                  cex.axis = cex.axis,
                  cex.lab = cex.lab,
                  xlab = xlab,
                  xticks = (xticks == "top"),
                  border = border, ...)
  g <- gg_genetracks(loc,
                     filter_gene_name, filter_gene_biotype,
                     border,
                     cex.axis, cex.lab, cex.text,
                     gene_col, exon_col, exon_border,
                     showExons,
                     maxrows, text_pos, italics,
                     xticks = (xticks != "top"), xlab,
                     highlight, highlight_col, prioritise,
                     blanks)

  plot_grid(p, g, nrow = 2, rel_heights = heights, align = "v")
}

#' Create gene tracks grob
#' 
#' Plot gene annotation tracks from `ensembldb` data using the grid package to
#' create a grob.
#' 
#' @details This function is called by [gg_genetracks()]. It can be used to
#'   generate a grob of the gene annotation tracks on their own.
#' @param locus Object of class 'locus' generated by [locus()].
#' @param filter_gene_name Vector of gene names to display.
#' @param filter_gene_biotype Vector of gene biotypes to be filtered. Use
#' [ensembldb::listGenebiotypes()] to display possible biotypes. For example, 
#' `ensembldb::listGenebiotypes(EnsDb.Hsapiens.v75)`
#' @param cex.text Font size for gene text.
#' @param showExons Logical whether to show exons or simply show whole gene as a
#'   rectangle. If `showExons = FALSE` colours are specified by `exon_border`
#'   for rectangle border and `gene_col` for the fill colour.
#' @param maxrows Specifies maximum number of rows to display in gene 
#' annotation panel.
#' @param border Logical whether a bounding box is plotted.
#' @param gene_col Colour for gene lines.
#' @param exon_col Fill colour for exons.
#' @param exon_border Border line colour outlining exons (or genes if
#'   `showExons` is `FALSE`). Set to `NA` for no border.
#' @param text_pos Character value of either 'top' or 'left' specifying 
#' placement of gene name labels.
#' @param italics Logical whether gene text is in italics.
#' @param highlight Vector of genes to highlight.
#' @param highlight_col Single colour or vector of colours for highlighted
#'   genes.
#' @param prioritise Vector of genes to be placed first in the gene tracks.
#' @param blanks Controls handling of genes with blank names: `"fill"` replaces
#'   blank gene symbols with ensembl gene ids. `"hide"` hides genes which are
#'   missing gene symbols.
#' @return A grob object.
#' @examples
#' if(require(EnsDb.Hsapiens.v75)) {
#' data(SLE_gwas_sub)
#' loc <- locus(SLE_gwas_sub, gene = 'IRF5', flank = c(7e4, 2e5), LD = "r2",
#'              ens_db = "EnsDb.Hsapiens.v75")
#' g <- genetracks_grob(loc)
#' grid::grid.newpage()
#' grid::grid.draw(g)
#' }
#' @importFrom grid viewport rectGrob textGrob xaxisGrob gList gTree gpar polylineGrob
#' @export

genetracks_grob <- function(locus,
                            filter_gene_name = NULL,
                            filter_gene_biotype = NULL,
                            border = FALSE,
                            cex.text = 0.7,
                            gene_col = ifelse(showExons, 'blue4', 'skyblue'),
                            exon_col = 'blue4',
                            exon_border = 'blue4',
                            showExons = TRUE,
                            maxrows = NULL,
                            text_pos = 'top',
                            italics = FALSE,
                            highlight = NULL,
                            highlight_col = "red",
                            prioritise = highlight,
                            blanks = c("fill", "hide")) {
  if (!inherits(locus, "locus")) stop("Object of class 'locus' required")
  blanks <- match.arg(blanks)
  TX <- locus$TX
  EX <- locus$EX
  xrange <- locus$xrange
  if (!is.null(filter_gene_name)) {
    TX <- TX[TX$gene_name %in% filter_gene_name, ]
  }
  if (!is.null(filter_gene_biotype)) {
    TX <- TX[TX$gene_biotype %in% filter_gene_biotype, ]
  }
  if (nrow(TX) == 0) {
    message('No genes to plot')
    return(invisible(NULL))
  }
  
  TX <- gene_colours(TX, gene_col, exon_col, exon_border, showExons,
                     highlight, highlight_col)
  
  TX <- mapRow(TX, xlim = xrange, cex.text = cex.text, text_pos = text_pos,
               blanks = blanks, prioritise = prioritise)
  maxrows <- if (is.null(maxrows)) max(TX$row) else min(c(max(TX$row), maxrows))
  if (max(TX$row) > maxrows) message(max(TX$row), " tracks needed to show all genes")
  TX <- TX[TX$row <= maxrows, ]
    
  ylim <- c(-maxrows - 0.3, -0.3)
  xrange <- xrange / 1e6
  TX$start <- TX$start / 1e6
  TX$end <- TX$end / 1e6
  TX[, c("mean", "tmin", "tmax", "min", "max")] <- TX[, c("mean", "tmin", "tmax", "min", "max")] / 1e6
  exheight <- switch(text_pos, "top" = 0.15, "left" = 0.3)
  
  gt <- gTree(
    childrenvp = genetrack.vp(xrange, ylim),
    children = gList(
      if (border) rectGrob(gp = gpar(lwd = 1.5), vp = "genetrack"),
      exonGrob(TX, EX, showExons, exheight),
      genetextGrob(text_pos, TX, xrange, cex.text, italics)),
    gp = gpar()
  )
  
  gt
}


genetrack.vp <- function(xrange, ylim) {
  viewport(name = "genetrack",
           x = unit(0, "lines"),
           y = unit(0, "lines"),
           width = unit(1, "npc"),
           height = unit(1, "npc") - unit(0, "lines"),
           just = c("left", "bottom"),
           xscale = xrange + c(-0.04, 0.04) * diff(xrange),
           yscale = ylim)
}


exonGrob <- function(TX, EX, showExons, exheight) {
  if (showExons) {
    LX <- unlist(t(TX[, c('start', 'end')]))
    LY <- cbind(-TX[, 'row'], -TX[, 'row'])
    LY <- unlist(t(LY))
    line_id <- rep(seq_len(nrow(TX)), each = 2)
    
    EXset <- lapply(seq_len(nrow(TX)), function(i) {
      e <- EX[EX$gene_id == TX$gene_id[i], ]
      exstart <- start(e) / 1e6
      exwidth <- end(e) / 1e6 - exstart
      data.frame(x = exstart,
                 y = -TX[i, 'row'] - exheight,
                 width = exwidth,
                 height = 2 * exheight,
                 exon_col = TX$exon_col[i],
                 exon_border = TX$exon_border[i])
    })
    EXset <- do.call("rbind", EXset)
    
    gList(
      polylineGrob(unit(LX, "native"),
                   unit(LY, "native"),
                   id = line_id,
                   gp = gpar(col = TX$gene_col, lwd = 1.5, lineend = "butt"),
                   vp = "genetrack"),
      rectGrob(x = unit(EXset[, "x"], "native"),
               y = unit(EXset[, "y"], "native"),
               width = unit(EXset[, "width"], "native"),
               height = unit(EXset[, "height"], "native"),
               just = c("left", "bottom"),
               gp = gpar(fill = EXset$exon_col, col = EXset$exon_border,
                         lwd = 0.5, lineend = "square", linejoin = "mitre"),
               vp = "genetrack")
    )
  } else {
    # without exons
    rectGrob(x = unit(TX$start, "native"),
             y = unit(-TX[, 'row'] - exheight, "native"),
             width = unit(TX$end - TX$start, "native"),
             height = unit(exheight*2, "native"),
             just = c("left", "bottom"),
             gp = gpar(fill = TX$gene_col, col = TX$exon_border,
                       lineend = "square", linejoin = "mitre"),
             vp = "genetrack")
  }
}


genetextGrob <- function(text_pos, TX, xrange, cex.text, italics) {
  if (text_pos == "top") {
    tfilter <- which(TX$tmin >= (xrange[1] - diff(xrange) * 0.04) & 
                       (TX$tmax <= xrange[2] + diff(xrange) * 0.04))
    tg <- lapply(tfilter, function(i) {
      textGrob(label = bquote_gene(TX$gene_name[i], TX$strand[i], italics),
      x = unit(TX$mean[i], "native"),
      y = unit(-TX$row[i] + 0.45, "native"),
      gp = gpar(cex = cex.text), vp = "genetrack")
    })
  } else if (text_pos == "left") {
    tfilter <- which(TX$tmin >= xrange[1])
    tg <- lapply(tfilter, function(i) {
      textGrob(label = bquote_gene(TX$gene_name[i], TX$strand[i], italics),
      x = unit(pmax(TX$start[i],
                    xrange[1] - diff(xrange) * 0.04) - diff(xrange) * 0.01,
               "native"),
      y = unit(-TX$row[i], "native"),
      just = "right",
      gp = gpar(cex = cex.text), vp = "genetrack")
    })
  }
  do.call(gList, tg)
}

#' Locus scatter plot using ggplot2
#'
#' Produces a scatter plot from a 'locus' class object (without gene tracks).
#'
#' @param loc Object of class 'locus' to use for plot. See [locus].
#' @param index_snp Specifies index SNP to be shown in a different colour and
#'   symbol. Defaults to the SNP with the lowest p-value. Set to `NULL` to not
#'   show this.
#' @param pcutoff Cut-off for p value significance. Defaults to p = 5e-08. Set
#'   to `NULL` to disable.
#' @param scheme Vector of 3 colours if LD is not shown: 1st = normal points,
#'   2nd = colour for significant points, 3rd = index SNP.
#' @param size Specifies size for points.
#' @param cex.axis Specifies font size for axis numbering.
#' @param cex.lab Specifies font size for axis titles.
#' @param xlab x axis title.
#' @param ylab y axis title.
#' @param ylim y axis limits (y1, y2).
#' @param ylim2 Secondary y axis limits for recombination line.
#' @param yzero Logical whether to force y axis limit to include y=0.
#' @param xticks Logical whether x axis numbers and axis title are plotted.
#' @param border Logical whether a bounding box is plotted around the plot.
#' @param showLD Logical whether to show LD with colours
#' @param LD_scheme Vector of colours for plotting LD. The first colour is for SNPs
#'   which lack LD information. The next 5 colours are for r2 or D' LD results
#'   ranging from 0 to 1 in intervals of 0.2. The final colour is for the index
#'   SNP.
#' @param recomb_col Colour for recombination rate line if recombination rate
#'   data is present. Set to NA to hide the line. See [link_recomb()] to add
#'   recombination rate data.
#' @param recomb_linewidth Line width for recombination rate line.
#' @param recomb_offset Offset from 0-1 which shifts the scatter plot up and
#'   recombination line plot down. Recommended value 0.1.
#' @param legend_pos Position of legend. Set to `NULL` to hide legend.
#' @param labels Character vector of SNP or genomic feature IDs to label. The
#'   value "index" selects the highest point or index SNP as defined when
#'   [locus()] is called. Set to `NULL` to remove all labels.
#' @param eqtl_gene Optional column name in `loc$data` for colouring eQTL genes.
#' @param beta Optional column name for beta coefficient to display upward
#'   triangles for positive beta and downward triangles for negative beta
#'   (significant SNPs only).
#' @param shape Optional column name in `loc$data` for controlling shapes.
#'   `beta` and `shape` cannot both be set. This column is expected to be a factor.
#' @param shape_values Vector of shape values which match levels of the column
#'   specified by `shape`. This vector is passed to
#'   `ggplot2::scale_shape_manual()` as the argument `values`. See [points()]
#'   for a list of shapes and the numbers they map to.
#' @param ... Optional arguments passed to `geom_text_repel()` to configure
#'   label drawing.
#' @return Returns a ggplot2 plot.
#' @details
#' If recombination rate data is included in the locus object following a call
#' to [link_recomb()], this is plotted as an additional line with a secondary y
#' axis. In the base graphics version the line is placed under the scatter
#' points, but this is not possible with ggplot2 as the secondary y axis data
#' must be plotted on top of the primary scatter point data.
#'
#' @seealso [locus()] [gg_addgenes()]
#' @examples
#' if (require(EnsDb.Hsapiens.v75)) {
#'     data(SLE_gwas_sub)
#'     loc <- locus(SLE_gwas_sub,
#'         gene = "IRF5", flank = c(7e4, 2e5), LD = "r2",
#'         ens_db = "EnsDb.Hsapiens.v75"
#'     )
#'     gg_scatter(loc)
#' }
#' @importFrom ggplot2 ggplot geom_point xlim ylim labs theme_classic theme scale_fill_manual scale_color_manual aes guide_legend element_text
#' @importFrom ggplot2 element_blank element_rect unit geom_hline scale_y_continuous sec_axis geom_line scale_shape_manual guides
#' @importFrom ggrepel geom_text_repel
#' @importFrom dplyr bind_rows
#' @importFrom rlang .data
#' @importFrom zoo na.approx
#' @export
#'
gg_scatter <- function(loc,
                       index_snp = loc$index_snp,
                       pcutoff = 5e-08,
                       scheme = c("grey", "dodgerblue", "red"),
                       size = 2,
                       cex.axis = 1,
                       cex.lab = 1,
                       xlab = NULL,
                       ylab = NULL,
                       ylim = NULL,
                       ylim2 = c(0, 100),
                       yzero = (loc$yvar == "logP"),
                       xticks = TRUE,
                       border = FALSE,
                       showLD = TRUE,
                       LD_scheme = c(
                           "grey", "royalblue", "cyan2", "green3",
                           "orange", "red", "purple"
                       ),
                       recomb_col = "blue",
                       recomb_linewidth = 0.35,
                       recomb_offset = 0,
                       legend_pos = "topleft",
                       labels = NULL,
                       eqtl_gene = NULL,
                       beta = NULL,
                       shape = NULL,
                       shape_values = c(21, 24, 25), ...) {
    if (!inherits(loc, "locus")) stop("Object of class 'locus' required")
    if (is.null(loc$data)) stop("No data points, only gene tracks")

    .call <- match.call()
    data <- loc$data
    if (is.null(xlab) & xticks) xlab <- paste("Chromosome", loc$seqname, "(Mb)")
    if (is.null(ylab)) {
        ylab <- if (loc$yvar == "logP") expression("-log"[10] ~ "P") else loc$yvar
    }
    hasLD <- "ld" %in% colnames(data)
    if (!"bg" %in% colnames(data)) {
        if (showLD & hasLD) {
            data$ld[data$ld == 0] <- 1e-6
            data$bg <- cut(data$ld, -1:6 / 5, labels = FALSE)
            data$bg[is.na(data$bg)] <- 1L
            data$bg[data[, loc$labs] %in% index_snp] <- 7L
            data$bg <- factor(data$bg, levels = 1:7)
            data <- data[order(data$bg), ]
            scheme <- rep_len(LD_scheme, 7)
            if (is.null(index_snp)) {
                scheme <- scheme[1:6]
                data$bg <- factor(data$bg, levels = 1:6)
            }
        } else if (!is.null(eqtl_gene)) {
            # eqtl gene colours
            bg <- data[, eqtl_gene]
            bg[data[, loc$p] > pcutoff] <- "ns"
            bg <- relevel(factor(bg, levels = unique(bg)), "ns")
            if (is.null(.call$scheme)) scheme <- eqtl_scheme(nlevels(bg))
            data$bg <- bg
        } else {
            data$bg <- scheme[1]
            if (loc$yvar == "logP") data$bg[data[, loc$p] < pcutoff] <- scheme[2]
            data$bg[data[, loc$labs] %in% index_snp] <- scheme[3]
            data$bg <- factor(data$bg, levels = scheme)
        }
    }

    # scatter plot
    if (!"col" %in% colnames(data)) data$col <- "black"
    data$col <- as.factor(data$col)

    # shapes
    if (!is.null(shape)) {
        if (!is.null(beta)) stop("cannot set both `shape` and `beta`")
        if (!shape %in% colnames(data)) stop("incorrect column name for `shape`")
        shape_breaks <- shape_labels <- levels(data[, shape])
    }
    if (!is.null(beta)) {
        # beta symbols
        data[, beta] <- signif(data[, beta], 3)
        symbol <- as.character(sign(data[, beta]))
        ind <- data[, loc$p] > pcutoff
        symbol[ind] <- "ns"
        data$.beta <- factor(symbol,
            levels = c("ns", "1", "-1"),
            labels = c("ns", "up", "down")
        )
        shape <- ".beta"
        shape_breaks <- c("ns", "up", "down")
        shape_labels <- c("ns", expression({
            beta > 0
        }), expression({
            beta < 0
        }))
    }

    # legend
    legend.justification <- NULL
    legend_labels <- legend_title <- NULL
    legend.position <- "none"
    if (!is.null(legend_pos)) {
        if (legend_pos == "topleft") {
            legend.justification <- c(0, 1)
            legend.position <- c(0.01, 0.99)
        } else if (legend_pos == "topright") {
            legend.justification <- c(1, 1)
            legend.position <- c(0.99, 0.99)
        } else {
            legend.position <- legend_pos
        }
        if (showLD & hasLD) {
            legend_title <- expression({
                r^2
            })
            legend_labels <- rev(c(
                "Index SNP", "0.8 - 1.0", "0.6 - 0.8", "0.4 - 0.6",
                "0.2 - 0.4", "0.0 - 0.2", "NA"
            ))
            if (is.null(index_snp)) legend_labels <- legend_labels[1:6]
        } else if (!is.null(eqtl_gene)) {
            legend_labels <- levels(bg)
        } else if (is.null(beta) & is.null(shape)) legend.position <- "none"
    }

    yrange <- if (is.null(ylim)) range(data[, loc$yvar], na.rm = TRUE) else ylim
    if (is.null(ylim) && yzero) yrange[1] <- min(c(0, yrange[1]))
    ycut <- -log10(pcutoff)

    # recombination line
    recomb <- !is.null(loc$recomb) & !is.na(recomb_col)
    if (recomb) {
        df <- loc$recomb[, c("start", "value")]
        colnames(df) <- c(loc$pos, "recomb")
        data <- dplyr::bind_rows(data, df)
        data <- data[order(data[, loc$pos]), ]
        data$recomb <- zoo::na.approx(data$recomb, data[, loc$pos], na.rm = FALSE)
        ymult <- 100 / diff(yrange)
        yd <- diff(yrange)
        yd2 <- diff(ylim2)
        yrange0 <- yrange
        yrange[1] <- yrange[1] - yd * recomb_offset
        outside <- df$recomb < ylim2[1] | df$recomb > (ylim2[2] + yd2 * recomb_offset)
        if (any(outside)) {
            nmessage(sum(outside), " recombination value(s) outside scale range (`ylim2`)")
        }
        fy2 <- function(yy) (yy - ylim2[1]) / yd2 * yd + yrange[1]
        inv_fy2 <- function(yy) (yy - yrange[1]) / yd * yd2 + ylim2[1]
    }
    outside <- loc$data[, loc$yvar] < yrange[1] | loc$data[, loc$yvar] > yrange[2]
    if (any(outside)) {
        nmessage(sum(outside), " value(s) outside scale range (`ylim`)")
    }
    data[, loc$pos] <- data[, loc$pos] / 1e6

    # add labels
    if (!is.null(labels)) {
        i <- grep("index", labels, ignore.case = TRUE)
        if (length(i) > 0) {
            if (length(index_snp) == 1) {
                labels[i] <- index_snp
            } else {
                labels <- labels[-i]
                labels <- c(index_snp, labels)
            }
        }
        text_label_ind <- match(labels, data[, loc$labs])
        if (any(is.na(text_label_ind))) {
            message(
                "label ", paste(labels[is.na(text_label_ind)], collapse = ", "),
                " not found"
            )
        }
    }
    ind <- data[, loc$labs] %in% index_snp

    if (!recomb) {
        if (is.null(shape)) {
            # standard plot
            p <- ggplot(data[!ind, ], aes(
                x = .data[[loc$pos]], y = .data[[loc$yvar]],
                color = .data$col, fill = .data$bg
            )) +
                (if (loc$yvar == "logP" & !is.null(pcutoff) &
                    ycut >= yrange[1] & ycut <= yrange[2]) {
                    geom_hline(
                        yintercept = ycut,
                        colour = "grey", linetype = "dashed"
                    )
                }) +
                geom_point(shape = 21, size = size, show.legend = TRUE) +
                # index SNP
                (if (any(ind)) {
                    geom_point(
                        data = data[ind, ],
                        aes(
                            y = .data[[loc$yvar]], color = .data$col,
                            fill = .data$bg
                        ),
                        shape = 23, size = size
                    )
                })
        } else {
            # shapes or beta triangles
            p <- ggplot(data, aes(
                x = .data[[loc$pos]], y = .data[[loc$yvar]],
                color = .data$col, fill = .data$bg,
                shape = .data[[shape]]
            )) +
                (if (loc$yvar == "logP" & !is.null(pcutoff) &
                    ycut >= yrange[1] & ycut <= yrange[2]) {
                    geom_hline(
                        yintercept = ycut,
                        colour = "grey", linetype = "dashed"
                    )
                }) +
                geom_point(size = size) +
                scale_shape_manual(
                    values = shape_values, name = NULL,
                    breaks = shape_breaks,
                    labels = shape_labels
                ) +
                (if (showLD & hasLD) {
                    guides(fill = guide_legend(
                        override.aes = list(shape = 21),
                        reverse = TRUE, order = 1
                    ))
                } else {
                    guides(fill = "none")
                })
        }
        p <- p +
            scale_fill_manual(
                breaks = levels(data$bg), values = scheme,
                guide = guide_legend(reverse = TRUE),
                drop = FALSE,
                labels = legend_labels, name = legend_title
            ) +
            scale_color_manual(
                breaks = levels(data$col), values = levels(data$col),
                guide = "none"
            ) +
            # scale_shape_manual(breaks = levels(data$pch), values = levels(data$pch)) +
            xlim(loc$xrange[1] / 1e6, loc$xrange[2] / 1e6) + ylim(yrange) +
            labs(x = xlab, y = ylab) +
            theme_classic() +
            theme(
                axis.text = element_text(colour = "black", size = 10 * cex.axis),
                axis.title = element_text(size = 10 * cex.lab),
                legend.justification = legend.justification,
                legend.position = legend.position,
                legend.title.align = 0.5,
                legend.text.align = 0,
                legend.key.size = unit(0.9, "lines"),
                legend.spacing.y = unit(0, "lines")
            ) +
            if (!xticks) {
                theme(
                    axis.text.x = element_blank(),
                    axis.ticks.x = element_blank()
                )
            }
    } else {
        # recombination plot with dual y axis
        if (is.null(shape)) {
            # standard plot
            p <- ggplot(data[!ind, ], aes(x = .data[[loc$pos]])) +
                (if (loc$yvar == "logP" & !is.null(pcutoff) &
                    ycut >= yrange[1] & ycut <= yrange[2]) {
                    geom_hline(
                        yintercept = ycut,
                        colour = "grey", linetype = "dashed"
                    )
                }) +
                geom_point(aes(
                    y = .data[[loc$yvar]], color = .data$col,
                    fill = .data$bg
                ), shape = 21, size = size, na.rm = TRUE) +
                # index SNP
                (if (any(ind)) {
                    geom_point(
                        data = data[ind, ],
                        aes(
                            y = .data[[loc$yvar]], color = .data$col,
                            fill = .data$bg
                        ), shape = 23, size = size, na.rm = TRUE
                    )
                })
        } else {
            # shapes or beta triangles
            p <- ggplot(data, aes(x = .data[[loc$pos]])) +
                (if (loc$yvar == "logP" & !is.null(pcutoff) &
                    ycut >= yrange[1] & ycut <= yrange[2]) {
                    geom_hline(
                        yintercept = ycut,
                        colour = "grey", linetype = "dashed"
                    )
                }) +
                geom_point(aes(
                    y = .data[[loc$yvar]], color = .data$col, fill = .data$bg,
                    shape = .data[[shape]]
                ), size = size, na.rm = TRUE) +
                scale_shape_manual(
                    values = shape_values, name = NULL,
                    breaks = shape_breaks,
                    labels = shape_labels
                ) +
                (if (showLD & hasLD) {
                    guides(fill = guide_legend(
                        override.aes = list(shape = 21),
                        reverse = TRUE, order = 1
                    ))
                } else {
                    guides(fill = "none")
                })
        }
        p <- p +
            scale_fill_manual(
                breaks = levels(data$bg), values = scheme,
                guide = guide_legend(reverse = TRUE),
                labels = legend_labels, name = legend_title
            ) +
            scale_color_manual(
                breaks = levels(data$col), values = levels(data$col),
                guide = "none"
            ) +
            geom_line(aes(y = fy2(.data$recomb)),
                color = recomb_col, linewidth = recomb_linewidth,
                na.rm = TRUE
            ) +
            scale_y_continuous(
                name = ylab,
                limits = yrange, breaks = pretty(yrange0),
                sec.axis = sec_axis(inv_fy2,
                    name = "Recombination rate (%)",
                    breaks = pretty(ylim2)
                )
            ) +
            xlim(loc$xrange[1] / 1e6, loc$xrange[2] / 1e6) +
            xlab(xlab) +
            theme_classic() +
            theme(
                axis.text = element_text(colour = "black", size = 10 * cex.axis),
                axis.title = element_text(size = 10 * cex.lab),
                legend.justification = legend.justification,
                legend.position = legend.position,
                legend.title.align = 0.5,
                legend.text.align = 0,
                legend.key.size = unit(0.9, "lines"),
                legend.spacing.y = unit(0, "lines")
            ) +
            (if (recomb_offset != 0) {
                theme(
                    axis.title.y.left = element_text(
                        hjust = min(c(0.5 + recomb_offset / 3, 0.9))
                    ),
                    axis.title.y.right = element_text(
                        hjust = min(c(0.5 + recomb_offset / 2, 1))
                    )
                )
            }) +
            if (!xticks) {
                theme(
                    axis.text.x = element_blank(),
                    axis.ticks.x = element_blank()
                )
            }
    }

    if (!is.null(labels)) {
        p <- p +
            geom_text_repel(
                data = data[text_label_ind, ],
                mapping = aes(
                    x = .data[[loc$pos]], y = .data[[loc$yvar]],
                    label = .data[[loc$labs]]
                ),
                point.size = size, ...
            )
    }

    if (border | recomb) {
        p <- p + theme(panel.border = element_rect(colour = "black", fill = NA))
    }
    p
}


nmessage <- function(...) {
    argList <- list(...)
    n_arg <- which(sapply(argList, class) %in% c("numeric", "integer"))
    n <- argList[[n_arg]]
    if (n == 0) {
        return()
    }
    msg <- paste0(argList)
    msg <- if (n == 1) gsub("\\(s\\)", "", msg) else gsub("\\(s\\)", "s", msg)
    message(msg)
}

#' Add gene tracks to a ggplot2 plot
#'
#' Adds gene tracks to an existing ggplot2 plot.
#'
#' @param p ggplot2 plot object. This can be generated by [gg_scatter()] and
#'   then modified.
#' @param loc Object of class 'locus' to use for plot. See [locus()].
#' @param heights Vector specifying ratio of heights of upper plot and lower
#'   gene track.
#' @param ... Additional arguments passed to [gg_genetracks()] to control
#'   colours of gene tracks etc.
#' @return A ggplot2 plotting object.
#' @seealso [gg_scatter()] [gg_genetracks()]
#' @examples
#' if(require(EnsDb.Hsapiens.v75)) {
#' data(SLE_gwas_sub)
#' loc <- locus(SLE_gwas_sub, gene = 'IRF5', flank = c(7e4, 2e5), LD = "r2",
#'              ens_db = "EnsDb.Hsapiens.v75")
#' p <- gg_scatter(loc)
#' gg_addgenes(p, loc)
#' }
#' @importFrom ggplot2 layer_scales
#' @importFrom cowplot plot_grid
#' @export

gg_addgenes <- function(p, loc,
                        heights = c(3, 2),
                        ...) {
  # check x axis range
  xl <- layer_scales(p)$x$get_limits() * 1e6
  if (!identical(as.numeric(loc$xrange), xl)) {
    message("Plot x axis limits and locus range differ: check correct locus?")
    message("x axis limits: ", xl[1], " to ", xl[2])
    message("locus range: ", loc$xrange[1], " to ", loc$xrange[2])
  }
  g <- gg_genetracks(loc, ...)
  
  plot_grid(p, g, nrow = 2, rel_heights = heights, align = "v")
}

#' Gene tracks using 'plotly'
#' 
#' Plot gene annotation tracks from `ensembldb` data using `plotly`.
#' 
#' @details This function can used to plot gene annotation tracks on their own.
#' @param locus Object of class 'locus' generated by [locus()].
#' @param filter_gene_name Vector of gene names to display.
#' @param filter_gene_biotype Vector of gene biotypes to be filtered. Use
#' [ensembldb::listGenebiotypes()] to display possible biotypes. For example, 
#' `ensembldb::listGenebiotypes(EnsDb.Hsapiens.v75)`
#' @param cex.text Font size for gene text.
#' @param italics Logical whether gene text is in italics.
#' @param gene_col Colour for gene lines.
#' @param exon_col Fill colour for exons.
#' @param exon_border Border line colour outlining exons (or genes if
#'   `showExons` is `FALSE`). Set to `NA` for no border.
#' @param showExons Logical whether to show exons or simply show whole gene as a
#'   rectangle. If `showExons = FALSE` colours are specified by `exon_border`
#'   for rectangle border and `gene_col` for the fill colour.
#' @param maxrows Specifies maximum number of rows to display in gene 
#' annotation panel.
#' @param width Width of plotly plot in pixels which is purely used to prevent
#'   overlapping text for gene names.
#' @param xlab Title for x axis. Defaults to chromosome `seqname` specified 
#' in `locus`.
#' @param prioritise Vector of genes to be placed first in the gene tracks.
#' @param blanks Controls handling of genes with blank names: `"fill"` replaces
#'   blank gene symbols with ensembl gene ids. `"hide"` completely hides genes
#'   which are missing gene symbols. `"show"` shows gene lines but no label
#'   (hovertext is still available).
#' @param height Height in pixels (optional, defaults to automatic sizing).
#' @param plot Logical whether to produce plotly object or return plot
#'   coordinates.
#' @return Either a 'plotly' plotting object showing gene tracks, or if 
#'   `plot = FALSE` a list containing `TX`, a dataframe of coordinates for
#'   gene transcripts, and `EX`, a dataframe of coordinates for exons.
#' @examples
#' if(require(EnsDb.Hsapiens.v75)) {
#' data(SLE_gwas_sub)
#' loc <- locus(SLE_gwas_sub, gene = 'UBE2L3', flank = 1e5,
#'              ens_db = "EnsDb.Hsapiens.v75")
#' genetrack_ly(loc)
#' }
#' @importFrom plotly plot_ly plotly_empty add_segments add_text %>%
#' @export

genetrack_ly <- function(locus,
                         filter_gene_name = NULL,
                         filter_gene_biotype = NULL,
                         cex.text = 0.7,
                         italics = FALSE,
                         gene_col = ifelse(showExons, 'blue4', 'skyblue'),
                         exon_col = 'blue4',
                         exon_border = 'blue4',
                         showExons = TRUE,
                         maxrows = 8,
                         width = 600,
                         xlab = NULL,
                         prioritise = NULL,
                         blanks = c("fill", "hide", "show"),
                         height = NULL,
                         plot = TRUE) {
  if (!inherits(locus, "locus")) stop("Object of class 'locus' required")
  blanks <- match.arg(blanks)
  TX <- locus$TX
  EX <- as.data.frame(locus$EX)
  xrange <- locus$xrange
  if (!is.null(filter_gene_name)) {
    TX <- TX[TX$gene_name %in% filter_gene_name, ]
  }
  if (!is.null(filter_gene_biotype)) {
    TX <- TX[TX$gene_biotype %in% filter_gene_biotype, ]
  }
  xlim <- xrange / 1e6
  xext <- diff(xlim) * 0.01
  xlim <- xlim + c(-xext, xext)
  if (is.null(xlab)) xlab <- paste("Chromosome", locus$seqname, "(Mb)")
  if (nrow(TX) == 0 & plot) {
    message('No genes to plot')
    # blank gene tracks
    p <- plot_ly(data.frame(NA), mode = "markers", type = "scatter",
                 source = "plotly_locus") %>%
      plotly::layout(xaxis = list(title = xlab, showgrid = FALSE, showline = TRUE,
                                  color = 'black', ticklen = 5,
                                  range = as.list(xlim)),
                     yaxis = list(title = "", showgrid = FALSE, zeroline = FALSE,
                                  showticklabels = FALSE)) %>%
      plotly::config(displaylogo = FALSE)
    return(p)
  }
  
  cex.width <- cex.text * par("pin")[1] * 80 / (width - 250)
  TX <- mapRow(TX, xlim = xrange, cex.text = cex.width, blanks = blanks,
               prioritise = prioritise)
  maxrows <- if (is.null(maxrows)) max(TX$row) else min(c(max(TX$row), maxrows))
  if (max(TX$row) > maxrows) message(max(TX$row), " tracks needed to show all genes")
  TX <- TX[TX$row <= maxrows, ]
  EX <- EX[EX$gene_id %in% TX$gene_id, ]
  
  gene_col <- col2hex(gene_col)
  exon_col <- col2hex(exon_col)
  exon_border <- col2hex(exon_border)
  EX$row <- TX$row[match(EX$gene_id, TX$gene_id)]
  
  EX[, c('start', 'end')] <- EX[, c('start', 'end')] / 1e6
  TX$tx <- rowMeans(TX[, c('start', 'end')])
  TX$ty <- -TX$row + 0.35
  TX[, c('start', 'end', 'tx')] <- TX[, c('start', 'end', 'tx')] / 1e6
  
  tfilter <- TX$tmin > (xrange[1] - diff(xrange) * 0.005) & 
             (TX$tmax < xrange[2] + diff(xrange) * 0.005) &
             TX$gene_name != ""
  pos <- TX$strand == "+"
  TX$gene_name2 <- if (italics) paste0("<i>", TX$gene_name, "</i>") else TX$gene_name
  TX$gene_name2[pos] <- paste0(TX$gene_name2[pos], "&#8594;")
  TX$gene_name2[!pos] <- paste0("&#8592;", TX$gene_name2[!pos])
  TX$gene_name2[!tfilter] <- NA
  
  if (!plot) return(list(TX = TX, EX = EX))
  
  if (showExons) {
    shapes <- lapply(seq_len(nrow(EX)), function(i) {
      list(type = "rect", fillcolor = exon_col, line = list(color = exon_border,
                                                            width = 0.5),
           x0 = EX$start[i], x1 = EX$end[i], xref = "x",
           y0 = -EX$row[i] - 0.15, y1 = -EX$row[i] + 0.15, yref = "y")
    })
  } else {
    shapes <- lapply(seq_len(nrow(TX)), function(i) {
      list(type = "rect", fillcolor = gene_col, line = list(color = exon_border,
                                                            width = 1),
           x0 = TX$start[i], x1 = TX$end[i], xref = "x",
           y0 = -TX$row[i] - 0.15, y1 = -TX$row[i] + 0.15, yref = "y")
    })
  }
  
  ok <- !is.na(TX$gene_name2)
  hovertext <- paste0(TX$gene_name,
                      TX$fullname,
                      "<br>Gene ID: ", TX$gene_id,
                      "<br>Biotype: ", TX$gene_biotype,
                      "<br>Start: ", TX$start * 1e6,
                      "<br>End: ", TX$end * 1e6)
  plot_ly(TX, source = "plotly_locus", height = height) %>%
    add_segments(x = ~start, y = ~-row,
                 xend = ~end, yend = ~-row,
                 color = I(gene_col),
                 text = hovertext, hoverinfo = 'text',
                 showlegend = FALSE) %>%
    add_text(x = TX$tx[ok], y = TX$ty[ok], text = TX$gene_name2[ok],
             textfont = list(size = 14 * cex.text),
             showlegend = FALSE, hoverinfo = 'none') %>%
    plotly::layout(shapes = shapes,
                   xaxis = list(title = xlab, showgrid = FALSE, showline = TRUE,
                                zeroline = FALSE,
                                color = 'black', ticklen = 5,
                                range = as.list(xlim)),
                   yaxis = list(title = "", showgrid = FALSE, zeroline = FALSE,
                                fixedrange = TRUE,
                                showticklabels = FALSE),
                   showlegend = TRUE, dragmode = "pan") %>%
    plotly::config(displaylogo = FALSE,
                   modeBarButtonsToRemove = c("select2d", "lasso2d",
                                              "autoScale2d", "resetScale2d",
                                              "hoverClosest", "hoverCompare"))
}


#' @importFrom grDevices col2rgb rgb

col2hex <- function(cname) {
  colMat <- col2rgb(cname)
  rgb(red = colMat[1, ]/255, green = colMat[2, ]/255, blue = colMat[3, ]/255)
}

#' Plot gene tracks
#' 
#' Plot gene annotation tracks from `ensembldb` data using ggplot2 and grid.
#' 
#' @param loc Object of class 'locus' generated by [locus()].
#' @param filter_gene_name Vector of gene names to display.
#' @param filter_gene_biotype Vector of gene biotypes to be filtered. Use
#' [ensembldb::listGenebiotypes()] to display possible biotypes. For example, 
#' `ensembldb::listGenebiotypes(EnsDb.Hsapiens.v75)`
#' @param border Logical whether a bounding box is plotted.
#' @param cex.axis Specifies font size for axis numbering.
#' @param cex.lab Specifies font size for axis titles.
#' @param cex.text Font size for gene text.
#' @param gene_col Colour for gene lines.
#' @param exon_col Fill colour for exons.
#' @param exon_border Border line colour outlining exons (or genes if
#'   `showExons` is `FALSE`). Set to `NA` for no border.
#' @param showExons Logical whether to show exons or simply show whole gene as a
#'   rectangle. If `showExons = FALSE` colours are specified by `exon_border`
#'   for rectangle border and `gene_col` for the fill colour.
#' @param maxrows Specifies maximum number of rows to display in gene annotation
#'   panel.
#' @param text_pos Character value of either 'top' or 'left' specifying
#'   placement of gene name labels.
#' @param italics Logical whether gene text is in italics.
#' @param xticks Logical whether x axis ticks and numbers are plotted.
#' @param xlab Title for x axis. Defaults to chromosome `seqname` specified in
#'   `locus`.
#' @param highlight Vector of genes to highlight.
#' @param highlight_col Single colour or vector of colours for highlighted
#'   genes.
#' @param prioritise Vector of genes to be placed first in the gene tracks.
#' @param blanks Controls handling of genes with blank names: `"fill"` replaces
#'   blank gene symbols with ensembl gene ids. `"hide"` hides genes which are
#'   missing gene symbols.
#' @details
#' This function is called by [locus_ggplot()], and in turn it calls
#' [genetracks_grob()]. It can be used to plot the gene annotation tracks on
#' their own as a ggplot2 object.
#' 
#' `gene_col`, `exon_col` and `exon_border` set colours for all genes, while
#' `highlight` and `highlight_col` can optionally be used together to highlight
#' specific genes of interest. For full control over every single gene, users
#' can add columns `gene_col`, `exon_col` and `exon_border` to the `TX` object
#' within the 'locus' object. Columns added to `TX` override their equivalent 
#' arguments.
#' @seealso [locus_ggplot()] [genetracks_grob()]
#' @return A ggplot2 object.
#' @examples
#' if(require(EnsDb.Hsapiens.v75)) {
#' data(SLE_gwas_sub)
#' loc <- locus(SLE_gwas_sub, gene = 'IRF5', flank = c(7e4, 2e5), LD = "r2",
#'              ens_db = "EnsDb.Hsapiens.v75")
#' gg_genetracks(loc)
#' }
#' @importFrom ggplot2 xlab
#' @importFrom gggrid grid_panel
#' @export

gg_genetracks <- function(loc,
                          filter_gene_name = NULL,
                          filter_gene_biotype = NULL,
                          border = FALSE,
                          cex.axis = 1,
                          cex.lab = 1,
                          cex.text = 0.7,
                          gene_col = ifelse(showExons, 'blue4', 'skyblue'),
                          exon_col = 'blue4',
                          exon_border = 'blue4',
                          showExons = TRUE,
                          maxrows = NULL,
                          text_pos = 'top',
                          italics = FALSE,
                          xticks = TRUE,
                          xlab = NULL,
                          highlight = NULL,
                          highlight_col = "red",
                          prioritise = highlight,
                          blanks = c("fill", "hide")) {
  if (!inherits(loc, "locus")) stop("Object of class 'locus' required")
  blanks <- match.arg(blanks)
  g <- genetracks_grob(loc,
                       filter_gene_name,
                       filter_gene_biotype,
                       border,
                       cex.text,
                       gene_col,
                       exon_col,
                       exon_border,
                       showExons,
                       maxrows,
                       text_pos,
                       italics,
                       highlight, highlight_col, prioritise,
                       blanks)
  if (is.null(xlab) & xticks) xlab <- paste("Chromosome", loc$seqname, "(Mb)")
  
  g2 <- ggplot(data.frame(x = NA),
               aes(xmin = loc$xrange[1] / 1e6, xmax = loc$xrange[2] / 1e6)) +
    (if (!is.null(g)) gggrid::grid_panel(g)) +
    xlab(xlab) +
    theme_classic() +
    theme(axis.text = element_text(colour = "black", size = 10 * cex.axis),
          axis.title = element_text(size = 10 * cex.lab),
          axis.line.y = element_blank())
  if (!xticks) {
    g2 <- g2 +
      theme(axis.line.x = element_blank(),
            axis.ticks.x = element_blank(),
            axis.text.x = element_blank())
  }
  g2
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE
)
library(locuszoomr)
ensdb_eval <- requireNamespace("EnsDb.Hsapiens.v75", quietly = TRUE)

## ----eval=FALSE---------------------------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE))
#   install.packages("BiocManager")
# BiocManager::install("ensembldb")
# BiocManager::install("EnsDb.Hsapiens.v75")

## ----eval=FALSE---------------------------------------------------------------
# install.packages("locuszoomr")

## ----eval=FALSE---------------------------------------------------------------
# devtools::install_github("myles-lewis/locuszoomr")

## -----------------------------------------------------------------------------
library(locuszoomr)
data(SLE_gwas_sub)  ## limited subset of data from SLE GWAS
head(SLE_gwas_sub)

## ----eval=ensdb_eval, message=FALSE, fig.dim=c(8, 7)--------------------------
library(EnsDb.Hsapiens.v75)
loc <- locus(data = SLE_gwas_sub, gene = 'UBE2L3', flank = 1e5,
             ens_db = "EnsDb.Hsapiens.v75")
summary(loc)
locus_plot(loc)

## ----eval=FALSE---------------------------------------------------------------
# library(AnnotationHub)
# ah <- AnnotationHub()
# query(ah, c("EnsDb", "Homo sapiens"))
# ## AnnotationHub with 25 records
# ## # snapshotDate(): 2023-04-25
# ## # $dataprovider: Ensembl
# ## # $species: Homo sapiens
# ## # $rdataclass: EnsDb
# ## # additional mcols(): taxonomyid, genome, description, coordinate_1_based, maintainer,
# ## #   rdatadateadded, preparerclass, tags, rdatapath, sourceurl, sourcetype
# ## # retrieve records with, e.g., 'object[["AH53211"]]'
# ##
# ##              title
# ##   AH53211  | Ensembl 87 EnsDb for Homo Sapiens
# ##   ...        ...
# ##   AH100643 | Ensembl 106 EnsDb for Homo sapiens
# ##   AH104864 | Ensembl 107 EnsDb for Homo sapiens
# ##   AH109336 | Ensembl 108 EnsDb for Homo sapiens
# ##   AH109606 | Ensembl 109 EnsDb for Homo sapiens
# ##   AH113665 | Ensembl 110 EnsDb for Homo sapiens

## ----eval=FALSE---------------------------------------------------------------
# ensDb_v106 <- ah[["AH100643"]]
# 
# # built-in mini dataset
# data("SLE_gwas_sub")
# loc <- locus(data = SLE_gwas_sub, gene = 'UBE2L3', fix_window = 1e6,
#              ens_db = ensDb_v106)
# locus_plot(loc)

## ----eval=FALSE---------------------------------------------------------------
# # Locus plot using SLE GWAS data from Bentham et al 2015
# # FTP download full summary statistics from
# # https://www.ebi.ac.uk/gwas/studies/GCST003156
# library(data.table)
# SLE_gwas <- fread('../bentham_2015_26502338_sle_efo0002690_1_gwas.sumstats.tsv')
# loc <- locus(SLE_gwas, gene = 'UBE2L3', flank = 1e5,
#              ens_db = "EnsDb.Hsapiens.v75")
# loc <- link_LD(loc, token = "your_token")
# locus_plot(loc)

## ----eval=ensdb_eval, message=FALSE, fig.dim=c(8, 7)--------------------------
loc <- locus(SLE_gwas_sub, gene = 'UBE2L3', flank = 1e5, LD = "r2",
             ens_db = "EnsDb.Hsapiens.v75")
locus_plot(loc, labels = c("index", "rs140492"), label_x = c(4, -5))

## ----eval=FALSE---------------------------------------------------------------
# loc3 <- locus(SLE_gwas_sub, gene = 'STAT4', flank = 1e5, LD = "r2",
#               ens_db = "EnsDb.Hsapiens.v75")
# loc3 <- link_recomb(loc3)
# locus_plot(loc3)

## ----out.width='80%', echo=FALSE----------------------------------------------
knitr::include_graphics("recomb.png")

## ----eval=FALSE---------------------------------------------------------------
# library(rtracklayer)
# recomb.hg19 <- import.bw("/../hapMapRelease24CombinedRecombMap.bw")
# loc3 <- link_recomb(loc3, recomb = recomb.hg19)
# locus_plot(loc3)

## ----eval=FALSE---------------------------------------------------------------
# # add column 'typed' as a factor which is 1 for typed, 0 for imputed
# loc$data$typed <- factor(rbinom(n = nrow(loc$data), 1, 0.3))
# # convert column to shapes by adding a column called 'pch'
# loc$data$pch <- c(21, 24)[loc$data$typed]
# locus_plot(loc)
# # pch 21 = circles = imputed
# # pch 24 = triangles = typed

## ----eval=FALSE---------------------------------------------------------------
# # Filter by gene biotype
# locus_plot(loc, filter_gene_biotype = "protein_coding")
# 
# # Custom selection of genes using gene names
# locus_plot(loc, filter_gene_name = c('UBE2L3', 'RIMBP3C', 'YDJC', 'PPIL2',
#                                      'PI4KAP2', 'MIR301B'))

## ----eval=ensdb_eval, fig.dim=c(7, 3.5)---------------------------------------
genetracks(loc, highlight = "UBE2L3")

## ----eval=ensdb_eval, fig.dim=c(7, 2.5)---------------------------------------
# Limit the number of tracks
# Filter by gene biotype
# Customise colours
genetracks(loc, maxrows = 3, filter_gene_biotype = 'protein_coding',
           gene_col = 'grey', exon_col = 'orange', exon_border = 'darkgrey')

## ----eval=FALSE---------------------------------------------------------------
# loc00 <- locus(gene = 'UBE2L3', flank = 1e5, ens_db = "EnsDb.Hsapiens.v75")
# 
# genetracks(loc00)  # base graphics
# gg_genetracks(loc00)  # ggplot2
# genetrack_ly(loc00)  # plotly

## ----eval=FALSE---------------------------------------------------------------
# # obtain GTEx eQTL data from LDlinkR
# # needs personal access token for LDlink
# loc2 <- locus(SLE_gwas_sub, gene = 'IRF5', flank = c(7e4, 2e5), LD = "r2",
#               ens_db = "EnsDb.Hsapiens.v75")
# loc2 <- link_eqtl(loc2, token = "..")
# head(loc2$LDexp)  # show eQTL data

## ----eval=FALSE---------------------------------------------------------------
# table(loc2$LDexp$Gene_Symbol)
# ##      AHCYL2           IRF5            KCP        METTL2B           NRF1   RP11-212P7.2
# ##           5           1618              1              2              9              3
# ## ...
# 
# table(loc2$LDexp$Tissue)
# ##             Adipose - Subcutaneous          Adipose - Visceral (Omentum)
# ##                                 78                                    69
# ##                      Adrenal Gland                        Artery - Aorta
# ##                                 38                                   109
# ## ...

## ----eval=FALSE---------------------------------------------------------------
# # just plot eQTL results
# eqtl_plot(loc2, tissue = "Whole Blood", eqtl_gene = "IRF5")
# 
# # overlay eqtl plot
# overlay_plot(loc2, eqtl_gene = "IRF5")

## ----out.width='60%', fig.align="center", echo=FALSE--------------------------
knitr::include_graphics("eqtl_overlay.png")

## ----eval=ensdb_eval, message=FALSE, fig.dim=c(8, 7)--------------------------
locb <- locus(SLE_gwas_sub, gene = 'UBE2L3', flank = 1e5, yvar = "beta",
              ens_db = "EnsDb.Hsapiens.v75")
locus_plot(locb)

## ----eval=FALSE---------------------------------------------------------------
# genes <- c("STAT4", "IRF5", "UBE2L3")
# 
# # generate list of 'locus' class objects, one for each gene
# loclist <- lapply(genes, locus,
#                   data = SLE_gwas_sub,
#                   ens_db = "EnsDb.Hsapiens.v75",
#                   LD = "r2")
# 
# ## produce 3 locus plots, one on each page
# pdf("myplot.pdf")
# multi_layout(loclist)
# dev.off()
# 
# ## place 3 locus plots in a row on a single page
# pdf("myplot.pdf")
# multi_layout(loclist, ncol = 3, labels = "index")
# dev.off()
# 
# ## full control
# loc2 <- locus(SLE_gwas_sub, gene = 'IRF5', flank = c(7e4, 2e5), LD = "r2",
#               ens_db = "EnsDb.Hsapiens.v75")
# loc3 <- locus(SLE_gwas_sub, gene = 'STAT4', flank = 1e5, LD = "r2",
#               ens_db = "EnsDb.Hsapiens.v75")
# 
# pdf("myplot.pdf", width = 9, height = 6)
# multi_layout(ncol = 3,
#              plots = {
#                locus_plot(loc, use_layout = FALSE, legend_pos = 'topleft')
#                locus_plot(loc2, use_layout = FALSE, legend_pos = NULL)
#                locus_plot(loc3, use_layout = FALSE, legend_pos = NULL,
#                           labels = "index")
#              })
# dev.off()

## ----echo=FALSE, message=FALSE, fig.dim = c(9,5)------------------------------
if (require(EnsDb.Hsapiens.v75)) {
loc2 <- locus(SLE_gwas_sub, gene = 'IRF5', flank = c(7e4, 2e5), LD = "r2",
              ens_db = "EnsDb.Hsapiens.v75")
loc3 <- locus(SLE_gwas_sub, gene = 'STAT4', flank = 1e5, LD = "r2",
              ens_db = "EnsDb.Hsapiens.v75")

multi_layout(ncol = 3,
             plots = {
               locus_plot(loc, use_layout = FALSE, legend_pos = 'topleft')
               locus_plot(loc2, use_layout = FALSE, legend_pos = NULL)
               locus_plot(loc3, use_layout = FALSE, legend_pos = NULL,
                          labels = "index")
             })
}

## ----eval=FALSE---------------------------------------------------------------
# pdf("myplot2.pdf", width = 6, height = 8)
# # set up layered plot with 2 plots & a gene track; store old par() settings
# oldpar <- set_layers(2)
# scatter_plot(loc, xticks = FALSE)
# line_plot(loc, col = "orange", xticks = FALSE)
# genetracks(loc)
# par(oldpar)  # revert par() settings
# dev.off()

## ----echo=FALSE, fig.dim = c(6,8)---------------------------------------------
if (require(EnsDb.Hsapiens.v75)) {
oldpar <- set_layers(2)
scatter_plot(loc, xticks = FALSE)
line_plot(loc, col = "orange", xticks = FALSE)
genetracks(loc)
par(oldpar)
}

## ----eval=FALSE---------------------------------------------------------------
# dat <- SLE_gwas_sub
# dat$p2 <- -log10(dat$p * 0.1)
# locp <- locus(dat, gene = 'UBE2L3', flank = 1e5, ens_db = "EnsDb.Hsapiens.v75")
# locp2 <- locus(dat, gene = 'UBE2L3', flank = 1e5, yvar = "p2",
#                ens_db = "EnsDb.Hsapiens.v75")
# 
# oldpar <- set_layers(1)
# scatter_plot(locp, xticks = FALSE, pcutoff = NULL, ylim = c(0, 16))
# scatter_plot(locp2, xticks = FALSE, pcutoff = NULL, scheme = "orange",
#              pch = 22, add = TRUE)
# genetracks(loc)
# par(oldpar)

## ----echo=FALSE, message=FALSE, fig.dim = c(7,7)------------------------------
if (require(EnsDb.Hsapiens.v75)) {
dat <- SLE_gwas_sub
dat$p2 <- -log10(dat$p * 0.1)
locp <- locus(dat, gene = 'UBE2L3', flank = 1e5, ens_db = "EnsDb.Hsapiens.v75")
locp2 <- locus(dat, gene = 'UBE2L3', flank = 1e5, yvar = "p2",
               ens_db = "EnsDb.Hsapiens.v75")

oldpar <- set_layers(1)
scatter_plot(locp, xticks = FALSE, pcutoff = NULL, ylim = c(0, 16))
scatter_plot(locp2, xticks = FALSE, pcutoff = NULL, scheme = "orange",
             pch = 22, add = TRUE)
genetracks(loc)
par(oldpar)
}

## ----eval=ensdb_eval, fig.dim = c(7,7)----------------------------------------
# add vertical lines for gene of interest under the main plot
pf <- quote({
  v <- locp$TX[locp$TX$gene_name == "UBE2L3", c("start", "end")]
  abline(v = v, col = "orange")
})

pl <- quote({
  # add custom text label for index SNP
  lx <- locp$data$pos[locp$data$rsid == locp$index_snp]
  ly <- locp$data$logP[locp$data$rsid == locp$index_snp]
  text(lx, ly, locp$index_snp, pos = 4, cex = 0.8)
  # add extra points
  px <- rep(22.05e6, 3)
  py <- 10:12
  points(px, py, pch = 21, bg = "green")
  # add custom legend
  legend("topleft", legend = c("group A", "group B"),
         pch = 21, pt.bg = c("blue", "green"), bty = "n")
})

locus_plot(locp, pcutoff = NULL, panel.first = pf, panel.last = pl)

## ----eval=FALSE---------------------------------------------------------------
# locus_ggplot(loc)

## ----eval=FALSE---------------------------------------------------------------
# gg_genetracks(loc)

## ----eval=FALSE---------------------------------------------------------------
# p <- gg_scatter(loc)
# p

## ----eval=FALSE---------------------------------------------------------------
# gg_addgenes(p, loc)

## ----eval=FALSE---------------------------------------------------------------
# p <- gg_scatter(loc) +
#   theme(legend.text = element_text(size = 14),
#         legend.title = element_text(size = 14))
# gg_addgenes(p, loc)

## ----eval=FALSE---------------------------------------------------------------
# locus_ggplot(loc, beta = "beta")
# 
# # add column 'typed' as a factor with levels "typed", "imputed"
# loc$data$typed <- factor(rbinom(n = nrow(loc$data), 1, 0.3), labels = c("imputed", "typed"))
# locus_ggplot(loc, shape = "typed")

## ----eval=FALSE---------------------------------------------------------------
# g <- gg_genetracks(loc)
# library(cowplot)
# plot_grid(p, p, g, ncol = 1, rel_heights = c(2, 2, 1), align = "v")
# 
# # patchwork method
# library(patchwork)
# p / p / g
# 
# # patchwork method 2
# wrap_plots(p, p, g, ncol = 1)

## ----eval=ensdb_eval, fig.dim = c(12, 7)--------------------------------------
p1 <- locus_ggplot(loc, labels = "index", nudge_x = 0.03)
p2 <- locus_ggplot(loc2, legend_pos = NULL)
cowplot::plot_grid(p1, p2, ncol = 2)

## ----eval=FALSE---------------------------------------------------------------
# library(ggpubr)
# pdf("my_ggplot.pdf", width = 10)
# ggarrange(p1, p2, ncol = 2)
# dev.off()
# 
# library(gridExtra)
# pdf("my_ggplot.pdf", width = 10)
# grid.arrange(p1, p2, ncol = 2)
# dev.off()

## ----eval=FALSE---------------------------------------------------------------
# locus_plotly(loc2)

## ----out.width='70%', fig.align="center", echo=FALSE--------------------------
knitr::include_graphics("plotly.png")

## ----eval=FALSE---------------------------------------------------------------
# # FTP download full summary statistics of this SLE GWAS from
# # https://www.ebi.ac.uk/gwas/studies/GCST003156
# library(data.table)
# SLE_gwas <- fread('../bentham_2015_26502338_sle_efo0002690_1_gwas.sumstats.tsv')
# 
# pks <- quick_peak(SLE_gwas)
# ## 34 peaks found (0.328 secs)
# 
# top_snps <- SLE_gwas$rsid[pks]
# head(top_snps)
# ## [1] "rs141910407" "rs114056368" "rs4274624"   "rs17849501"  "rs1143679"   "rs114115096"

## ----eval=FALSE---------------------------------------------------------------
# all_loci <- lapply(top_snps, function(i) {
#   loc <- locus(data = SLE_gwas, index_snp = i, fix_window = 1e6,
#         ens_db = "EnsDb.Hsapiens.v75")
#   link_recomb(loc)
# })
# 
# pdf("sle_loci.pdf")
# tmp <- lapply(all_loci, locus_plot, labels = "index")
# dev.off()

## ----eval=FALSE---------------------------------------------------------------
# pp <- lapply(all_loci, locus_ggplot, labels = "index", nudge_y = 1)
# 
# library(gridExtra)
# pdf("sle_loci_gg.pdf")
# for (i in seq_along(pp)) {
#   grid.arrange(pp[[i]])
# }
# dev.off()

#' Obtain GTEx eQTL data via LDlinkR
#' 
#' Adds eQTL (expression quantitative trait loci) information from GTEx
#' (https://gtexportal.org/) to a 'locus' class object. It queries LDlink
#' (https://ldlink.nci.nih.gov/) via the `LDlinkR` package to retrieve GTEx eQTL
#' information on a reference SNP.
#' 
#' @param loc Object of class 'locus' generated by [locus()]
#' @param pop A 1000 Genomes Project population, (e.g. YRI or CEU), multiple 
#' allowed, default = "CEU". Passed to `LDlinkR::LDexpress()`.
#' @param r2d Either "r2" for LD r^2 or "d" for LD D', default = "r2". Passed 
#' to `LDlinkR::LDexpress()`.
#' @param token Personal access token for accessing 1000 Genomes LD data via 
#' LDlink API. See `LDlinkR` package documentation.
#' @param ... Optional arguments such as `genome_build` which are passed on to
#'   `LDlinkR::LDexpress()`
#' @return Returns an object of class 'locus' with an extra list element 'LDexp'
#'   containing a dataframe of information obtained via `LDexpress()`.
#' @details
#' The additional eQTL information obtained from LDlink web server can be
#' displayed using [eqtl_plot()] which generates a scatter plot with gene tracks
#' similar to a locus plot, or with [overlay_plot()] which tries to overlay the
#' EQTL analysis over the original locus results (e.g. GWAS).
#' 
#' @seealso [locus()] [eqtl_plot()] [overlay_plot()]
#' @export

link_eqtl <- function(loc,
                      pop = "CEU",
                      r2d = "r2",
                      token = "", ...) {
  if (!inherits(loc, "locus")) stop("Not a locus object")
  if (!requireNamespace("LDlinkR", quietly = TRUE)) {
    stop("Package 'LDlinkR' must be installed to use this feature",
         call. = FALSE)
  }
  labs <- loc$labs
  index_snp <- loc$index_snp
  
  if (token == "") stop("token is missing")
  LDexp <- mem_LDexpress(snps = index_snp, pop = pop, r2d = r2d, 
                         token = token, ...)
  for (i in c("R2", "D'", "Effect_Size", "P_value")) {
    LDexp[, i] <- as.numeric(LDexp[, i])
  }
  LDexp$Effect_Allele <- gsub("=.*", "", LDexp$Effect_Allele_Freq)
  loc$LDexp <- LDexp
  
  loc
}

mem_LDexpress <- memoise(LDlinkR::LDexpress)

---
title: "locuszoomr"
author: "Myles Lewis"
output:
  html_document:
    toc: true
    toc_float:
      collapsed: false
    toc_depth: 2
    number_sections: true
vignette: >
  %\VignetteIndexEntry{locuszoomr}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE
)
library(locuszoomr)
```

# Introduction

The `locuszoomr` package allows users to produce publication ready gene locus 
plots very similar to those produced by the web interface 'locuszoom' 
(http://locuszoom.org), but running purely locally in R. Plots can easily be 
customised, labelled and stacked.

These gene annotation plots are produced via R base graphics or 'ggplot2'. A
'plotly' version can also be generated.

# Installation

Bioconductor packages `ensembldb` and an Ensembl database installed either as a
package or obtained through Bioconductor packages `AnnotationHub` are required
before installation. To run the examples in this vignette the
'EnsDb.Hsapiens.v75' ensembl database package needs to be installed from
Bioconductor.

```{r eval = FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
  install.packages("BiocManager")
BiocManager::install("ensembldb")
BiocManager::install("EnsDb.Hsapiens.v75")
```

Install from CRAN
```{r eval = FALSE}
install.packages("locuszoomr")
```

Install from Github
```{r eval = FALSE}
devtools::install_github("myles-lewis/locuszoomr")
```

`locuszoomr` can access the `LDlinkR` package to query 1000 Genomes for linkage
disequilibrium (LD) across SNPs. In order to make use of this API function you
will need a personal access token, available from the
[LDlink website](https://ldlink.nih.gov/?tab=apiaccess).

We recommend that users who want to add recombination rate lines to multiple
plots download the recombination rate track from UCSC and use it as described in
the section [Add recombination rate](#add-recombination-rate).

# Example locus plot

The quick example below uses a small subset (3 loci) of a GWAS dataset
incorporated into the package as a demo. The dataset is from a genetic study on
Systemic Lupus Erythematosus (SLE) by Bentham et al (2015). The full GWAS
summary statistics can be downloaded from
https://www.ebi.ac.uk/gwas/studies/GCST003156. The data format is shown below.

```{r}
library(locuszoomr)
data(SLE_gwas_sub)  ## limited subset of data from SLE GWAS
head(SLE_gwas_sub)
```

We plot a locus from this dataset by extracting a subset of the data using the
`locus()` function. Make sure you load the correct Ensembl database.

```{r eval=FALSE}
library(EnsDb.Hsapiens.v75)
loc <- locus(data = SLE_gwas_sub, gene = 'UBE2L3', flank = 1e5,
             ens_db = "EnsDb.Hsapiens.v75")
summary(loc)
locus_plot(loc)
```

```{r, echo=FALSE, message=FALSE, fig.dim=c(8, 7)}
if (require(EnsDb.Hsapiens.v75)) {
loc <- locus(data = SLE_gwas_sub, gene = 'UBE2L3', flank = 1e5,
             ens_db = "EnsDb.Hsapiens.v75")
summary(loc)
locus_plot(loc)
}
```

For users who only want to plot the gene tracks alone alongside their own plots, 
see section [Plot gene annotation only](#plot-gene-annotation-only) below.

When `locus()` is called, the function tries to autodetect which columns in the
data object refer to chromosome, position, SNP/feature ID and p-value. These
columns may need to be specified manually using the arguments `chrom`, `pos`,
`labs` and `p` respectively.

# Accessing Ensembl databases

Ensembl databases up to version 86 for *Homo sapiens* were loaded as individual
packages on Bioconductor. Recent databases are available through the
`AnnotationHub` Bioconductor package. Below we show a toy example to load
*H. sapiens* ensembl database v106 (even though it is misaligned with the
genotype data). If the argument `ens_db` in `locus()` is a character string it
specifies an Ensembl package which is queried through `get()`. For
`AnnotationHub` databases `ens_db` needs to be set to be the object containing
the database (not a string).

```{r eval=FALSE}
library(AnnotationHub)
ah <- AnnotationHub()
query(ah, c("EnsDb", "Homo sapiens"))
```

```{r eval=FALSE}
## AnnotationHub with 25 records
## # snapshotDate(): 2023-04-25
## # $dataprovider: Ensembl
## # $species: Homo sapiens
## # $rdataclass: EnsDb
## # additional mcols(): taxonomyid, genome, description, coordinate_1_based, maintainer,
## #   rdatadateadded, preparerclass, tags, rdatapath, sourceurl, sourcetype
## # retrieve records with, e.g., 'object[["AH53211"]]'
## 
##              title
##   AH53211  | Ensembl 87 EnsDb for Homo Sapiens
##   ...        ...
##   AH100643 | Ensembl 106 EnsDb for Homo sapiens
##   AH104864 | Ensembl 107 EnsDb for Homo sapiens
##   AH109336 | Ensembl 108 EnsDb for Homo sapiens
##   AH109606 | Ensembl 109 EnsDb for Homo sapiens
##   AH113665 | Ensembl 110 EnsDb for Homo sapiens
```

Fetch ensembl database version 106.

```{r eval=FALSE}
ensDb_v106 <- ah[["AH100643"]]

# built-in mini dataset
data("SLE_gwas_sub")
loc <- locus(data = SLE_gwas_sub, gene = 'UBE2L3', fix_window = 1e6,
             ens_db = ensDb_v106)
locus_plot(loc)
```

# Controlling the locus

The genomic locus can be specified in several ways. The simplest is to specify a
gene by name/symbol using the `gene` argument. The location of the gene is
obtained from the specified Ensembl database. The amount of flanking regions can
either be controlled by specifying `flank` which defaults to 50kb either side of
the ends of the gene. `flank` can either be a single number or a vector of 2
numbers if different down/upstream flanking lengths are required. Alternatively
a fixed genomic window (eg. 1 Mb) centred on the gene of interest can be
specified using the argument `fix_window`. The locus can be specified
manually by specifying the chromosome using `seqname` and genomic position range
using `xrange`. Finally, a region can be specified by naming the `index_snp`, in
which case the object `data` is searched for the coordinates of that SNP and the
size of the region defined using `fix_window` or `flank`.

# Obtaining LD information

Once an API personal access token has been obtained, the
[LDlink](https://ldlink.nih.gov/?tab=home) API can be called using the function
`link_LD()` to retrieve LD (linkage disequilibrium) information at the locus
which is overlaid on the locus plot. This is shown as a colour overlay showing
the level of $r^2$ between SNPs and the index SNP which defaults to the SNP with
the lowest p-value (or the SNP can be specified manually). Requests to LDlink
are cached using the `memoise` package, to reduce API requests.

```{r eval = FALSE}
# Locus plot using SLE GWAS data from Bentham et al 2015
# FTP download full summary statistics from
# https://www.ebi.ac.uk/gwas/studies/GCST003156
library(data.table)
SLE_gwas <- fread('../bentham_2015_26502338_sle_efo0002690_1_gwas.sumstats.tsv')
loc <- locus(SLE_gwas, gene = 'UBE2L3', flank = 1e5,
             ens_db = "EnsDb.Hsapiens.v75")
loc <- link_LD(loc, token = "your_token")
locus_plot(loc)
```

The subset of GWAS data included in the `locuszoomr` package has LD data already
acquired from LDlink which is included in the `r2` column. This can be plotted
by setting `LD = "r2"`. This method also allows users to add their own LD
information from their own datasets to loci.

```{r eval=FALSE}
loc <- locus(SLE_gwas_sub, gene = 'UBE2L3', flank = 1e5, LD = "r2",
             ens_db = "EnsDb.Hsapiens.v75")
locus_plot(loc, labels = c("index", "rs140492"),
                label_x = c(4, -5))
```


```{r echo=FALSE, message=FALSE, fig.dim=c(8, 7)}
if (require(EnsDb.Hsapiens.v75)) {
loc <- locus(SLE_gwas_sub, gene = 'UBE2L3', flank = 1e5, LD = "r2",
             ens_db = "EnsDb.Hsapiens.v75")
locus_plot(loc, labels = c("index", "rs140492"), label_x = c(4, -5))
}
```

# Add recombination rate

In keeping with the original locuszoom, recombination rate can be shown on a
secondary *y* axis using the `link_recomb()` function to retrieve recombination
rate data from [UCSC genome browser](https://genome.ucsc.edu). Calls to UCSC are 
cached using `memoise` to reduce API requests.

```{r eval=FALSE}
loc3 <- locus(SLE_gwas_sub, gene = 'STAT4', flank = 1e5, LD = "r2",
              ens_db = "EnsDb.Hsapiens.v75")
loc3 <- link_recomb(loc3)
locus_plot(loc3)
```

```{r, out.width='80%', echo=FALSE}
knitr::include_graphics("recomb.png")
```

The recombination line can be offset from the scatter points to make it more 
visible by setting `recomb_offset = 0.1`.

If you are performing multiple UCSC queries, it is much faster to download the
whole recombination rate track data file (around 30 MB) from UCSC genome browser
(documented [here](https://genome.ucsc.edu/cgi-bin/hgTrackUi?g=recombRate2)).

The download site can be accessed at
http://hgdownload.soe.ucsc.edu/gbdb/hg38/recombRate/. For hg38, download
`recomb1000GAvg.bw`.

For hg19, the link is http://hgdownload.soe.ucsc.edu/gbdb/hg19/decode/ and the
default track we use is `hapMapRelease24CombinedRecombMap.bw`.

The .bw track file can then be loaded into R as a `GRanges` object using
`import.bw()`, and then used directly by `link_recomb()` as follows:

```{r eval=FALSE}
library(rtracklayer)
recomb.hg19 <- import.bw("/../hapMapRelease24CombinedRecombMap.bw")
loc3 <- link_recomb(loc3, recomb = recomb.hg19)
locus_plot(loc3)
```

# Customise plots and labels

Various plotting options can be customised through arguments via the call to
`locus_plot()`. When calling `locus_plot()` arguments are passed to either
`genetracks()` to control the gene tracks or passed via the `...` system onto
`scatter_plot()` to control the scatter plot.

Plot borders can be set using `border = TRUE`. The chromosome 
position $x$ axis labels can be placed under the top or bottom plots using 
`xtick = "top"` or `"bottom"`.

Additional arguments can also be passed onto `plot()` via the `...` system, e.g.
`ylim`, `par()` settings etc. For example `col = NA` can be added to
`locus_plot()` or `scatter_plot()` to remove the black outline around the
scatter plot markers.

Labels can be added by specifying a vector of SNP or genomic feature IDs as
shown in the plot above using the argument `labels` (see `scatter_plot()`). The
value `"index"` refers to the index SNP as the highest point in the locus or as
defined by the argument `index_snp` when `locus()` is called. The easiest way to
identify points is using the [plotly version](#plotly-version) `locus_plotly()`
which allows you to hover over points and see their rsid or feature label.
`label_x` and `label_y` control the position of the labels and can be specified
as a single value or a vector of values.

## Point shapes

Advanced users familiar with base graphics can customise every single point
on the scatter plot, by adding columns named `bg`, `col`, `pch` or `cex`
directly to the dataframe stored in `$data` element of the `locus` object.
Setting these will overrule any default settings. These columns refer to their
respective base graphics arguments: `bg` sets the fill colour for points, `col`
sets the outline colour, `pch` sets the symbols (see `?points` for a list of
these) and `cex` sets the size of points (default is 1).

```{r eval=FALSE}
# add column 'typed' as a factor which is 1 for typed, 0 for imputed
loc$data$typed <- factor(rbinom(n = nrow(loc$data), 1, 0.3))
# convert column to shapes by adding a column called 'pch'
loc$data$pch <- c(21, 24)[loc$data$typed]
locus_plot(loc)
# pch 21 = circles = imputed
# pch 24 = triangles = typed
```

See the help pages at `?locus_plot` and `?scatter_plot` for more details.

# Customise gene tracks

The gene tracks can be also customised with colours and gene label text
position. Gene names can be italicised by setting the argument `italics = TRUE`.
See the help page at `?genetracks` and `?locus_plot` for more details.

```{r eval=FALSE}
# Filter by gene biotype
locus_plot(loc, filter_gene_biotype = "protein_coding")

# Custom selection of genes using gene names
locus_plot(loc, filter_gene_name = c('UBE2L3', 'RIMBP3C', 'YDJC', 'PPIL2',
                                     'PI4KAP2', 'MIR301B'))
```

# Plot gene annotation only

The gene track can be plotted from a `locus` class object using the function 
`genetracks()`. This uses base graphics, so `layout()` can be used to stack 
custom-made plots above or below the gene tracks. The function `set_layers()` is
designed to make this easier for users. See section [Layering
plots](#layering-plots) below.

```{r, eval=FALSE}
genetracks(loc, highlight = "UBE2L3")
```

```{r, echo=FALSE, fig.dim=c(7, 3.5)}
if (require(EnsDb.Hsapiens.v75)) {
genetracks(loc, highlight = "UBE2L3")
}
```

The function allows control over plotting of the gene tracks such as changing
the number of gene annotation tracks and the colour scheme. Set
`showExons=FALSE` to show only genes and hide the exons.

```{r, eval=FALSE}
# Limit the number of tracks
# Filter by gene biotype
# Customise colours
genetracks(loc, maxrows = 3, filter_gene_biotype = 'protein_coding',
           gene_col = 'grey', exon_col = 'orange', exon_border = 'darkgrey')
```

```{r, echo=FALSE, fig.dim=c(7, 2.5)}
if (require(EnsDb.Hsapiens.v75)) {
genetracks(loc, maxrows = 3, filter_gene_biotype = 'protein_coding',
           gene_col = 'grey', exon_col = 'orange', exon_border = 'darkgrey')
}
```

For advanced users who only want the gene tracks to add to their own plots,
`locus()` can be called without the `data` argument specified (or `data` can be
set to `NULL`). Then genetracks can be plotted in base graphics, ggplot2 or 
plotly.

```{r eval = FALSE}
loc00 <- locus(gene = 'UBE2L3', flank = 1e5, ens_db = "EnsDb.Hsapiens.v75")

genetracks(loc00)  # base graphics
gg_genetracks(loc00)  # ggplot2
genetrack_ly(loc00)  # plotly
```

# Add eQTL data via LDlink

GTEx eQTL data can be accessed via the LDlinkR package which queries the
[LDlink](https://ldlink.nih.gov/?tab=home) server API to download data and add
it to a 'locus' object. The eQTL data is stored in `LDexp` slot in the locus
object.

```{r eval=FALSE}
# obtain GTEx eQTL data from LDlinkR
# needs personal access token for LDlink
loc2 <- locus(SLE_gwas_sub, gene = 'IRF5', flank = c(7e4, 2e5), LD = "r2",
              ens_db = "EnsDb.Hsapiens.v75")
loc2 <- link_eqtl(loc2, token = "..")
head(loc2$LDexp)  # show eQTL data
```

Often at a locus eQTL data may be available on a range of genes and tissues. The
following code shows how to see which genes and tissues are available and how
eQTL SNPs they each have.

```{r eval=FALSE}
table(loc2$LDexp$Gene_Symbol)
##      AHCYL2           IRF5            KCP        METTL2B           NRF1   RP11-212P7.2 
##           5           1618              1              2              9              3 
## ...

table(loc2$LDexp$Tissue)
##             Adipose - Subcutaneous          Adipose - Visceral (Omentum) 
##                                 78                                    69 
##                      Adrenal Gland                        Artery - Aorta 
##                                 38                                   109
## ...
```

Embedded eQTL data can be plotted using `eqtl_plot()` or shown as an overlay
plot using `overlay_plot()` which shows GWAS or association data on the *y* axis
and overlays eQTL results using symbols and colours.

```{r eval=FALSE}
# just plot eQTL results
eqtl_plot(loc2, tissue = "Whole Blood", eqtl_gene = "IRF5")

# overlay eqtl plot
overlay_plot(loc2, eqtl_gene = "IRF5")
```

```{r, out.width='60%', fig.align="center", echo=FALSE}
knitr::include_graphics("eqtl_overlay.png")
```

# Change *y*-axis variable

Instead of plotting -log~10~ p-value on the y axis, it is possible to specify a
different variable in your dataset using the argument `yvar`.

```{r eval=FALSE}
locb <- locus(SLE_gwas_sub, gene = 'UBE2L3', flank = 1e5, yvar = "beta",
              ens_db = "EnsDb.Hsapiens.v75")
locus_plot(locb)
```


```{r, echo=FALSE, message=FALSE, fig.dim=c(8, 7)}
if (require(EnsDb.Hsapiens.v75)) {
locb <- locus(SLE_gwas_sub, gene = 'UBE2L3', flank = 1e5, yvar = "beta",
              ens_db = "EnsDb.Hsapiens.v75")
locus_plot(locb)
}
```

# Arrange multiple locus plots

`locuszoomr` uses `graphics::layout` to arrange plots. To layout multiple locus
plots side by side, use the function `multi_layout()` to set the number of locus
plots per row and column. The `plots` argument in `multi_layout()` can either be
a list of locus class objects, one for each gene. Or for full control it can be
an 'expression' with a series of manual calls to `locus_plot()`. Alternatively a
`for` loop could be called within the `plots` expression.

```{r eval=FALSE}
genes <- c("STAT4", "IRF5", "UBE2L3")

# generate list of 'locus' class objects, one for each gene
loclist <- lapply(genes, locus,
                  data = SLE_gwas_sub,
                  ens_db = "EnsDb.Hsapiens.v75",
                  LD = "r2")

## produce 3 locus plots, one on each page
pdf("myplot.pdf")
multi_layout(loclist)
dev.off()

## place 3 locus plots in a row on a single page
pdf("myplot.pdf")
multi_layout(loclist, ncol = 3, labels = "index")
dev.off()

## full control
loc2 <- locus(SLE_gwas_sub, gene = 'IRF5', flank = c(7e4, 2e5), LD = "r2",
              ens_db = "EnsDb.Hsapiens.v75")
loc3 <- locus(SLE_gwas_sub, gene = 'STAT4', flank = 1e5, LD = "r2",
              ens_db = "EnsDb.Hsapiens.v75")

pdf("myplot.pdf", width = 9, height = 6)
multi_layout(ncol = 3,
             plots = {
               locus_plot(loc, use_layout = FALSE, legend_pos = 'topleft')
               locus_plot(loc2, use_layout = FALSE, legend_pos = NULL)
               locus_plot(loc3, use_layout = FALSE, legend_pos = NULL,
                          labels = "index")
             })
dev.off()
```

```{r, echo=FALSE, message=FALSE, fig.dim = c(9,5)}
if (require(EnsDb.Hsapiens.v75)) {
loc2 <- locus(SLE_gwas_sub, gene = 'IRF5', flank = c(7e4, 2e5), LD = "r2",
              ens_db = "EnsDb.Hsapiens.v75")
loc3 <- locus(SLE_gwas_sub, gene = 'STAT4', flank = 1e5, LD = "r2",
              ens_db = "EnsDb.Hsapiens.v75")

multi_layout(ncol = 3,
             plots = {
               locus_plot(loc, use_layout = FALSE, legend_pos = 'topleft')
               locus_plot(loc2, use_layout = FALSE, legend_pos = NULL)
               locus_plot(loc3, use_layout = FALSE, legend_pos = NULL,
                          labels = "index")
             })
}
```

# Layering plots

## Column of plots

`locuszoomr` has been designed with modular functions to enable layering of
plots on top of each other in a column with gene tracks on the bottom.
`set_layers()` is used to set layout of plots in base graphics by calling
`layout()`. You will need to reset `par()` to original layout settings once the
multi-panel plot is finished.

`scatter_plot()` is used to generate the locus plot. `line_plot()` is used as an
example of an additional plot. Also see `eqtl_plot()` for plotting eQTL
information retrieved via the LDlinkR package.

```{r eval=FALSE}
pdf("myplot2.pdf", width = 6, height = 8)
# set up layered plot with 2 plots & a gene track; store old par() settings
oldpar <- set_layers(2)
scatter_plot(loc, xticks = FALSE)
line_plot(loc, col = "orange", xticks = FALSE)
genetracks(loc)
par(oldpar)  # revert par() settings
dev.off()
```

```{r, echo=FALSE, fig.dim = c(6,8)}
if (require(EnsDb.Hsapiens.v75)) {
oldpar <- set_layers(2)
scatter_plot(loc, xticks = FALSE)
line_plot(loc, col = "orange", xticks = FALSE)
genetracks(loc)
par(oldpar)
}
```

## Overlaid plots

`scatter_plot()` can be called with argument `add = TRUE` to add multiple sets
of points overlaid on one plot.

```{r eval=FALSE}
dat <- SLE_gwas_sub
dat$p2 <- -log10(dat$p * 0.1)
locp <- locus(dat, gene = 'UBE2L3', flank = 1e5, ens_db = "EnsDb.Hsapiens.v75")
locp2 <- locus(dat, gene = 'UBE2L3', flank = 1e5, yvar = "p2",
               ens_db = "EnsDb.Hsapiens.v75")

oldpar <- set_layers(1)
scatter_plot(locp, xticks = FALSE, pcutoff = NULL, ylim = c(0, 16))
scatter_plot(locp2, xticks = FALSE, pcutoff = NULL, scheme = "orange",
             pch = 22, add = TRUE)
genetracks(loc)
par(oldpar)
```

```{r, echo=FALSE, message=FALSE, fig.dim = c(7,7)}
if (require(EnsDb.Hsapiens.v75)) {
dat <- SLE_gwas_sub
dat$p2 <- -log10(dat$p * 0.1)
locp <- locus(dat, gene = 'UBE2L3', flank = 1e5, ens_db = "EnsDb.Hsapiens.v75")
locp2 <- locus(dat, gene = 'UBE2L3', flank = 1e5, yvar = "p2",
               ens_db = "EnsDb.Hsapiens.v75")

oldpar <- set_layers(1)
scatter_plot(locp, xticks = FALSE, pcutoff = NULL, ylim = c(0, 16))
scatter_plot(locp2, xticks = FALSE, pcutoff = NULL, scheme = "orange",
             pch = 22, add = TRUE)
genetracks(loc)
par(oldpar)
}
```

# Add custom legend / features

The power of base graphics is that it gives complete control over plotting. In
the example below, we show how to add your own legend, text labels, lines to
demarcate a gene and extra points on top or underneath the main plot. When
`plot()` is called, base graphics allows additional plotting using the arguments
`panel.first` and `panel.last`. Since these are called inside the `locus_plot()`
function they need to be quoted using `quote()`.

```{r eval=FALSE}
# add vertical lines for gene of interest under the main plot
pf <- quote({
  v <- locp$TX[locp$TX$gene_name == "UBE2L3", c("start", "end")]
  abline(v = v, col = "orange")
})

pl <- quote({
  # add custom text label for index SNP
  lx <- locp$data$pos[locp$data$rsid == locp$index_snp]
  ly <- locp$data$logP[locp$data$rsid == locp$index_snp]
  text(lx, ly, locp$index_snp, pos = 4, cex = 0.8)
  # add extra points
  px <- rep(22.05e6, 3)
  py <- 10:12
  points(px, py, pch = 21, bg = "green")
  # add custom legend
  legend("topleft", legend = c("group A", "group B"),
         pch = 21, pt.bg = c("blue", "green"), bty = "n")
})

locus_plot(locp, pcutoff = NULL, panel.first = pf, panel.last = pl)
```

```{r, echo=FALSE, fig.dim = c(7,7)}
if (require(EnsDb.Hsapiens.v75)) {
pf <- quote({
  v <- locp$TX[locp$TX$gene_name == "UBE2L3", c("start", "end")]
  abline(v = v, col = "orange")
})

pl <- quote({
  lx <- locp$data$pos[locp$data$rsid == locp$index_snp]
  ly <- locp$data$logP[locp$data$rsid == locp$index_snp]
  text(lx, ly, locp$index_snp, pos = 4, cex = 0.8)
  px <- rep(22.05e6, 3)
  py <- 10:12
  points(px, py, pch = 21, bg = "green")
  legend("topleft", legend = c("group A", "group B"),
         pch = 21, pt.bg = c("blue", "green"), bty = "n")
})

locus_plot(locp, pcutoff = NULL, panel.first = pf, panel.last = pl)
}
```

# ggplot2 version

`ggplot2` versions of several of the above functions are available. For a whole
locus plot use `locus_ggplot()`.

```{r eval=FALSE}
locus_ggplot(loc)
```

The gene tracks can be produced on their own as a ggplot2 object using
`gg_genetracks()`.

```{r eval=FALSE}
gg_genetracks(loc)
```

The scatter plot alone can be produced as a ggplot2 object.

```{r eval=FALSE}
p <- gg_scatter(loc)
p
```

Finally, `gg_addgenes()` can be used to add gene tracks to an existing ggplot2
plot that has been previously created and customised.

```{r eval=FALSE}
gg_addgenes(p, loc)
```

If you are using ggplot2, the easiest way to edit the scatter plot and change
legend and other details, is to use `gg_scatter()` and modify the plot using the
ggplot2 `+` notation. For example, to increase the font size of the legend you
could do the following:

```{r eval=FALSE}
p <- gg_scatter(loc) +
  theme(legend.text = element_text(size = 14),
        legend.title = element_text(size = 14))
gg_addgenes(p, loc)
```

## Shapes

Shapes can be overlaid in the scatter plot by setting either `beta` or `shape`
to a column name in the dataset. Setting `beta` shows up-triangles for positive
beta values and down-triangles for negative beta values. If `shape` is set, for
example to show imputed vs typed SNPs, the column needs to be a factor (to make
the legend nice). `shape_values` is then set as a vector of shapes to which the
levels of the factor are mapped on the scatter plot.

```{r eval=FALSE}
locus_ggplot(loc, beta = "beta")

# add column 'typed' as a factor with levels "typed", "imputed"
loc$data$typed <- factor(rbinom(n = nrow(loc$data), 1, 0.3), labels = c("imputed", "typed"))
locus_ggplot(loc, shape = "typed")
```

## Layering plots

Plots can be layered above gene tracks, using `cowplot::plot_grid()` or using
the `patchwork` package.

```{r eval=FALSE}
g <- gg_genetracks(loc)
library(cowplot)
plot_grid(p, p, g, ncol = 1, rel_heights = c(2, 2, 1), align = "v")

# patchwork method
library(patchwork)
p / p / g

# patchwork method 2
wrap_plots(p, p, g, ncol = 1)
```

For users who prefer the grid system we recommend also looking at Bioconductor
packages `Gviz` or `ggbio` as possible alternatives.

## Arrange multiple ggplots

It is possible to use either the `cowplot` or `ggpubr` packages to layout
multiple locus ggplots on the same page.

```{r eval=FALSE}
library(cowplot)
p1 <- locus_ggplot(loc, labels = "index", nudge_x = 0.03)
p2 <- locus_ggplot(loc2, legend_pos = NULL)
plot_grid(p1, p2, ncol = 2)
```


```{r echo=FALSE, fig.dim = c(12, 7)}
if (require(EnsDb.Hsapiens.v75)) {
p1 <- locus_ggplot(loc, labels = "index", nudge_x = 0.03)
p2 <- locus_ggplot(loc2, legend_pos = NULL)
cowplot::plot_grid(p1, p2, ncol = 2)
}
```

Or using ggpubr or gridExtra:

```{r eval=FALSE}
library(ggpubr)
pdf("my_ggplot.pdf", width = 10)
ggarrange(p1, p2, ncol = 2)
dev.off()

library(gridExtra)
pdf("my_ggplot.pdf", width = 10)
grid.arrange(p1, p2, ncol = 2)
dev.off()
```

A known issue with the ggplot2 version `locus_ggplot()` is that the system for
ensuring that gene labels seems to go wrong when the plot window is resized
during arrangement of multiple plots. The workable solution at present is to
make sure that the height and width of the final pdf/output is enlarged as per
the example above, which plots fine without any text overlapping when exported
to pdf at an appropriate size.

# Plotly version

`locuszoomr` includes a ['plotly'](https://plotly.com/r/) version for plotting
locus plots which is interactive. This allows users to hover over the plot and
reveal additional information such as SNP rs IDs for each point and information
about each gene in the gene tracks. This can help when exploring a locus or
region and trying to identify particular SNPs (or genomic features) of interest.

```{r eval=FALSE}
locus_plotly(loc2)
```

```{r, out.width='70%', fig.align="center", echo=FALSE}
knitr::include_graphics("plotly.png")
```

# Plot all GWAS peaks

`locuszoomr` includes a function `quick_peak()` to quickly find GWAS peaks based
on a set p-value threshold (default 5e-08) and specifying a minimum distance
between peaks (default 1 Mb). The function will either find all peaks ordered by
lowest p-value, or the top *n* peaks can be identified by setting `npeaks`.

```{r eval=FALSE}
# FTP download full summary statistics of this SLE GWAS from
# https://www.ebi.ac.uk/gwas/studies/GCST003156
library(data.table)
SLE_gwas <- fread('../bentham_2015_26502338_sle_efo0002690_1_gwas.sumstats.tsv')

pks <- quick_peak(SLE_gwas)
## 34 peaks found (0.328 secs)

top_snps <- SLE_gwas$rsid[pks]
head(top_snps)
## [1] "rs141910407" "rs114056368" "rs4274624"   "rs17849501"  "rs1143679"   "rs114115096"
```

We can now plot locus plots for all the GWAS peaks as a single pdf, as specified
by the vector of top SNPs. The `link_recomb()` line below can be sped up by
downloading the recombination track file from UCSC - see example code in 
[Add recombination rate](#add-recombination-rate).

```{r eval=FALSE}
all_loci <- lapply(top_snps, function(i) {
  loc <- locus(data = SLE_gwas, index_snp = i, fix_window = 1e6,
        ens_db = "EnsDb.Hsapiens.v75")
  link_recomb(loc)
})

pdf("sle_loci.pdf")
tmp <- lapply(all_loci, locus_plot, labels = "index")
dev.off()
```

Alternatively export plots using ggplot2.

```{r eval=FALSE}
pp <- lapply(all_loci, locus_ggplot, labels = "index", nudge_y = 1)

library(gridExtra)
pdf("sle_loci_gg.pdf")
for (i in seq_along(pp)) {
  grid.arrange(pp[[i]])
}
dev.off()
```

# Manhattan & other plots

For Manhattan plots, log p-value QQ plot and easy labelling of volcano plots or
other scatter plots with draggable labels, check out our sister package
`easylabel` on CRAN at https://cran.r-project.org/package=easylabel.

# Citation

If you use this package please cite as:

Lewis MJ, Wang S. (2025) locuszoomr: an R package for visualising
publication-ready regional gene locus plots. *Bioinformatics Advances* 2025;
vbaf006, [doi:10.1093/bioadv/vbaf006](https://doi.org/10.1093/bioadv/vbaf006)

# References

Pruim RJ, Welch RP, Sanna S, Teslovich TM, Chines PS, Gliedt TP, Boehnke M,
Abecasis GR, Willer CJ. (2010) LocusZoom: Regional visualization of genome-wide
association scan results. *Bioinformatics* 2010; 26(18): 2336-7.

---
title: "locuszoomr"
author: "Myles Lewis"
output:
  html_document:
    toc: true
    toc_float:
      collapsed: false
    toc_depth: 2
    number_sections: true
vignette: >
  %\VignetteIndexEntry{locuszoomr}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE
)
library(locuszoomr)
ensdb_eval <- requireNamespace("EnsDb.Hsapiens.v75", quietly = TRUE)
```

# Introduction

The `locuszoomr` package allows users to produce publication ready gene locus 
plots very similar to those produced by the web interface 'locuszoom' 
(http://locuszoom.org), but running purely locally in R. Plots can easily be 
customised, labelled and stacked.

These gene annotation plots are produced via R base graphics or 'ggplot2'. A
'plotly' version can also be generated.

# Installation

Bioconductor packages `ensembldb` and an Ensembl database installed either as a
package or obtained through Bioconductor packages `AnnotationHub` are required
before installation. To run the examples in this vignette the
'EnsDb.Hsapiens.v75' ensembl database package needs to be installed from
Bioconductor.

```{r eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
  install.packages("BiocManager")
BiocManager::install("ensembldb")
BiocManager::install("EnsDb.Hsapiens.v75")
```

Install from CRAN
```{r eval=FALSE}
install.packages("locuszoomr")
```

Install from Github
```{r eval=FALSE}
devtools::install_github("myles-lewis/locuszoomr")
```

`locuszoomr` can access the `LDlinkR` package to query 1000 Genomes for linkage
disequilibrium (LD) across SNPs. In order to make use of this API function you
will need a personal access token, available from the
[LDlink website](https://ldlink.nih.gov/?tab=apiaccess).

We recommend that users who want to add recombination rate lines to multiple
plots download the recombination rate track from UCSC and use it as described in
the section [Add recombination rate](#add-recombination-rate).

# Example locus plot

The quick example below uses a small subset (3 loci) of a GWAS dataset
incorporated into the package as a demo. The dataset is from a genetic study on
Systemic Lupus Erythematosus (SLE) by Bentham et al (2015). The full GWAS
summary statistics can be downloaded from
https://www.ebi.ac.uk/gwas/studies/GCST003156. The data format is shown below.

```{r}
library(locuszoomr)
data(SLE_gwas_sub)  ## limited subset of data from SLE GWAS
head(SLE_gwas_sub)
```

We plot a locus from this dataset by extracting a subset of the data using the
`locus()` function. Make sure you load the correct Ensembl database.

```{r, eval=ensdb_eval, message=FALSE, fig.dim=c(8, 7)}
library(EnsDb.Hsapiens.v75)
loc <- locus(data = SLE_gwas_sub, gene = 'UBE2L3', flank = 1e5,
             ens_db = "EnsDb.Hsapiens.v75")
summary(loc)
locus_plot(loc)
```

For users who only want to plot the gene tracks alone alongside their own plots, 
see section [Plot gene annotation only](#plot-gene-annotation-only) below.

When `locus()` is called, the function tries to autodetect which columns in the
data object refer to chromosome, position, SNP/feature ID and p-value. These
columns may need to be specified manually using the arguments `chrom`, `pos`,
`labs` and `p` respectively.

# Accessing Ensembl databases

Ensembl databases up to version 86 for *Homo sapiens* were loaded as individual
packages on Bioconductor. Recent databases are available through the
`AnnotationHub` Bioconductor package. Below we show a toy example to load
*H. sapiens* ensembl database v106 (even though it is misaligned with the
genotype data). If the argument `ens_db` in `locus()` is a character string it
specifies an Ensembl package which is queried through `get()`. For
`AnnotationHub` databases `ens_db` needs to be set to be the object containing
the database (not a string).

```{r eval=FALSE}
library(AnnotationHub)
ah <- AnnotationHub()
query(ah, c("EnsDb", "Homo sapiens"))
## AnnotationHub with 25 records
## # snapshotDate(): 2023-04-25
## # $dataprovider: Ensembl
## # $species: Homo sapiens
## # $rdataclass: EnsDb
## # additional mcols(): taxonomyid, genome, description, coordinate_1_based, maintainer,
## #   rdatadateadded, preparerclass, tags, rdatapath, sourceurl, sourcetype
## # retrieve records with, e.g., 'object[["AH53211"]]'
## 
##              title
##   AH53211  | Ensembl 87 EnsDb for Homo Sapiens
##   ...        ...
##   AH100643 | Ensembl 106 EnsDb for Homo sapiens
##   AH104864 | Ensembl 107 EnsDb for Homo sapiens
##   AH109336 | Ensembl 108 EnsDb for Homo sapiens
##   AH109606 | Ensembl 109 EnsDb for Homo sapiens
##   AH113665 | Ensembl 110 EnsDb for Homo sapiens
```

Fetch ensembl database version 106.

```{r eval=FALSE}
ensDb_v106 <- ah[["AH100643"]]

# built-in mini dataset
data("SLE_gwas_sub")
loc <- locus(data = SLE_gwas_sub, gene = 'UBE2L3', fix_window = 1e6,
             ens_db = ensDb_v106)
locus_plot(loc)
```

# Controlling the locus

The genomic locus can be specified in several ways. The simplest is to specify a
gene by name/symbol using the `gene` argument. The location of the gene is
obtained from the specified Ensembl database. The amount of flanking regions can
either be controlled by specifying `flank` which defaults to 50kb either side of
the ends of the gene. `flank` can either be a single number or a vector of 2
numbers if different down/upstream flanking lengths are required. Alternatively
a fixed genomic window (eg. 1 Mb) centred on the gene of interest can be
specified using the argument `fix_window`. The locus can be specified
manually by specifying the chromosome using `seqname` and genomic position range
using `xrange`. Finally, a region can be specified by naming the `index_snp`, in
which case the object `data` is searched for the coordinates of that SNP and the
size of the region defined using `fix_window` or `flank`.

# Obtaining LD information

Once an API personal access token has been obtained, the
[LDlink](https://ldlink.nih.gov/?tab=home) API can be called using the function
`link_LD()` to retrieve LD (linkage disequilibrium) information at the locus
which is overlaid on the locus plot. This is shown as a colour overlay showing
the level of $r^2$ between SNPs and the index SNP which defaults to the SNP with
the lowest p-value (or the SNP can be specified manually). Requests to LDlink
are cached using the `memoise` package, to reduce API requests.

```{r eval=FALSE}
# Locus plot using SLE GWAS data from Bentham et al 2015
# FTP download full summary statistics from
# https://www.ebi.ac.uk/gwas/studies/GCST003156
library(data.table)
SLE_gwas <- fread('../bentham_2015_26502338_sle_efo0002690_1_gwas.sumstats.tsv')
loc <- locus(SLE_gwas, gene = 'UBE2L3', flank = 1e5,
             ens_db = "EnsDb.Hsapiens.v75")
loc <- link_LD(loc, token = "your_token")
locus_plot(loc)
```

The subset of GWAS data included in the `locuszoomr` package has LD data already
acquired from LDlink which is included in the `r2` column. This can be plotted
by setting `LD = "r2"`. This method also allows users to add their own LD
information from their own datasets to loci.

```{r, eval=ensdb_eval, message=FALSE, fig.dim=c(8, 7)}
loc <- locus(SLE_gwas_sub, gene = 'UBE2L3', flank = 1e5, LD = "r2",
             ens_db = "EnsDb.Hsapiens.v75")
locus_plot(loc, labels = c("index", "rs140492"), label_x = c(4, -5))
```

# Add recombination rate

In keeping with the original locuszoom, recombination rate can be shown on a
secondary *y* axis using the `link_recomb()` function to retrieve recombination
rate data from [UCSC genome browser](https://genome.ucsc.edu). Calls to UCSC are 
cached using `memoise` to reduce API requests.

```{r eval=FALSE}
loc3 <- locus(SLE_gwas_sub, gene = 'STAT4', flank = 1e5, LD = "r2",
              ens_db = "EnsDb.Hsapiens.v75")
loc3 <- link_recomb(loc3)
locus_plot(loc3)
```

```{r, out.width='80%', echo=FALSE}
knitr::include_graphics("recomb.png")
```

The recombination line can be offset from the scatter points to make it more 
visible by setting `recomb_offset = 0.1`.

If you are performing multiple UCSC queries, it is much faster to download the
whole recombination rate track data file (around 30 MB) from UCSC genome browser
(documented [here](https://genome.ucsc.edu/cgi-bin/hgTrackUi?g=recombRate2)).

The download site can be accessed at
http://hgdownload.soe.ucsc.edu/gbdb/hg38/recombRate/. For hg38, download
`recomb1000GAvg.bw`.

For hg19, the link is http://hgdownload.soe.ucsc.edu/gbdb/hg19/decode/ and the
default track we use is `hapMapRelease24CombinedRecombMap.bw`.

The .bw track file can then be loaded into R as a `GRanges` object using
`import.bw()`, and then used directly by `link_recomb()` as follows:

```{r eval=FALSE}
library(rtracklayer)
recomb.hg19 <- import.bw("/../hapMapRelease24CombinedRecombMap.bw")
loc3 <- link_recomb(loc3, recomb = recomb.hg19)
locus_plot(loc3)
```

# Customise plots and labels

Various plotting options can be customised through arguments via the call to
`locus_plot()`. When calling `locus_plot()` arguments are passed to either
`genetracks()` to control the gene tracks or passed via the `...` system onto
`scatter_plot()` to control the scatter plot.

Plot borders can be set using `border = TRUE`. The chromosome 
position $x$ axis labels can be placed under the top or bottom plots using 
`xtick = "top"` or `"bottom"`.

Additional arguments can also be passed onto `plot()` via the `...` system, e.g.
`ylim`, `par()` settings etc. For example `col = NA` can be added to
`locus_plot()` or `scatter_plot()` to remove the black outline around the
scatter plot markers.

Labels can be added by specifying a vector of SNP or genomic feature IDs as
shown in the plot above using the argument `labels` (see `scatter_plot()`). The
value `"index"` refers to the index SNP as the highest point in the locus or as
defined by the argument `index_snp` when `locus()` is called. The easiest way to
identify points is using the [plotly version](#plotly-version) `locus_plotly()`
which allows you to hover over points and see their rsid or feature label.
`label_x` and `label_y` control the position of the labels and can be specified
as a single value or a vector of values.

## Point shapes

Advanced users familiar with base graphics can customise every single point
on the scatter plot, by adding columns named `bg`, `col`, `pch` or `cex`
directly to the dataframe stored in `$data` element of the `locus` object.
Setting these will overrule any default settings. These columns refer to their
respective base graphics arguments: `bg` sets the fill colour for points, `col`
sets the outline colour, `pch` sets the symbols (see `?points` for a list of
these) and `cex` sets the size of points (default is 1).

```{r eval=FALSE}
# add column 'typed' as a factor which is 1 for typed, 0 for imputed
loc$data$typed <- factor(rbinom(n = nrow(loc$data), 1, 0.3))
# convert column to shapes by adding a column called 'pch'
loc$data$pch <- c(21, 24)[loc$data$typed]
locus_plot(loc)
# pch 21 = circles = imputed
# pch 24 = triangles = typed
```

See the help pages at `?locus_plot` and `?scatter_plot` for more details.

# Customise gene tracks

The gene tracks can be also customised with colours and gene label text
position. Gene names can be italicised by setting the argument `italics = TRUE`.
See the help page at `?genetracks` and `?locus_plot` for more details.

```{r eval=FALSE}
# Filter by gene biotype
locus_plot(loc, filter_gene_biotype = "protein_coding")

# Custom selection of genes using gene names
locus_plot(loc, filter_gene_name = c('UBE2L3', 'RIMBP3C', 'YDJC', 'PPIL2',
                                     'PI4KAP2', 'MIR301B'))
```

# Plot gene annotation only

The gene track can be plotted from a `locus` class object using the function 
`genetracks()`. This uses base graphics, so `layout()` can be used to stack 
custom-made plots above or below the gene tracks. The function `set_layers()` is
designed to make this easier for users. See section [Layering
plots](#layering-plots) below.

```{r, eval=ensdb_eval, fig.dim=c(7, 3.5)}
genetracks(loc, highlight = "UBE2L3")
```

The function allows control over plotting of the gene tracks such as changing
the number of gene annotation tracks and the colour scheme. Set
`showExons=FALSE` to show only genes and hide the exons.

```{r, eval=ensdb_eval, fig.dim=c(7, 2.5)}
# Limit the number of tracks
# Filter by gene biotype
# Customise colours
genetracks(loc, maxrows = 3, filter_gene_biotype = 'protein_coding',
           gene_col = 'grey', exon_col = 'orange', exon_border = 'darkgrey')
```

For advanced users who only want the gene tracks to add to their own plots,
`locus()` can be called without the `data` argument specified (or `data` can be
set to `NULL`). Then genetracks can be plotted in base graphics, ggplot2 or 
plotly.

```{r eval=FALSE}
loc00 <- locus(gene = 'UBE2L3', flank = 1e5, ens_db = "EnsDb.Hsapiens.v75")

genetracks(loc00)  # base graphics
gg_genetracks(loc00)  # ggplot2
genetrack_ly(loc00)  # plotly
```

# Add eQTL data via LDlink

GTEx eQTL data can be accessed via the LDlinkR package which queries the
[LDlink](https://ldlink.nih.gov/?tab=home) server API to download data and add
it to a 'locus' object. The eQTL data is stored in `LDexp` slot in the locus
object.

```{r eval=FALSE}
# obtain GTEx eQTL data from LDlinkR
# needs personal access token for LDlink
loc2 <- locus(SLE_gwas_sub, gene = 'IRF5', flank = c(7e4, 2e5), LD = "r2",
              ens_db = "EnsDb.Hsapiens.v75")
loc2 <- link_eqtl(loc2, token = "..")
head(loc2$LDexp)  # show eQTL data
```

Often at a locus eQTL data may be available on a range of genes and tissues. The
following code shows how to see which genes and tissues are available and how
eQTL SNPs they each have.

```{r eval=FALSE}
table(loc2$LDexp$Gene_Symbol)
##      AHCYL2           IRF5            KCP        METTL2B           NRF1   RP11-212P7.2 
##           5           1618              1              2              9              3 
## ...

table(loc2$LDexp$Tissue)
##             Adipose - Subcutaneous          Adipose - Visceral (Omentum) 
##                                 78                                    69 
##                      Adrenal Gland                        Artery - Aorta 
##                                 38                                   109
## ...
```

Embedded eQTL data can be plotted using `eqtl_plot()` or shown as an overlay
plot using `overlay_plot()` which shows GWAS or association data on the *y* axis
and overlays eQTL results using symbols and colours.

```{r eval=FALSE}
# just plot eQTL results
eqtl_plot(loc2, tissue = "Whole Blood", eqtl_gene = "IRF5")

# overlay eqtl plot
overlay_plot(loc2, eqtl_gene = "IRF5")
```

```{r, out.width='60%', fig.align="center", echo=FALSE}
knitr::include_graphics("eqtl_overlay.png")
```

# Change *y*-axis variable

Instead of plotting -log~10~ p-value on the y axis, it is possible to specify a
different variable in your dataset using the argument `yvar`.

```{r, eval=ensdb_eval, message=FALSE, fig.dim=c(8, 7)}
locb <- locus(SLE_gwas_sub, gene = 'UBE2L3', flank = 1e5, yvar = "beta",
              ens_db = "EnsDb.Hsapiens.v75")
locus_plot(locb)
```

# Arrange multiple locus plots

`locuszoomr` uses `graphics::layout` to arrange plots. To layout multiple locus
plots side by side, use the function `multi_layout()` to set the number of locus
plots per row and column. The `plots` argument in `multi_layout()` can either be
a list of locus class objects, one for each gene. Or for full control it can be
an 'expression' with a series of manual calls to `locus_plot()`. Alternatively a
`for` loop could be called within the `plots` expression.

```{r eval=FALSE}
genes <- c("STAT4", "IRF5", "UBE2L3")

# generate list of 'locus' class objects, one for each gene
loclist <- lapply(genes, locus,
                  data = SLE_gwas_sub,
                  ens_db = "EnsDb.Hsapiens.v75",
                  LD = "r2")

## produce 3 locus plots, one on each page
pdf("myplot.pdf")
multi_layout(loclist)
dev.off()

## place 3 locus plots in a row on a single page
pdf("myplot.pdf")
multi_layout(loclist, ncol = 3, labels = "index")
dev.off()

## full control
loc2 <- locus(SLE_gwas_sub, gene = 'IRF5', flank = c(7e4, 2e5), LD = "r2",
              ens_db = "EnsDb.Hsapiens.v75")
loc3 <- locus(SLE_gwas_sub, gene = 'STAT4', flank = 1e5, LD = "r2",
              ens_db = "EnsDb.Hsapiens.v75")

pdf("myplot.pdf", width = 9, height = 6)
multi_layout(ncol = 3,
             plots = {
               locus_plot(loc, use_layout = FALSE, legend_pos = 'topleft')
               locus_plot(loc2, use_layout = FALSE, legend_pos = NULL)
               locus_plot(loc3, use_layout = FALSE, legend_pos = NULL,
                          labels = "index")
             })
dev.off()
```

```{r, echo=FALSE, message=FALSE, fig.dim = c(9,5)}
if (require(EnsDb.Hsapiens.v75)) {
loc2 <- locus(SLE_gwas_sub, gene = 'IRF5', flank = c(7e4, 2e5), LD = "r2",
              ens_db = "EnsDb.Hsapiens.v75")
loc3 <- locus(SLE_gwas_sub, gene = 'STAT4', flank = 1e5, LD = "r2",
              ens_db = "EnsDb.Hsapiens.v75")

multi_layout(ncol = 3,
             plots = {
               locus_plot(loc, use_layout = FALSE, legend_pos = 'topleft')
               locus_plot(loc2, use_layout = FALSE, legend_pos = NULL)
               locus_plot(loc3, use_layout = FALSE, legend_pos = NULL,
                          labels = "index")
             })
}
```

# Layering plots

## Column of plots

`locuszoomr` has been designed with modular functions to enable layering of
plots on top of each other in a column with gene tracks on the bottom.
`set_layers()` is used to set layout of plots in base graphics by calling
`layout()`. You will need to reset `par()` to original layout settings once the
multi-panel plot is finished.

`scatter_plot()` is used to generate the locus plot. `line_plot()` is used as an
example of an additional plot. Also see `eqtl_plot()` for plotting eQTL
information retrieved via the LDlinkR package.

```{r eval=FALSE}
pdf("myplot2.pdf", width = 6, height = 8)
# set up layered plot with 2 plots & a gene track; store old par() settings
oldpar <- set_layers(2)
scatter_plot(loc, xticks = FALSE)
line_plot(loc, col = "orange", xticks = FALSE)
genetracks(loc)
par(oldpar)  # revert par() settings
dev.off()
```

```{r, echo=FALSE, fig.dim = c(6,8)}
if (require(EnsDb.Hsapiens.v75)) {
oldpar <- set_layers(2)
scatter_plot(loc, xticks = FALSE)
line_plot(loc, col = "orange", xticks = FALSE)
genetracks(loc)
par(oldpar)
}
```

## Overlaid plots

`scatter_plot()` can be called with argument `add = TRUE` to add multiple sets
of points overlaid on one plot.

```{r eval=FALSE}
dat <- SLE_gwas_sub
dat$p2 <- -log10(dat$p * 0.1)
locp <- locus(dat, gene = 'UBE2L3', flank = 1e5, ens_db = "EnsDb.Hsapiens.v75")
locp2 <- locus(dat, gene = 'UBE2L3', flank = 1e5, yvar = "p2",
               ens_db = "EnsDb.Hsapiens.v75")

oldpar <- set_layers(1)
scatter_plot(locp, xticks = FALSE, pcutoff = NULL, ylim = c(0, 16))
scatter_plot(locp2, xticks = FALSE, pcutoff = NULL, scheme = "orange",
             pch = 22, add = TRUE)
genetracks(loc)
par(oldpar)
```

```{r, echo=FALSE, message=FALSE, fig.dim = c(7,7)}
if (require(EnsDb.Hsapiens.v75)) {
dat <- SLE_gwas_sub
dat$p2 <- -log10(dat$p * 0.1)
locp <- locus(dat, gene = 'UBE2L3', flank = 1e5, ens_db = "EnsDb.Hsapiens.v75")
locp2 <- locus(dat, gene = 'UBE2L3', flank = 1e5, yvar = "p2",
               ens_db = "EnsDb.Hsapiens.v75")

oldpar <- set_layers(1)
scatter_plot(locp, xticks = FALSE, pcutoff = NULL, ylim = c(0, 16))
scatter_plot(locp2, xticks = FALSE, pcutoff = NULL, scheme = "orange",
             pch = 22, add = TRUE)
genetracks(loc)
par(oldpar)
}
```

# Add custom legend / features

The power of base graphics is that it gives complete control over plotting. In
the example below, we show how to add your own legend, text labels, lines to
demarcate a gene and extra points on top or underneath the main plot. When
`plot()` is called, base graphics allows additional plotting using the arguments
`panel.first` and `panel.last`. Since these are called inside the `locus_plot()`
function they need to be quoted using `quote()`.

```{r, eval=ensdb_eval, fig.dim = c(7,7)}
# add vertical lines for gene of interest under the main plot
pf <- quote({
  v <- locp$TX[locp$TX$gene_name == "UBE2L3", c("start", "end")]
  abline(v = v, col = "orange")
})

pl <- quote({
  # add custom text label for index SNP
  lx <- locp$data$pos[locp$data$rsid == locp$index_snp]
  ly <- locp$data$logP[locp$data$rsid == locp$index_snp]
  text(lx, ly, locp$index_snp, pos = 4, cex = 0.8)
  # add extra points
  px <- rep(22.05e6, 3)
  py <- 10:12
  points(px, py, pch = 21, bg = "green")
  # add custom legend
  legend("topleft", legend = c("group A", "group B"),
         pch = 21, pt.bg = c("blue", "green"), bty = "n")
})

locus_plot(locp, pcutoff = NULL, panel.first = pf, panel.last = pl)
```

# ggplot2 version

`ggplot2` versions of several of the above functions are available. For a whole
locus plot use `locus_ggplot()`.

```{r eval=FALSE}
locus_ggplot(loc)
```

The gene tracks can be produced on their own as a ggplot2 object using
`gg_genetracks()`.

```{r eval=FALSE}
gg_genetracks(loc)
```

The scatter plot alone can be produced as a ggplot2 object.

```{r eval=FALSE}
p <- gg_scatter(loc)
p
```

Finally, `gg_addgenes()` can be used to add gene tracks to an existing ggplot2
plot that has been previously created and customised.

```{r eval=FALSE}
gg_addgenes(p, loc)
```

If you are using ggplot2, the easiest way to edit the scatter plot and change
legend and other details, is to use `gg_scatter()` and modify the plot using the
ggplot2 `+` notation. For example, to increase the font size of the legend you
could do the following:

```{r eval=FALSE}
p <- gg_scatter(loc) +
  theme(legend.text = element_text(size = 14),
        legend.title = element_text(size = 14))
gg_addgenes(p, loc)
```

## Shapes

Shapes can be overlaid in the scatter plot by setting either `beta` or `shape`
to a column name in the dataset. Setting `beta` shows up-triangles for positive
beta values and down-triangles for negative beta values. If `shape` is set, for
example to show imputed vs typed SNPs, the column needs to be a factor (to make
the legend nice). `shape_values` is then set as a vector of shapes to which the
levels of the factor are mapped on the scatter plot.

```{r eval=FALSE}
locus_ggplot(loc, beta = "beta")

# add column 'typed' as a factor with levels "typed", "imputed"
loc$data$typed <- factor(rbinom(n = nrow(loc$data), 1, 0.3), labels = c("imputed", "typed"))
locus_ggplot(loc, shape = "typed")
```

## Layering plots

Plots can be layered above gene tracks, using `cowplot::plot_grid()` or using
the `patchwork` package.

```{r eval=FALSE}
g <- gg_genetracks(loc)
library(cowplot)
plot_grid(p, p, g, ncol = 1, rel_heights = c(2, 2, 1), align = "v")

# patchwork method
library(patchwork)
p / p / g

# patchwork method 2
wrap_plots(p, p, g, ncol = 1)
```

For users who prefer the grid system we recommend also looking at Bioconductor
packages `Gviz` or `ggbio` as possible alternatives.

## Arrange multiple ggplots

It is possible to use either the `cowplot` or `ggpubr` packages to layout
multiple locus ggplots on the same page.


```{r eval=ensdb_eval, fig.dim = c(12, 7)}
p1 <- locus_ggplot(loc, labels = "index", nudge_x = 0.03)
p2 <- locus_ggplot(loc2, legend_pos = NULL)
cowplot::plot_grid(p1, p2, ncol = 2)
```

Or using ggpubr or gridExtra:

```{r eval=FALSE}
library(ggpubr)
pdf("my_ggplot.pdf", width = 10)
ggarrange(p1, p2, ncol = 2)
dev.off()

library(gridExtra)
pdf("my_ggplot.pdf", width = 10)
grid.arrange(p1, p2, ncol = 2)
dev.off()
```

A known issue with the ggplot2 version `locus_ggplot()` is that the system for
ensuring that gene labels seems to go wrong when the plot window is resized
during arrangement of multiple plots. The workable solution at present is to
make sure that the height and width of the final pdf/output is enlarged as per
the example above, which plots fine without any text overlapping when exported
to pdf at an appropriate size.

# Plotly version

`locuszoomr` includes a ['plotly'](https://plotly.com/r/) version for plotting
locus plots which is interactive. This allows users to hover over the plot and
reveal additional information such as SNP rs IDs for each point and information
about each gene in the gene tracks. This can help when exploring a locus or
region and trying to identify particular SNPs (or genomic features) of interest.

```{r eval=FALSE}
locus_plotly(loc2)
```

```{r, out.width='70%', fig.align="center", echo=FALSE}
knitr::include_graphics("plotly.png")
```

# Plot all GWAS peaks

`locuszoomr` includes a function `quick_peak()` to quickly find GWAS peaks based
on a set p-value threshold (default 5e-08) and specifying a minimum distance
between peaks (default 1 Mb). The function will either find all peaks ordered by
lowest p-value, or the top *n* peaks can be identified by setting `npeaks`.

```{r eval=FALSE}
# FTP download full summary statistics of this SLE GWAS from
# https://www.ebi.ac.uk/gwas/studies/GCST003156
library(data.table)
SLE_gwas <- fread('../bentham_2015_26502338_sle_efo0002690_1_gwas.sumstats.tsv')

pks <- quick_peak(SLE_gwas)
## 34 peaks found (0.328 secs)

top_snps <- SLE_gwas$rsid[pks]
head(top_snps)
## [1] "rs141910407" "rs114056368" "rs4274624"   "rs17849501"  "rs1143679"   "rs114115096"
```

We can now plot locus plots for all the GWAS peaks as a single pdf, as specified
by the vector of top SNPs. The `link_recomb()` line below can be sped up by
downloading the recombination track file from UCSC - see example code in 
[Add recombination rate](#add-recombination-rate).

```{r eval=FALSE}
all_loci <- lapply(top_snps, function(i) {
  loc <- locus(data = SLE_gwas, index_snp = i, fix_window = 1e6,
        ens_db = "EnsDb.Hsapiens.v75")
  link_recomb(loc)
})

pdf("sle_loci.pdf")
tmp <- lapply(all_loci, locus_plot, labels = "index")
dev.off()
```

Alternatively export plots using ggplot2.

```{r eval=FALSE}
pp <- lapply(all_loci, locus_ggplot, labels = "index", nudge_y = 1)

library(gridExtra)
pdf("sle_loci_gg.pdf")
for (i in seq_along(pp)) {
  grid.arrange(pp[[i]])
}
dev.off()
```

# Manhattan & other plots

For Manhattan plots, log p-value QQ plot and easy labelling of volcano plots or
other scatter plots with draggable labels, check out our sister package
`easylabel` on CRAN at https://cran.r-project.org/package=easylabel.

# Citation

If you use this package please cite as:

Lewis MJ, Wang S. (2025) locuszoomr: an R package for visualising
publication-ready regional gene locus plots. *Bioinformatics Advances* 2025;
vbaf006, [doi:10.1093/bioadv/vbaf006](https://doi.org/10.1093/bioadv/vbaf006)

# References

Pruim RJ, Welch RP, Sanna S, Teslovich TM, Chines PS, Gliedt TP, Boehnke M,
Abecasis GR, Willer CJ. (2010) LocusZoom: Regional visualization of genome-wide
association scan results. *Bioinformatics* 2010; 26(18): 2336-7.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/link_eqtl.R
\name{link_eqtl}
\alias{link_eqtl}
\title{Obtain GTEx eQTL data via LDlinkR}
\usage{
link_eqtl(loc, pop = "CEU", r2d = "r2", token = "", ...)
}
\arguments{
\item{loc}{Object of class 'locus' generated by \code{\link[=locus]{locus()}}}

\item{pop}{A 1000 Genomes Project population, (e.g. YRI or CEU), multiple
allowed, default = "CEU". Passed to \code{LDlinkR::LDexpress()}.}

\item{r2d}{Either "r2" for LD r^2 or "d" for LD D', default = "r2". Passed
to \code{LDlinkR::LDexpress()}.}

\item{token}{Personal access token for accessing 1000 Genomes LD data via
LDlink API. See \code{LDlinkR} package documentation.}

\item{...}{Optional arguments such as \code{genome_build} which are passed on to
\code{LDlinkR::LDexpress()}}
}
\value{
Returns an object of class 'locus' with an extra list element 'LDexp'
containing a dataframe of information obtained via \code{LDexpress()}.
}
\description{
Adds eQTL (expression quantitative trait loci) information from GTEx
(https://gtexportal.org/) to a 'locus' class object. It queries LDlink
(https://ldlink.nci.nih.gov/) via the \code{LDlinkR} package to retrieve GTEx eQTL
information on a reference SNP.
}
\details{
The additional eQTL information obtained from LDlink web server can be
displayed using \code{\link[=eqtl_plot]{eqtl_plot()}} which generates a scatter plot with gene tracks
similar to a locus plot, or with \code{\link[=overlay_plot]{overlay_plot()}} which tries to overlay the
EQTL analysis over the original locus results (e.g. GWAS).
}
\seealso{
\code{\link[=locus]{locus()}} \code{\link[=eqtl_plot]{eqtl_plot()}} \code{\link[=overlay_plot]{overlay_plot()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/overlay_plot.R
\name{overlay_plot}
\alias{overlay_plot}
\title{Plot overlaying eQTL and GWAS data}
\usage{
overlay_plot(
  loc,
  base_col = "black",
  alpha = 0.5,
  scheme = "RdYlBu",
  tissue = "Whole Blood",
  eqtl_gene = loc$gene,
  legend_pos = "topright",
  ...
)
}
\arguments{
\item{loc}{Object of class 'locus' to use for plot. See \code{\link[=locus]{locus()}}.}

\item{base_col}{Colour of points for SNPs which do not have eQTLs.}

\item{alpha}{Alpha opacity for non-eQTL points}

\item{scheme}{Character string specifying palette for effect size showing
up/downregulation eQTL using \link[grDevices:hcl.colors]{grDevices::hcl.colors}. Alternatively a
vector of 6 colours.}

\item{tissue}{GTex tissue in which eQTL has been measured}

\item{eqtl_gene}{Gene showing eQTL effect}

\item{legend_pos}{Character value specifying legend position. See \code{\link[=legend]{legend()}}.}

\item{...}{Other arguments passed to \code{\link[=locus_plot]{locus_plot()}} for the locus plot.}
}
\value{
No return value. Produces a plot using base graphics.
}
\description{
Experimental plotting function for overlaying eQTL data from GTEx on top of
GWAS results. y axis shows the -log10 p-value for the GWAS result.
Significant eQTL for the specified gene are overlaid using colours and
symbols.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_genetracks.R
\name{gg_genetracks}
\alias{gg_genetracks}
\title{Plot gene tracks}
\usage{
gg_genetracks(
  loc,
  filter_gene_name = NULL,
  filter_gene_biotype = NULL,
  border = FALSE,
  cex.axis = 1,
  cex.lab = 1,
  cex.text = 0.7,
  gene_col = ifelse(showExons, "blue4", "skyblue"),
  exon_col = "blue4",
  exon_border = "blue4",
  showExons = TRUE,
  maxrows = NULL,
  text_pos = "top",
  italics = FALSE,
  xticks = TRUE,
  xlab = NULL,
  highlight = NULL,
  highlight_col = "red",
  prioritise = highlight,
  blanks = c("fill", "hide")
)
}
\arguments{
\item{loc}{Object of class 'locus' generated by \code{\link[=locus]{locus()}}.}

\item{filter_gene_name}{Vector of gene names to display.}

\item{filter_gene_biotype}{Vector of gene biotypes to be filtered. Use
\code{\link[ensembldb:listGenebiotypes]{ensembldb::listGenebiotypes()}} to display possible biotypes. For example,
\code{ensembldb::listGenebiotypes(EnsDb.Hsapiens.v75)}}

\item{border}{Logical whether a bounding box is plotted.}

\item{cex.axis}{Specifies font size for axis numbering.}

\item{cex.lab}{Specifies font size for axis titles.}

\item{cex.text}{Font size for gene text.}

\item{gene_col}{Colour for gene lines.}

\item{exon_col}{Fill colour for exons.}

\item{exon_border}{Border line colour outlining exons (or genes if
\code{showExons} is \code{FALSE}). Set to \code{NA} for no border.}

\item{showExons}{Logical whether to show exons or simply show whole gene as a
rectangle. If \code{showExons = FALSE} colours are specified by \code{exon_border}
for rectangle border and \code{gene_col} for the fill colour.}

\item{maxrows}{Specifies maximum number of rows to display in gene annotation
panel.}

\item{text_pos}{Character value of either 'top' or 'left' specifying
placement of gene name labels.}

\item{italics}{Logical whether gene text is in italics.}

\item{xticks}{Logical whether x axis ticks and numbers are plotted.}

\item{xlab}{Title for x axis. Defaults to chromosome \code{seqname} specified in
\code{locus}.}

\item{highlight}{Vector of genes to highlight.}

\item{highlight_col}{Single colour or vector of colours for highlighted
genes.}

\item{prioritise}{Vector of genes to be placed first in the gene tracks.}

\item{blanks}{Controls handling of genes with blank names: \code{"fill"} replaces
blank gene symbols with ensembl gene ids. \code{"hide"} hides genes which are
missing gene symbols.}
}
\value{
A ggplot2 object.
}
\description{
Plot gene annotation tracks from \code{ensembldb} data using ggplot2 and grid.
}
\details{
This function is called by \code{\link[=locus_ggplot]{locus_ggplot()}}, and in turn it calls
\code{\link[=genetracks_grob]{genetracks_grob()}}. It can be used to plot the gene annotation tracks on
their own as a ggplot2 object.

\code{gene_col}, \code{exon_col} and \code{exon_border} set colours for all genes, while
\code{highlight} and \code{highlight_col} can optionally be used together to highlight
specific genes of interest. For full control over every single gene, users
can add columns \code{gene_col}, \code{exon_col} and \code{exon_border} to the \code{TX} object
within the 'locus' object. Columns added to \code{TX} override their equivalent
arguments.
}
\examples{
if(require(EnsDb.Hsapiens.v75)) {
data(SLE_gwas_sub)
loc <- locus(SLE_gwas_sub, gene = 'IRF5', flank = c(7e4, 2e5), LD = "r2",
             ens_db = "EnsDb.Hsapiens.v75")
gg_genetracks(loc)
}
}
\seealso{
\code{\link[=locus_ggplot]{locus_ggplot()}} \code{\link[=genetracks_grob]{genetracks_grob()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_scatter.R
\name{gg_scatter}
\alias{gg_scatter}
\title{Locus scatter plot using ggplot2}
\usage{
gg_scatter(
  loc,
  index_snp = loc$index_snp,
  pcutoff = 5e-08,
  scheme = c("grey", "dodgerblue", "red"),
  size = 2,
  cex.axis = 1,
  cex.lab = 1,
  xlab = NULL,
  ylab = NULL,
  ylim = NULL,
  ylim2 = c(0, 100),
  yzero = (loc$yvar == "logP"),
  xticks = TRUE,
  border = FALSE,
  showLD = TRUE,
  LD_scheme = c("grey", "royalblue", "cyan2", "green3", "orange", "red", "purple"),
  recomb_col = "blue",
  recomb_linewidth = 0.35,
  recomb_offset = 0,
  legend_pos = "topleft",
  labels = NULL,
  eqtl_gene = NULL,
  beta = NULL,
  shape = NULL,
  shape_values = c(21, 24, 25),
  ...
)
}
\arguments{
\item{loc}{Object of class 'locus' to use for plot. See \link{locus}.}

\item{index_snp}{Specifies index SNP to be shown in a different colour and
symbol. Defaults to the SNP with the lowest p-value. Set to \code{NULL} to not
show this.}

\item{pcutoff}{Cut-off for p value significance. Defaults to p = 5e-08. Set
to \code{NULL} to disable.}

\item{scheme}{Vector of 3 colours if LD is not shown: 1st = normal points,
2nd = colour for significant points, 3rd = index SNP.}

\item{size}{Specifies size for points.}

\item{cex.axis}{Specifies font size for axis numbering.}

\item{cex.lab}{Specifies font size for axis titles.}

\item{xlab}{x axis title.}

\item{ylab}{y axis title.}

\item{ylim}{y axis limits (y1, y2).}

\item{ylim2}{Secondary y axis limits for recombination line.}

\item{yzero}{Logical whether to force y axis limit to include y=0.}

\item{xticks}{Logical whether x axis numbers and axis title are plotted.}

\item{border}{Logical whether a bounding box is plotted around the plot.}

\item{showLD}{Logical whether to show LD with colours}

\item{LD_scheme}{Vector of colours for plotting LD. The first colour is for SNPs
which lack LD information. The next 5 colours are for r2 or D' LD results
ranging from 0 to 1 in intervals of 0.2. The final colour is for the index
SNP.}

\item{recomb_col}{Colour for recombination rate line if recombination rate
data is present. Set to NA to hide the line. See \code{\link[=link_recomb]{link_recomb()}} to add
recombination rate data.}

\item{recomb_linewidth}{Line width for recombination rate line.}

\item{recomb_offset}{Offset from 0-1 which shifts the scatter plot up and
recombination line plot down. Recommended value 0.1.}

\item{legend_pos}{Position of legend. Set to \code{NULL} to hide legend.}

\item{labels}{Character vector of SNP or genomic feature IDs to label. The
value "index" selects the highest point or index SNP as defined when
\code{\link[=locus]{locus()}} is called. Set to \code{NULL} to remove all labels.}

\item{eqtl_gene}{Optional column name in \code{loc$data} for colouring eQTL genes.}

\item{beta}{Optional column name for beta coefficient to display upward
triangles for positive beta and downward triangles for negative beta
(significant SNPs only).}

\item{shape}{Optional column name in \code{loc$data} for controlling shapes.
\code{beta} and \code{shape} cannot both be set. This column is expected to be a factor.}

\item{shape_values}{Vector of shape values which match levels of the column
specified by \code{shape}. This vector is passed to
\code{ggplot2::scale_shape_manual()} as the argument \code{values}. See \code{\link[=points]{points()}}
for a list of shapes and the numbers they map to.}

\item{...}{Optional arguments passed to \code{geom_text_repel()} to configure
label drawing.}
}
\value{
Returns a ggplot2 plot.
}
\description{
Produces a scatter plot from a 'locus' class object (without gene tracks).
}
\details{
If recombination rate data is included in the locus object following a call
to \code{\link[=link_recomb]{link_recomb()}}, this is plotted as an additional line with a secondary y
axis. In the base graphics version the line is placed under the scatter
points, but this is not possible with ggplot2 as the secondary y axis data
must be plotted on top of the primary scatter point data.
}
\examples{
if (require(EnsDb.Hsapiens.v75)) {
    data(SLE_gwas_sub)
    loc <- locus(SLE_gwas_sub,
        gene = "IRF5", flank = c(7e4, 2e5), LD = "r2",
        ens_db = "EnsDb.Hsapiens.v75"
    )
    gg_scatter(loc)
}
}
\seealso{
\code{\link[=locus]{locus()}} \code{\link[=gg_addgenes]{gg_addgenes()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scatter_plot.R
\name{scatter_plot}
\alias{scatter_plot}
\title{Locus scatter plot}
\usage{
scatter_plot(
  loc,
  index_snp = loc$index_snp,
  pcutoff = 5e-08,
  scheme = c("grey", "dodgerblue", "red"),
  cex = 1,
  cex.axis = 0.9,
  cex.lab = 1,
  xlab = NULL,
  ylab = NULL,
  ylim = NULL,
  ylim2 = c(0, 100),
  yzero = (loc$yvar == "logP"),
  xticks = TRUE,
  border = FALSE,
  showLD = TRUE,
  LD_scheme = c("grey", "royalblue", "cyan2", "green3", "orange", "red", "purple"),
  recomb_col = "blue",
  recomb_offset = 0,
  legend_pos = "topleft",
  labels = NULL,
  label_x = 4,
  label_y = 4,
  eqtl_gene = NULL,
  beta = NULL,
  add = FALSE,
  align = TRUE,
  ...
)
}
\arguments{
\item{loc}{Object of class 'locus' to use for plot. See \link{locus}.}

\item{index_snp}{Specifies index SNP or a vector of SNPs to be shown in a
different colour and symbol. Defaults to the SNP with the lowest p-value.
Set to \code{NULL} to not show this.}

\item{pcutoff}{Cut-off for p value significance. Defaults to p = 5e-08. Set
to \code{NULL} to disable.}

\item{scheme}{Vector of 3 colours if LD is not shown: 1st = normal points,
2nd = colour for significant points, 3rd = index SNP(s).}

\item{cex}{Specifies size for points.}

\item{cex.axis}{Specifies font size for axis numbering.}

\item{cex.lab}{Specifies font size for axis titles.}

\item{xlab}{x axis title.}

\item{ylab}{y axis title.}

\item{ylim}{y axis limits (y1, y2).}

\item{ylim2}{Secondary y axis limits for recombination line, if present.}

\item{yzero}{Logical whether to force y axis limit to include y=0.}

\item{xticks}{Logical whether x axis numbers and axis title are plotted.}

\item{border}{Logical whether a bounding box is plotted around upper and
lower plots.}

\item{showLD}{Logical whether to show LD with colours}

\item{LD_scheme}{Vector of colours for plotting LD. The first colour is for
SNPs which lack LD information. The next 5 colours are for r2 or D' LD
results ranging from 0 to 1 in intervals of 0.2. The final colour is for
the index SNP.}

\item{recomb_col}{Colour for recombination rate line if recombination rate
data is present. Set to \code{NA} to hide the line. See \code{\link[=link_recomb]{link_recomb()}} to add
recombination rate data.}

\item{recomb_offset}{Offset from 0-1 which shifts the scatter plot up and
recombination line plot down. Recommended value 0.1.}

\item{legend_pos}{Position of legend. See \code{\link[=legend]{legend()}}. Set to \code{NULL} to hide
legend.}

\item{labels}{Character vector of SNP or genomic feature IDs to label. The
value "index" selects the highest point or index SNP as defined when
\code{\link[=locus]{locus()}} is called. Set to \code{NULL} to remove all labels.}

\item{label_x}{Value or vector for position of label as percentage of x axis
scale.}

\item{label_y}{Value or vector for position of label as percentage of y axis
scale.}

\item{eqtl_gene}{Column name in \code{loc$data} for colouring eQTL genes.}

\item{beta}{Optional column name for beta coefficient to display upward
triangles for positive beta and downward triangles for negative beta
(significant SNPs only).}

\item{add}{Logical whether to add points to an existing plot or generate a
new plot.}

\item{align}{Logical whether to set \code{\link[=par]{par()}} to align the plot.}

\item{...}{Other arguments passed to \code{\link[=plot]{plot()}} to control the scatter plot
e.g. \code{main}, \code{ylim} etc.}
}
\value{
No return value. Produces a scatter plot using base graphics.
}
\description{
Produces a base graphics scatter plot from a 'locus' class object. This
function is called by \code{\link[=locus_plot]{locus_plot()}} to generate the scatter plot portion.
Can be used manually with \code{\link[=set_layers]{set_layers()}}.
}
\details{
Advanced users familiar with base graphics can customise every single point
on the scatter plot, by adding columns named \code{bg}, \code{col}, \code{pch} or \code{cex}
directly to the dataframe stored in \verb{$data} element of the 'locus' object.
Setting these will overrule any default settings. These columns refer to
their respective base graphics arguments, see \code{\link[graphics:points]{graphics::points()}}.
}
\seealso{
\code{\link[=locus]{locus()}} \code{\link[=set_layers]{set_layers()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/locus_ggplot.R
\name{locus_ggplot}
\alias{locus_ggplot}
\title{Locus plot using ggplot2}
\usage{
locus_ggplot(
  loc,
  heights = c(3, 2),
  filter_gene_name = NULL,
  filter_gene_biotype = NULL,
  border = FALSE,
  cex.axis = 1,
  cex.lab = 1,
  cex.text = 0.7,
  gene_col = ifelse(showExons, "blue4", "skyblue"),
  exon_col = "blue4",
  exon_border = "blue4",
  showExons = TRUE,
  maxrows = 12,
  text_pos = "top",
  italics = FALSE,
  xticks = "top",
  xlab = NULL,
  highlight = NULL,
  highlight_col = "red",
  prioritise = highlight,
  blanks = "fill",
  ...
)
}
\arguments{
\item{loc}{Object of class 'locus' to use for plot. See \code{\link[=locus]{locus()}}.}

\item{heights}{Vector supplying the ratio of top to bottom plot.}

\item{filter_gene_name}{Vector of gene names to display.}

\item{filter_gene_biotype}{Vector of gene biotypes to be filtered. Use
\code{\link[ensembldb:listGenebiotypes]{ensembldb::listGenebiotypes()}} to display possible biotypes. For example,
\code{ensembldb::listGenebiotypes(EnsDb.Hsapiens.v75)}}

\item{border}{Logical whether a bounding box is plotted.}

\item{cex.axis}{Specifies font size for axis numbering.}

\item{cex.lab}{Specifies font size for axis titles.}

\item{cex.text}{Font size for gene text.}

\item{gene_col}{Colour for gene lines.}

\item{exon_col}{Fill colour for exons.}

\item{exon_border}{Border line colour outlining exons (or genes if
\code{showExons} is \code{FALSE}). Set to \code{NA} for no border.}

\item{showExons}{Logical whether to show exons or simply show whole gene as a
rectangle. If \code{showExons = FALSE} colours are specified by \code{exon_border}
for rectangle border and \code{gene_col} for the fill colour.}

\item{maxrows}{Specifies maximum number of rows to display in gene annotation
panel.}

\item{text_pos}{Character value of either 'top' or 'left' specifying
placement of gene name labels.}

\item{italics}{Logical whether gene text is in italics.}

\item{xticks}{Logical whether x axis ticks and numbers are plotted.}

\item{xlab}{Title for x axis. Defaults to chromosome \code{seqname} specified in
\code{locus}.}

\item{highlight}{Vector of genes to highlight.}

\item{highlight_col}{Single colour or vector of colours for highlighted
genes.}

\item{prioritise}{Vector of genes to be placed first in the gene tracks.}

\item{blanks}{Controls handling of genes with blank names: \code{"fill"} replaces
blank gene symbols with ensembl gene ids. \code{"hide"} hides genes which are
missing gene symbols.}

\item{...}{Additional arguments passed to \code{\link[=gg_scatter]{gg_scatter()}} to control
the scatter plot, e.g. \code{pcutoff}, \code{scheme}, \code{recomb_offset} etc.}
}
\value{
Returns a ggplot2 plot containing a scatter plot with genetracks
underneath.
}
\description{
Genomic locus plot similar to locuszoom.
}
\details{
Arguments to control plotting of the gene tracks are passed onto
\code{\link[=gg_genetracks]{gg_genetracks()}} and for the scatter plot are passed via \code{...} to
\code{\link[=gg_scatter]{gg_scatter()}}. See the documentation for each of these functions for
details.
}
\examples{
if(require(EnsDb.Hsapiens.v75)) {
data(SLE_gwas_sub)
loc <- locus(SLE_gwas_sub, gene = 'IRF5', flank = c(7e4, 2e5), LD = "r2",
             ens_db = "EnsDb.Hsapiens.v75")
locus_ggplot(loc)
}
}
\seealso{
\code{\link[=gg_scatter]{gg_scatter()}} \code{\link[=gg_genetracks]{gg_genetracks()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scatter_plotly.R
\name{scatter_plotly}
\alias{scatter_plotly}
\title{Locus scatter plotly}
\usage{
scatter_plotly(
  loc,
  index_snp = loc$index_snp,
  pcutoff = 5e-08,
  scheme = c("grey", "dodgerblue", "red"),
  xlab = NULL,
  ylab = NULL,
  yzero = (loc$yvar == "logP"),
  showLD = TRUE,
  LD_scheme = c("grey", "royalblue", "cyan2", "green3", "orange", "red", "purple"),
  marker_outline = "black",
  marker_size = 7,
  recomb_col = "blue",
  eqtl_gene = NULL,
  beta = NULL,
  add_hover = NULL,
  showlegend = TRUE,
  height = NULL,
  webGL = TRUE
)
}
\arguments{
\item{loc}{Object of class 'locus' to use for plot. See \link{locus}.}

\item{index_snp}{Specifies index SNP or a vector of SNPs to be shown in a
different colour and symbol. Defaults to the SNP with the lowest p-value.
Set to \code{NULL} to not show this.}

\item{pcutoff}{Cut-off for p value significance. Defaults to p = 5e-08. Set
to \code{NULL} to disable.}

\item{scheme}{Vector of 3 colours if LD is not shown: 1st = normal points,
2nd = colour for significant points, 3rd = index SNP(s).}

\item{xlab}{x axis title.}

\item{ylab}{y axis title.}

\item{yzero}{Logical whether to force y axis limit to include y=0.}

\item{showLD}{Logical whether to show LD with colours}

\item{LD_scheme}{Vector of colours for plotting LD. The first colour is for
SNPs which lack LD information. The next 5 colours are for r^2 or D' LD
results ranging from 0 to 1 in intervals of 0.2. The final colour is for
the index SNP.}

\item{marker_outline}{Specifies colour for outlining points.}

\item{marker_size}{Value for size of markers in plotly units.}

\item{recomb_col}{Colour for recombination rate line if recombination rate
data is present. Set to \code{NA} to hide the line. See \code{\link[=link_recomb]{link_recomb()}} to add
recombination rate data.}

\item{eqtl_gene}{Column name in \code{loc$data} for eQTL genes.}

\item{beta}{Optional column name for beta coefficient to display upward
triangles for positive beta and downward triangles for negative beta
(significant SNPs only).}

\item{add_hover}{Optional vector of column names in \code{loc$data} to add to the
plotly hover text for scatter points.}

\item{showlegend}{Logical whether to show a legend for the scatter points.}

\item{height}{Height in pixels (optional, defaults to automatic sizing).}

\item{webGL}{Logical whether to use webGL or SVG for scatter plot.}
}
\value{
A \code{plotly} scatter plot.
}
\description{
Produces a scatter plot from a 'locus' class object using plotly.
}
\seealso{
\code{\link[=locus]{locus()}} \code{\link[=locus_plotly]{locus_plotly()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multi_layout.R
\name{multi_layout}
\alias{multi_layout}
\title{Layout multiple locus plots}
\usage{
multi_layout(
  plots,
  nrow = 1,
  ncol = 1,
  heights = c(3, 2),
  legend_pos = "topleft",
  ...
)
}
\arguments{
\item{plots}{Either an 'expression' to be evaluated which is a series of
calls to \code{\link[=locus_plot]{locus_plot()}} or similar plotting functions, or a list of 'locus'
class objects which are plotted in sequence.}

\item{nrow}{Number of rows of plots}

\item{ncol}{Number of columns of plots}

\item{heights}{Vector of length 2 specifying height for plot and gene tracks}

\item{legend_pos}{A keyword either "topleft" or "topright" or \code{NULL} to hide
the legend. Not invoked if \code{plots} is an expression. The legend is only
shown on one plot on each page.}

\item{...}{Optional arguments passed to \code{\link[=locus_plot]{locus_plot()}} if \code{plots} contains a
list}
}
\value{
No return value.
}
\description{
Produces pages with multiple locus plots on.
}
\examples{
if(require(EnsDb.Hsapiens.v75)) {

data(SLE_gwas_sub)
genes <- c("STAT4", "UBE2L3", "IRF5")
loclist <- lapply(genes, locus,
                  data = SLE_gwas_sub,
                  ens_db = "EnsDb.Hsapiens.v75",
                  LD = "r2")
## produce 3 locus plots, one on each page
multi_layout(loclist)

## place 3 locus plots in a row on a single page
multi_layout(loclist, ncol = 3)

## full control
loc <- locus(SLE_gwas_sub, gene = 'STAT4', flank = 1e5, LD = "r2",
             ens_db = "EnsDb.Hsapiens.v75")
loc2 <- locus(SLE_gwas_sub, gene = 'IRF5', flank = c(7e4, 2e5), LD = "r2",
              ens_db = "EnsDb.Hsapiens.v75")
loc3 <- locus(SLE_gwas_sub, gene = 'UBE2L3', LD = "r2",
              ens_db = "EnsDb.Hsapiens.v75")
multi_layout(ncol = 3,
             plots = {
               locus_plot(loc, use_layout = FALSE, legend_pos = 'topleft')
               locus_plot(loc2, use_layout = FALSE, legend_pos = NULL)
               locus_plot(loc3, use_layout = FALSE, legend_pos = NULL)
             })

}
}
\seealso{
\code{\link[=locus_plot]{locus_plot()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/link_LD.R
\name{link_LD}
\alias{link_LD}
\title{Obtain LD at a locus from LDlink}
\usage{
link_LD(
  loc,
  pop = "CEU",
  r2d = "r2",
  token = "",
  method = c("proxy", "matrix"),
  genome_build = loc$genome,
  ...
)
}
\arguments{
\item{loc}{Object of class 'locus' generated by \code{\link[=locus]{locus()}}}

\item{pop}{A 1000 Genomes Project population, (e.g. YRI or CEU), multiple
allowed, default = "CEU". Passed to \code{LDlinkR::LDmatrix()}.}

\item{r2d}{Either "r2" for LD r^2 or "d" for LD D', default = "r2". Passed
to \code{LDlinkR::LDmatrix()} or \code{LDproxy()}.}

\item{token}{Personal access token for accessing 1000 Genomes LD data via
LDlink API. See \code{LDlinkR} package documentation.}

\item{method}{Either \code{"proxy"} or \code{"matrix"}. Controls whether to use
\code{LDproxy()} or \code{LDmatrix()} to obtain LD data.}

\item{genome_build}{Choose between one of the three options: 'grch37' for
genome build GRCh37 (hg19), 'grch38' for GRCh38 (hg38), or
'grch38_high_coverage' for GRCh38 High Coverage (hg38) 1000 Genome Project
data sets. Default is GRCh37 (hg19).}

\item{...}{Optional arguments which are passed on to \code{LDlinkR::LDmatrix()} or
\code{LDlinkR::LDproxy()}}
}
\value{
Returns a list object of class 'locus'. LD information is added as a
column \code{ld} in list element \code{data}.
}
\description{
Adds LD information to a 'locus' class object. It queries LDlink
(https://ldlink.nci.nih.gov/) via the \code{LDlinkR} package to retrieve linkage
disequilibrium (LD) information on a reference SNP.
}
\details{
The argument \code{method} controls which LDlinkR function is used to retrieve LD
data. \code{LDmatrix()} is slower but usually more complete for small queries
(<1000 SNPs). However, it has a limit of 1000 SNPs which can be queried.
\code{LDproxy()} is faster but data on some SNPs may be absent.

Note, SNPs have to be correctly formatted as required by LDlinkR, either as
rsID (works with either method) or chromosome coordinate e.g. "chr7:24966446"
(works with LDproxy only). Default genome build is \code{grch37}, see \code{LDproxy()}
or \code{LDmatrix()}.
}
\seealso{
\code{\link[=locus]{locus()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SLE_gwas_sub.R
\docType{data}
\name{SLE_gwas_sub}
\alias{SLE_gwas_sub}
\title{SLE GWAS data subset}
\format{
Data frame with 1990 rows and 11 variables
}
\source{
\url{https://www.ebi.ac.uk/gwas/studies/GCST003156}
}
\usage{
data(SLE_gwas_sub)
}
\description{
Dataset of SNPs at 3 gene loci (UBE2L3, STAT4, IRF5) from GWAS on SLE
(Bentham et al, 2015, Nature Genetics 47(12):1457-64, PMID: 26502338).
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/locus_plot.R
\name{locus_plot}
\alias{locus_plot}
\title{Locus plot}
\usage{
locus_plot(
  loc,
  filter_gene_name = NULL,
  filter_gene_biotype = NULL,
  xlab = NULL,
  cex = 1,
  cex.axis = 0.9,
  cex.lab = 1,
  cex.text = 0.7,
  use_layout = TRUE,
  heights = c(3, 2),
  showExons = TRUE,
  maxrows = 7,
  xticks = "bottom",
  border = FALSE,
  gene_col = ifelse(showExons, "blue4", "skyblue"),
  exon_col = "blue4",
  exon_border = "blue4",
  text_pos = "top",
  italics = FALSE,
  highlight = NULL,
  highlight_col = "red",
  prioritise = highlight,
  blanks = "fill",
  recomb_col = "blue",
  ...
)
}
\arguments{
\item{loc}{Object of class 'locus' to use for plot. See \code{\link[=locus]{locus()}}.}

\item{filter_gene_name}{Vector of gene names to display.}

\item{filter_gene_biotype}{Vector of gene biotypes to be filtered. Use
\code{\link[ensembldb:listGenebiotypes]{ensembldb::listGenebiotypes()}} to display possible biotypes. For example,
\code{ensembldb::listGenebiotypes(EnsDb.Hsapiens.v75)}}

\item{xlab}{x axis title.}

\item{cex}{Specifies size for points.}

\item{cex.axis}{Specifies font size for axis numbering.}

\item{cex.lab}{Specifies font size for axis titles.}

\item{cex.text}{Font size for gene text.}

\item{use_layout}{Logical whether \code{graphics::layout} is called. Default
\code{TRUE} is for a standard single plot. Set to \code{FALSE} if a more complex
layout with multiple plots is required e.g. using \code{\link[=multi_layout]{multi_layout()}}.}

\item{heights}{Ratio of top to bottom plot. See \link[graphics:layout]{graphics::layout}.}

\item{showExons}{Logical whether to show exons or simply show whole gene as a
rectangle}

\item{maxrows}{Specifies maximum number of rows to display in gene
annotation panel.}

\item{xticks}{Character value of either 'top' or 'bottom' specifying
whether x axis ticks and numbers are plotted on top or bottom plot window.}

\item{border}{Logical whether a bounding box is plotted around upper and
lower plots.}

\item{gene_col}{Colour for gene lines.}

\item{exon_col}{Fill colour for exons.}

\item{exon_border}{Border line colour outlining exons (or genes if
\code{showExons} is \code{FALSE}). Set to \code{NA} for no border.}

\item{text_pos}{Character value of either 'top' or 'left' specifying
placement of gene name labels.}

\item{italics}{Logical whether gene text is in italics.}

\item{highlight}{Vector of genes to highlight.}

\item{highlight_col}{Single colour or vector of colours for highlighted
genes.}

\item{prioritise}{Vector of genes to be placed first in the gene tracks.}

\item{blanks}{Controls handling of genes with blank names: \code{"fill"} replaces
blank gene symbols with ensembl gene ids. \code{"hide"} hides genes which are
missing gene symbols.}

\item{recomb_col}{Colour for recombination rate line if recombination rate
data is present. Set to \code{NA} to hide the line. See \code{\link[=link_recomb]{link_recomb()}} to add
recombination rate data.}

\item{...}{Other arguments passed to \code{\link[=scatter_plot]{scatter_plot()}} e.g. \code{index_snp},
\code{pcutoff}, \code{scheme}, \code{recomb_offset}, etc, and arguments for \code{\link[=plot]{plot()}} e.g.
\code{ylab}, \code{main}, etc to control the scatter plot.}
}
\value{
No return value.
}
\description{
Genomic locus plot similar to locuszoom.
}
\details{
This is an R version of locuszoom for generating publication ready Manhattan
plots of gene loci. It references Ensembl databases for annotating genes
and exons. Use \code{\link[=locus]{locus()}} first to generate an object of class 'locus' for
plotting. LDlink web server can be queried using function \code{\link[=link_LD]{link_LD()}} to
retrieve linkage disequilibrium (LD) information on the index SNP.

Arguments to control plotting of the gene tracks are passed onto
\code{\link[=genetracks]{genetracks()}} and for the scatter plot are passed via \code{...} to
\code{\link[=scatter_plot]{scatter_plot()}}. See the documentation for each of these functions for
details.
}
\examples{
if(require(EnsDb.Hsapiens.v75)) {
data(SLE_gwas_sub)
loc <- locus(SLE_gwas_sub, gene = 'UBE2L3', flank = 1e5,
             ens_db = "EnsDb.Hsapiens.v75")
locus_plot(loc)

## Use embedded LD information in column `r2`
loc2 <- locus(SLE_gwas_sub, gene = 'IRF5', flank = c(7e4, 2e5), LD = "r2",
              ens_db = "EnsDb.Hsapiens.v75")
## Add label for index SNP
locus_plot(loc2, labels = "index")
}
}
\seealso{
\code{\link[=locus]{locus()}} \code{\link[=scatter_plot]{scatter_plot()}} \code{\link[=genetracks]{genetracks()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quick_peak.R
\name{quick_peak}
\alias{quick_peak}
\title{Fast peak finder in GWAS data}
\usage{
quick_peak(
  data,
  npeaks = NA,
  p_cutoff = 5e-08,
  span = 1e+06,
  min_points = 2,
  chrom = NULL,
  pos = NULL,
  p = NULL
)
}
\arguments{
\item{data}{GWAS dataset (data.frame or data.table)}

\item{npeaks}{Number of peaks to find. If set to \code{NA}, algorithm finds all
distinct peaks separated from one another by region size specified by
\code{span}.}

\item{p_cutoff}{Specifies cut-off for p-value significance above which
p-values are ignored.}

\item{span}{Minimum genomic distance between peaks (default 1 Mb)}

\item{min_points}{Minimum number of p-value significant points which must lie
within the span of a peak. This removes peaks with single or only a few low
p-value SNPs. To disable set \code{min_points} to 1 or less.}

\item{chrom}{Determines which column in \code{data} contains chromosome
information. If \code{NULL} tries to autodetect the column.}

\item{pos}{Determines which column in \code{data} contains position information.
If \code{NULL} tries to autodetect the column.}

\item{p}{Determines which column in \code{data} contains SNP p-values. If \code{NULL}
tries to autodetect the column.}
}
\value{
Vector of row indices
}
\description{
Simple but fast function for finding peaks in genome-wide association study
(GWAS) data based on setting a minimum distance between peaks.
}
\details{
This function is designed for speed. SNP p-values are filtered to only those
which are significant as specified by \code{p_cutoff}. Each peak is identified as
the SNP with the lowest p-value and then SNPs in proximity to each peak
within the distance specified by \code{span} are removed. Regions such as the HLA
whose peaks may well be broader than \code{span} may produce multiple entries.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eqtl_plot.R
\name{eqtl_plot}
\alias{eqtl_plot}
\title{Locus eQTL plot}
\usage{
eqtl_plot(
  loc,
  tissue = "Whole Blood",
  eqtl_gene = loc$gene,
  scheme = "RdYlBu",
  col = NA,
  pcutoff = NULL,
  xlab = NULL,
  ylab = expression("-log"[10] ~ "P"),
  cex.axis = 0.9,
  xticks = TRUE,
  border = FALSE,
  add = FALSE,
  align = TRUE,
  legend_pos = "topright",
  ...
)
}
\arguments{
\item{loc}{Object of class 'locus' to use for plot. See \link{locus}.}

\item{tissue}{GTex tissue in which eQTL has been measured}

\item{eqtl_gene}{Gene showing eQTL effect}

\item{scheme}{Character string specifying palette for effect size showing
up/downregulation eQTL using \link[grDevices:hcl.colors]{grDevices::hcl.colors}. Alternatively a
vector of 6 colours.}

\item{col}{Outline point colour. \code{NA} for no outlines.}

\item{pcutoff}{Cut-off for p value significance. Defaults to p = 5e-08. Set
to \code{NULL} to disable.}

\item{xlab}{x axis title.}

\item{ylab}{y axis title.}

\item{cex.axis}{Specifies font size for axis numbering.}

\item{xticks}{Logical whether x axis numbers and axis title are plotted.}

\item{border}{Logical whether a bounding box is plotted around upper and
lower plots.}

\item{add}{Logical whether to add points to an existing plot or generate a
new plot.}

\item{align}{Logical whether set \code{\link[=par]{par()}} to align the plot.}

\item{legend_pos}{Character value specifying legend position. See \code{\link[=legend]{legend()}}.}

\item{...}{Other arguments passed to \code{\link[=plot]{plot()}} for the scatter plot.}
}
\value{
No return value. Produces a scatter plot using base graphics.
}
\description{
Produces a plot of eQTL data embedded in a 'locus' class object. Intended for
use with \code{\link[=set_layers]{set_layers()}}.
}
\seealso{
\code{\link[=locus]{locus()}} \code{\link[=set_layers]{set_layers()}} \code{\link[=scatter_plot]{scatter_plot()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/link_recomb.R
\name{link_recomb}
\alias{link_recomb}
\title{Query UCSC for Recombination data}
\usage{
link_recomb(loc, genome = loc$genome, table = NULL, recomb = NULL)
}
\arguments{
\item{loc}{Object of class 'locus' generated by \code{\link[=locus]{locus()}}}

\item{genome}{Either \code{"hg38"} or \code{"hg19"}}

\item{table}{Optional character value specifying which recombination table to
use.}

\item{recomb}{Optional \code{GRanges} class object of recombination data.}
}
\value{
A list object of class 'locus'. Recombination data is added as list
element \code{recomb}.
}
\description{
Adds recombination data to a 'locus' object by querying UCSC genome browser.
}
\details{
Uses the \code{rtracklayer} package to query UCSC genome browser for recombination
rate data.

Possible options for \code{table} for hg19 are \code{"hapMapRelease24YRIRecombMap"},
\code{"hapMapRelease24CEURecombMap"}, \code{"hapMapRelease24CombinedRecombMap"} (the
default). The only option for \code{table} for hg38 is \code{"recomb1000GAvg"} (the
default).

If you are doing many queries, it may be much faster to download the entire
recombination track data (around 30 MB for hg38) from the Recombination Rate
Tracks page at
\href{https://genome.ucsc.edu/cgi-bin/hgTrackUi?g=recombRate2}{UCSC genome browser}.
The link to the hg38 download folder is
\url{http://hgdownload.soe.ucsc.edu/gbdb/hg38/recombRate/} and for hg19 is
\url{http://hgdownload.soe.ucsc.edu/gbdb/hg19/decode/}. These .bw files can be
converted to useable \code{GRanges} objects using \code{rtracklayer::import.bw()} (see
the vignette).

Sometimes \code{rtracklayer} generates intermittent API errors or warnings: try
calling \code{link_recomb()} again. If warnings persist restart your R session.
Errors are handled gracefully using \code{try()} to allow users to wrap
\code{link_recomb()} in a loop without quitting halfway. Error messages are still
shown. Successful API calls are cached using \code{memoise} to reduce API
requests.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/locus_plotly.R
\name{locus_plotly}
\alias{locus_plotly}
\title{Locus plotly}
\usage{
locus_plotly(
  loc,
  heights = c(0.6, 0.4),
  filter_gene_name = NULL,
  filter_gene_biotype = NULL,
  cex.text = 0.7,
  italics = FALSE,
  gene_col = ifelse(showExons, "blue4", "skyblue"),
  exon_col = "blue4",
  exon_border = "blue4",
  showExons = TRUE,
  maxrows = 8,
  width = 600,
  xlab = NULL,
  prioritise = NULL,
  blanks = "show",
  ...
)
}
\arguments{
\item{loc}{Object of class 'locus' to use for plot. See \code{\link[=locus]{locus()}}.}

\item{heights}{Vector controlling relative height of each panel on 0-1 scale.
Alternatively a vector of length 2 of height in pixels passed to
\code{scatter_plotly()} and \code{genetrack_ly()}.}

\item{filter_gene_name}{Vector of gene names to display.}

\item{filter_gene_biotype}{Vector of gene biotypes to be filtered. Use
\code{\link[ensembldb:listGenebiotypes]{ensembldb::listGenebiotypes()}} to display possible biotypes. For example,
\code{ensembldb::listGenebiotypes(EnsDb.Hsapiens.v75)}}

\item{cex.text}{Font size for gene text.}

\item{italics}{Logical whether gene text is in italics.}

\item{gene_col}{Colour for gene lines.}

\item{exon_col}{Fill colour for exons.}

\item{exon_border}{Border line colour outlining exons (or genes if
\code{showExons} is \code{FALSE}). Set to \code{NA} for no border.}

\item{showExons}{Logical whether to show exons or simply show whole gene as a
rectangle. If \code{showExons = FALSE} colours are specified by \code{exon_border}
for rectangle border and \code{gene_col} for the fill colour.}

\item{maxrows}{Specifies maximum number of rows to display in gene
annotation panel.}

\item{width}{Width of plotly plot in pixels which is purely used to prevent
overlapping text for gene names.}

\item{xlab}{Title for x axis. Defaults to chromosome \code{seqname} specified
in \code{locus}.}

\item{prioritise}{Vector of genes to be placed first in the gene tracks.}

\item{blanks}{Controls handling of genes with blank names: \code{"fill"} replaces
blank gene symbols with ensembl gene ids. \code{"hide"} completely hides genes
which are missing gene symbols. \code{"show"} shows gene lines but no label
(hovertext is still available).}

\item{...}{Optional arguments passed to \code{\link[=scatter_plotly]{scatter_plotly()}} to control the
scatter plot.}
}
\value{
A 'plotly' plotting object showing a scatter plot above gene tracks.
}
\description{
Genomic locus plot similar to locuszoom, using plotly.
}
\details{
This is an R/plotly version of locuszoom for exploring regional Manhattan
plots of gene loci. Use \code{\link[=locus]{locus()}} first to generate an object of class
'locus' for plotting. This references a selected Ensembl database for
annotating genes and exons. Hover over the points or gene tracks to reveal
more information.
}
\examples{
if(require(EnsDb.Hsapiens.v75)) {
data(SLE_gwas_sub)
loc <- locus(SLE_gwas_sub, gene = "IRF5", flank = c(7e4, 2e5), LD = "r2",
             ens_db = "EnsDb.Hsapiens.v75")
locus_plotly(loc)
}
}
\seealso{
\code{\link[=locus]{locus()}} \code{\link[=genetrack_ly]{genetrack_ly()}} \code{\link[=scatter_plotly]{scatter_plotly()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_layers.R
\name{set_layers}
\alias{set_layers}
\title{Set up a column of multiple plots}
\usage{
set_layers(n = 1, heights = c(rep(3, n), 2), rev = FALSE)
}
\arguments{
\item{n}{Number of plots (not including gene tracks on bottom)}

\item{heights}{Vector of length \code{nrow + 1} specifying height for plots with
a gene track on the bottom}

\item{rev}{Logical whether to reverse plotting order and plot from bottom to
top}
}
\value{
Sets \code{\link[graphics:layout]{graphics::layout()}} to enable multiple plots aligned in a column. The
gene track is assumed to be positioned on the bottom. Returns \code{par()}
invisibly so that layout can be reset to default at the end of plotting.
}
\description{
Uses \code{\link[graphics:layout]{graphics::layout()}} to set up multiple locus plots aligned in a column.
}
\seealso{
\code{\link[graphics:layout]{graphics::layout()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/locus.R
\name{locus}
\alias{locus}
\title{Create locus object for plotting}
\usage{
locus(
  data = NULL,
  gene = NULL,
  xrange = NULL,
  seqname = NULL,
  flank = NULL,
  fix_window = NULL,
  ens_db,
  chrom = NULL,
  pos = NULL,
  p = NULL,
  yvar = NULL,
  labs = NULL,
  index_snp = NULL,
  LD = NULL,
  std_filter = TRUE
)
}
\arguments{
\item{data}{Dataset (data.frame or data.table) to use for plot. We recommend
that tibbles are converted to a normal data.frame. If unspecified or
\code{NULL}, gene track information alone is returned.}

\item{gene}{Optional character value specifying which gene to view. Either
\code{gene}, or \code{xrange} plus \code{seqname}, or \code{index_snp} must be specified.}

\item{xrange}{Optional vector of genomic position range for the x axis.}

\item{seqname}{Optional, specifies which chromosome to plot.}

\item{flank}{Single value or vector with 2 values for how much flanking
region left and right of the gene to show. Defaults to 100kb.}

\item{fix_window}{Optional alternative to \code{flank}, which allows users to
specify a fixed genomic window centred on the specified gene. Both \code{flank}
and \code{fix_window} cannot be specified simultaneously.}

\item{ens_db}{Either a character string which specifies which Ensembl
database package (version 86 and earlier for Homo sapiens) to query for
gene and exon positions (see \code{ensembldb} Bioconductor package). Or an
\code{ensembldb} object which can be obtained from the AnnotationHub database.
See the vignette and the \code{AnnotationHub} Bioconductor package for how to
create this object.}

\item{chrom}{Determines which column in \code{data} contains chromosome
information. If \code{NULL} tries to autodetect the column.}

\item{pos}{Determines which column in \code{data} contains position information.
If \code{NULL} tries to autodetect the column.}

\item{p}{Determines which column in \code{data} contains SNP p-values.
If \code{NULL} tries to autodetect the column.}

\item{yvar}{Specifies column in \code{data} for plotting on the y axis as an
alternative to specifying p-values. Both \code{p} and \code{yvar} cannot be specified
simultaneously.}

\item{labs}{Determines which column in \code{data} contains SNP rs IDs.
If \code{NULL} tries to autodetect the column.}

\item{index_snp}{Specifies the index SNP. If not specified, the SNP with the
lowest P value is selected. Can be used to specify locus region instead of
specifying \code{gene}, or \code{seqname} and \code{xrange}.}

\item{LD}{Optional character value to specify which column in \code{data} contains
LD information.}

\item{std_filter}{Logical, whether standard filters on chromosomes 1-22, X &
Y, and filtering of genes to only those whose transcript ids start with
"ENS" are applied. For users with novel genome assemblies, this probably
needs to be set to \code{FALSE}.}
}
\value{
Returns a list object of class 'locus' ready for plotting,
containing:
\item{seqname}{chromosome value}
\item{xrange}{vector of genomic position range}
\item{gene}{gene name}
\item{ens_db}{Ensembl or AnnotationHub database}
\item{ens_version}{Ensembl database version}
\item{organism}{Ensembl database organism}
\item{genome}{Ensembl data genome build}
\item{chrom}{column name in \code{data} containing chromosome information}
\item{pos}{column name in \code{data} containing position}
\item{p}{column name in \code{data} containing p-value}
\item{yvar}{column name in \code{data} to be plotted on y axis as alternative to
\code{p}}
\item{labs}{column name in \code{data} containing SNP IDs}
\item{index_snp}{id of the most significant SNP}
\item{data}{the subset of GWAS data to be plotted}
\item{TX}{dataframe of transcript annotations}
\item{EX}{\code{GRanges} object of exon annotations}
If \code{data} is \code{NULL} when \code{locus()} is called then gene track information
alone is returned.
}
\description{
Creates object of class 'locus' for genomic locus plot similar to
\code{locuszoom}.
}
\details{
This is an R version of \code{locuszoom} (http://locuszoom.org) for generating
publication ready Manhattan plots of gene loci. It references Ensembl
databases using the \code{ensembldb} Bioconductor package framework for annotating
genes and exons in the locus.
}
\examples{
## Bioconductor package EnsDb.Hsapiens.v75 is needed for these examples
if(require(EnsDb.Hsapiens.v75)) {
data(SLE_gwas_sub)
loc <- locus(SLE_gwas_sub, gene = 'UBE2L3', flank = 1e5,
             ens_db = "EnsDb.Hsapiens.v75")
summary(loc)
locus_plot(loc)
loc2 <- locus(SLE_gwas_sub, gene = 'STAT4', flank = 1e5,
              ens_db = "EnsDb.Hsapiens.v75")
locus_plot(loc2)
}
}
\seealso{
\code{\link[=locus_plot]{locus_plot()}} \code{\link[=locus_ggplot]{locus_ggplot()}} \code{\link[=locus_plotly]{locus_plotly()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gg_addgenes.R
\name{gg_addgenes}
\alias{gg_addgenes}
\title{Add gene tracks to a ggplot2 plot}
\usage{
gg_addgenes(p, loc, heights = c(3, 2), ...)
}
\arguments{
\item{p}{ggplot2 plot object. This can be generated by \code{\link[=gg_scatter]{gg_scatter()}} and
then modified.}

\item{loc}{Object of class 'locus' to use for plot. See \code{\link[=locus]{locus()}}.}

\item{heights}{Vector specifying ratio of heights of upper plot and lower
gene track.}

\item{...}{Additional arguments passed to \code{\link[=gg_genetracks]{gg_genetracks()}} to control
colours of gene tracks etc.}
}
\value{
A ggplot2 plotting object.
}
\description{
Adds gene tracks to an existing ggplot2 plot.
}
\examples{
if(require(EnsDb.Hsapiens.v75)) {
data(SLE_gwas_sub)
loc <- locus(SLE_gwas_sub, gene = 'IRF5', flank = c(7e4, 2e5), LD = "r2",
             ens_db = "EnsDb.Hsapiens.v75")
p <- gg_scatter(loc)
gg_addgenes(p, loc)
}
}
\seealso{
\code{\link[=gg_scatter]{gg_scatter()}} \code{\link[=gg_genetracks]{gg_genetracks()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genetrack_ly.R
\name{genetrack_ly}
\alias{genetrack_ly}
\title{Gene tracks using 'plotly'}
\usage{
genetrack_ly(
  locus,
  filter_gene_name = NULL,
  filter_gene_biotype = NULL,
  cex.text = 0.7,
  italics = FALSE,
  gene_col = ifelse(showExons, "blue4", "skyblue"),
  exon_col = "blue4",
  exon_border = "blue4",
  showExons = TRUE,
  maxrows = 8,
  width = 600,
  xlab = NULL,
  prioritise = NULL,
  blanks = c("fill", "hide", "show"),
  height = NULL,
  plot = TRUE
)
}
\arguments{
\item{locus}{Object of class 'locus' generated by \code{\link[=locus]{locus()}}.}

\item{filter_gene_name}{Vector of gene names to display.}

\item{filter_gene_biotype}{Vector of gene biotypes to be filtered. Use
\code{\link[ensembldb:listGenebiotypes]{ensembldb::listGenebiotypes()}} to display possible biotypes. For example,
\code{ensembldb::listGenebiotypes(EnsDb.Hsapiens.v75)}}

\item{cex.text}{Font size for gene text.}

\item{italics}{Logical whether gene text is in italics.}

\item{gene_col}{Colour for gene lines.}

\item{exon_col}{Fill colour for exons.}

\item{exon_border}{Border line colour outlining exons (or genes if
\code{showExons} is \code{FALSE}). Set to \code{NA} for no border.}

\item{showExons}{Logical whether to show exons or simply show whole gene as a
rectangle. If \code{showExons = FALSE} colours are specified by \code{exon_border}
for rectangle border and \code{gene_col} for the fill colour.}

\item{maxrows}{Specifies maximum number of rows to display in gene
annotation panel.}

\item{width}{Width of plotly plot in pixels which is purely used to prevent
overlapping text for gene names.}

\item{xlab}{Title for x axis. Defaults to chromosome \code{seqname} specified
in \code{locus}.}

\item{prioritise}{Vector of genes to be placed first in the gene tracks.}

\item{blanks}{Controls handling of genes with blank names: \code{"fill"} replaces
blank gene symbols with ensembl gene ids. \code{"hide"} completely hides genes
which are missing gene symbols. \code{"show"} shows gene lines but no label
(hovertext is still available).}

\item{height}{Height in pixels (optional, defaults to automatic sizing).}

\item{plot}{Logical whether to produce plotly object or return plot
coordinates.}
}
\value{
Either a 'plotly' plotting object showing gene tracks, or if
\code{plot = FALSE} a list containing \code{TX}, a dataframe of coordinates for
gene transcripts, and \code{EX}, a dataframe of coordinates for exons.
}
\description{
Plot gene annotation tracks from \code{ensembldb} data using \code{plotly}.
}
\details{
This function can used to plot gene annotation tracks on their own.
}
\examples{
if(require(EnsDb.Hsapiens.v75)) {
data(SLE_gwas_sub)
loc <- locus(SLE_gwas_sub, gene = 'UBE2L3', flank = 1e5,
             ens_db = "EnsDb.Hsapiens.v75")
genetrack_ly(loc)
}
}
